VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Server"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
    Dim tmpOldDirectory As String
    Dim tmpOldFile As String
        
'// Port server listens on
Private Port As Long

'// Allows server admin to set max connections,
'// must be MAX_CONNECTIONS <= the server constant
Private MaxClients As Integer

'// Specifies the packet size (in bytes) to send data
'// Shouldn't really be over 4096 bytes (4K)
Private TransferBufferSize As Long

'// Every time a new clients connectes ClientCounter is
'// incremented by 1, that new number becomes the new
'// clients unique ID number.
Private ClientCounter As Long

'// Every time a new client connects ConnectedClients is
'// incremented by 1, every time a client disconnects,
'// the number is decremented by 1.
Private ConnectedClients As Long

'// This gets set to True when the server has been started
'// and gets set back to False when it is shutdown.
'// It should be False by default.
Private ServerActive As Boolean

'// ''''''''''''''''''''''''''''''''''''''''''
'// Server types
'// ''''''''''''''''''''''''''''''''''''''''''

'// Client actions
Private Enum ClientStatus

    '// Client status constants
    stat_IDLE = 0
    stat_LOGGING_IN = 1
    stat_GETTING_DIR_LIST = 2
    stat_UPLOADING = 3
    stat_DOWNLOADING = 4

End Enum

Private Enum ConnectModes
    
    '// Connection mode constants
    cMode_NORMAL = 0
    cMode_PASV = 1

End Enum

Private Type FtpClient

    InUse As Boolean                'Identifies if this slot is being used.
    GroupName As String             'Group name of the user.
    Flags As String                 'Flags for the user.
    Logins As String                'Number of logins for the user.
    Ratio As String                 'Ratio for the user.
    Id As Long                      'Unique number to identify a client.
    UserName As String              'User name client is is logged in as.
    IPAddress As String             'IP address of the client.
    DataPort As Long                'Port number open on the client for the server to connect to.
    ConnectedAt As String           'Time the client first connected.
    IdleSince As String             'Last recorded time the client sent a command to the server.
    TotalBytesUploaded As Long      'Total bytes uploaded by client from the current session.
    TotalBytesDownloaded As Long    'Total bytes downloaded by client from the current session.
    TotalFilesUploaded As Long      'Total files uploaded by client from the current session.
    TotalFilesDownloaded As Long    'Total files downloaded by client from the current session.
    CurrentFile As String           'Current file being transfer, if any.
    cFileTotalBytes As Long         'Total number of bytes of the file being transfered.
    cTotalBytesXfer As Long         'Total bytes of the current file that has been transfered.
    fFile As Long                   'Reference number to an open file on the server, if any.
    ConnectMode As ConnectModes     'If the client uses PASV mode or not.
    HomeDir As String               'Initial directory client starts in when they first connect.
    CurrentDir As String            'Current directory.
    Status As ClientStatus          'What the client is currently doing.
    OnlyFile As String
    HomeDir2 As String
    MountDir As Boolean

End Type

'// ''''''''''''''''''''''''''''''''''''''''''
'// Server constants
'// ''''''''''''''''''''''''''''''''''''''''''

'// 900 seconds (15 mins.) of idle time will disconnect the client
'// without regard to specific user settings
Private Const MAX_IDLE_TIME = 900

'// 500 simultaneous connections for the server
Private Const MAX_CONNECTIONS = 500

'Array that holds client information for every client.
Private Client(MAX_CONNECTIONS) As FtpClient

'''''''''''''''''''''''''''''''''''''''''''
'Server events
'''''''''''''''''''''''''''''''''''''''''''
'ServerErrorOccurred() event fires when an error occurs
'in a sub/function that wasnt called outside of the object.
Public Event ServerErrorOccurred(ByVal errNumber As Long)

'ServerStarted() event fires after successful startup of Winsock
'and the specified port is successfully opened, server is ready to accept clients.
Public Event ServerStarted()

'ServerStopped() event fires after all connected clients
'have been disconnected, Winsock is shutdown and other
'misc. variables are reset.
Public Event ServerStopped()

'NewClient() event fires when a new connection is accepted.
Public Event NewClient(ByVal ClientID As Long)

'ClientLoggedIn() event fires when a connected client
'successfully logs in with a correct user name and password.
Public Event ClientLoggedIn(ByVal ClientID As Long)

'ClientLoggedOut() event fires when a client
'disconnects or is disconnected by the server.
'When this event fires, the information on that
'client will no longer be available.
Public Event ClientLoggedOut(ByVal ClientID As Long)

'ClientSentCommand() event fires when a client
'sends a FTP command to the server.
Public Event ClientSentCommand(ByVal ClientID As Long, Command As String, Args As String)

'ClientStatusChanged() event fires when a clients
'status has changed.  This event will most likely
'follow after another.  eg. client starts a file
'transfer...
Public Event ClientStatusChanged(ByVal ClientID As Long)

'These functions are for generating a list of files and directories.
Private Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FindNextFile Lib "kernel32" Alias "FindNextFileA" (ByVal hFindFile As Long, lpFindFileData As WIN32_FIND_DATA) As Long
Private Declare Function FileTimeToSystemTime Lib "kernel32" (lpFileTime As FileTime, lpSystemTime As SYSTEMTIME) As Long

Private Type FileTime
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Private Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long

Private Const MAX_PATH = 260

Private Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime As FileTime
    ftLastAccessTime As FileTime
    ftLastWriteTime As FileTime
    nFileSizeHigh As Long
    nFileSizeLow As Long
    dwReserved0 As Long
    dwReserved1 As Long
    cFileName As String * MAX_PATH
    cAlternate As String * 14
End Type

Private Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Long
End Type

'// ''''''''''''''''''''''''''''''''''''''''''
'// Server public properties
'// ''''''''''''''''''''''''''''''''''''''''''

Public Property Get ListeningPort() As Long

    ListeningPort = Port

End Property

Public Property Let ListeningPort(NewPort As Long)
    
    If Port = 0 Then
        Port = NewPort
    End If

End Property

Public Property Get ServerMaxClients() As Integer
    
    ServerMaxClients = MaxClients

End Property

Public Property Let ServerMaxClients(Max As Integer)
    
    If Max >= 0 Then
        MaxClients = Max
    End If

End Property

Public Property Get TransBufferSize() As Long
    
    TransBufferSize = TransferBufferSize

End Property

Public Property Let TransBufferSize(BuffSize As Long)
    
    If BuffSize > 0 Then
        TransferBufferSize = BuffSize
    End If

End Property

Public Property Get CurrentConnectedClients() As Long
    
    CurrentConnectedClients = ConnectedClients

End Property

Public Property Get CurrentClientCounter() As Long
    
    CurrentClientCounter = ClientCounter

End Property

'// Use these properties to get information about a specific client
'// outside of this class module.
Public Property Get GetClientConnectedAt(ClientID As Long) As String
    
    GetClientConnectedAt = Client(GetClientArrayLocByID(ClientID)).ConnectedAt

End Property

Public Property Get GetClientConnectMode(ClientID As Long) As String
    
    GetClientConnectMode = Client(GetClientArrayLocByID(ClientID)).ConnectMode

End Property

Public Property Get GetClientcTotalBytesXfer(ClientID As Long) As Long
    
    GetClientcTotalBytesXfer = Client(GetClientArrayLocByID(ClientID)).cTotalBytesXfer

End Property

Public Property Get GetClientcFileTotalBytes(ClientID As Long) As Long
    
    GetClientcFileTotalBytes = Client(GetClientArrayLocByID(ClientID)).cFileTotalBytes

End Property

Public Property Get GetClientCurrentDir(ClientID As Long) As String
    
    GetClientCurrentDir = Client(GetClientArrayLocByID(ClientID)).CurrentDir

End Property

Public Property Get GetClientCurrentFile(ClientID As Long) As String
    
    GetClientCurrentFile = Client(GetClientArrayLocByID(ClientID)).CurrentFile

End Property

Public Property Get GetClientDataPort(ClientID As Long) As Long
    
    GetClientDataPort = Client(GetClientArrayLocByID(ClientID)).DataPort

End Property

Public Property Get GetClientfFile(ClientID As Long) As Long
    
    GetClientfFile = Client(GetClientArrayLocByID(ClientID)).fFile

End Property

Public Property Get GetClientHomeDir(ClientID As Long) As String
    
    GetClientHomeDir = Client(GetClientArrayLocByID(ClientID)).HomeDir

End Property

Public Property Get GetClientIdleSince(ClientID As Long) As Long
    
    GetClientIdleSince = Client(GetClientArrayLocByID(ClientID)).IdleSince

End Property

Public Property Get GetClientIPAddress(ClientID As Long) As String
    
    GetClientIPAddress = Client(GetClientArrayLocByID(ClientID)).IPAddress

End Property

Public Property Get GetClientStatus(ClientID As Long) As String
    
    GetClientStatus = ServerGetClientStatusDescription(Client(GetClientArrayLocByID(ClientID)).Status)

End Property

Public Property Get GetClientTotalBytesDownloaded(ClientID As Long) As Long
    
    GetClientTotalBytesDownloaded = Client(GetClientArrayLocByID(ClientID)).TotalBytesDownloaded

End Property

Public Property Get GetClientTotalBytesUploaded(ClientID As Long) As Long
    
    GetClientTotalBytesUploaded = Client(GetClientArrayLocByID(ClientID)).TotalBytesUploaded

End Property

Public Property Get GetClientTotalFilesDownloaded(ClientID As Long) As Long
    
    GetClientTotalFilesDownloaded = Client(GetClientArrayLocByID(ClientID)).TotalFilesDownloaded

End Property

Public Property Get GetClientTotalFilesUploaded(ClientID As Long) As Long
    
    GetClientTotalFilesUploaded = Client(GetClientArrayLocByID(ClientID)).TotalFilesUploaded

End Property

Public Property Get GetClientUserName(ClientID As Long) As String
    
    GetClientUserName = Client(GetClientArrayLocByID(ClientID)).UserName

End Property

'// ''''''''''''''''''''''''''''''''''''''''''
'// Server functions
'// ''''''''''''''''''''''''''''''''''''''''''
Public Function StartServer() As Long

    '// Initiate the server.
    Call AddLogItem("Initiating the server...", "Administrator", frmMain.lViewLog)
    frmMain.lblStatus.Caption = "Active"
    frmMain.lblStatus.ForeColor = &H4000&

    '// Make sure you dont start the server again.
    If FTPRunning = True Then
        Call AddLogItem("Server is already running!", "Administrator", frmMain.lViewLog)
        StartServer = 1001
        Exit Function
    End If

    '// Make sure the server has been givin a port before doing anything.
    If FTPPort < 1 Then
        Call AddLogItem("No port given!", "Administrator", frmMain.lViewLog)
        StartServer = 1002
        Exit Function
    End If

    '// If no TransferBufferSize was assigned then default to 4096 (4K)
    If TransferBufferSize < 1 Then TransferBufferSize = 4096

    '// Start listening using the intial instance of the Winsock control.
    With frmWinsock.CommandSock(0)
        .LocalPort = FTPPort
        .Listen
    End With

    ServerActive = True
    
    '// Server started with out error and is ready to accept clients.
    RaiseEvent ServerStarted
    
    '// Add log item.
    Call AddLogItem("Server started at port: " & FTPPort, "Administrator", frmMain.lViewLog)

End Function

Public Sub NewClient(requestID As Long)

    '// Increment counters
    FTPConnUsers = FTPConnUsers + 1
    frmMain.lblUsers.Caption = FTPConnUsers
    frmMain.TrayForm1.ToolTip = "RedFTPd (u:" & FTPConnUsers & "/" & FTPMaxUsers & ")"

    '// This just helps to make sure that things dont get
    '// screwed up incase two clients connected at nearly
    '// the exact same time.
    FTPId = FTPConnUsers

    '// Find an open slot in the client() array.
    Do
        I = I + 1
    Loop Until Client(I).InUse = False

    '// Found a empty client array, use that.
    With Client(I)
        .InUse = True

        '// Create new instances of the Winsock control for the
        '// new client. This is to keep the first instance, CommandSock(0)
        '// listening for more clients.
        Load frmWinsock.CommandSock(I)
        Load frmWinsock.DataSock(I)
    
        '// Accept the new connection with the new instance of the Winsock control.
        frmWinsock.CommandSock(I).Accept requestID

        '// Fill client data
        .ConnectedAt = Now
        .Id = FTPId
        .Status = stat_LOGGING_IN
        .IdleSince = Now
        .IPAddress = frmWinsock.CommandSock(I).RemoteHostIP
    End With

    '// Raise event to tell the outside program that a new client has connected.
    RaiseEvent NewClient(Client(I).Id)

    '// If max clients has been reached then disconnect the client.
    If ((FTPConnUsers > FTPMaxUsers) And (FTPMaxUsers <> 0)) Or (FTPConnUsers > FTPMaxUsers) Then
        SendResponse I, "421 Too many users - try again later."
        LogoutClient , I
        Exit Sub
    End If

    SendResponse I, "220 Ready to log in..."

End Sub
Public Sub SendMessageFile(cArrayLoc As Integer, MessageFile As String, MessageType As String, IncludeEnd As Boolean)

    On Error Resume Next

    '// Declares
    Dim mMessage As Collection
    Dim mMessageLine As String
    Dim mFile As Integer
    Dim mContent As String
    Dim mI As Integer
    
    mFile = FreeFile
    
    '// Read the file, and put the content into an array.
    '// Then send the content.
    Open MessageFile For Input As mFile
        Do
        Line Input #mFile, mContent
        mMessage.Add mContent, mContent
        mMessageLine = mContent
        
        '// Check the message line and eventually change the
        '// reply codes to understandable replies ;)
        
        '// User reply
        mMessageLine = Replace(mMessageLine, "#uU", Client(cArrayLoc).UserName, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uG", Client(cArrayLoc).GroupName, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uD", Client(cArrayLoc).CurrentDir, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uF", Client(cArrayLoc).CurrentFile, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uQ", Client(cArrayLoc).Flags, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uH", Client(cArrayLoc).HomeDir, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uI", Client(cArrayLoc).Id, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uW", Client(cArrayLoc).IdleSince, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uP", Client(cArrayLoc).IPAddress, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uL", Client(cArrayLoc).Logins, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uR", Client(cArrayLoc).Ratio, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uS", Client(cArrayLoc).Status, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#u1", Client(cArrayLoc).TotalBytesDownloaded, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#u2", Client(cArrayLoc).TotalBytesUploaded, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#u3", Client(cArrayLoc).TotalFilesDownloaded, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#u4", Client(cArrayLoc).TotalFilesUploaded, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uB", Client(cArrayLoc).cFileTotalBytes, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uE", Client(cArrayLoc).ConnectedAt, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uM", Client(cArrayLoc).ConnectMode, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uX", Client(cArrayLoc).cTotalBytesXfer, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uZ", Client(cArrayLoc).DataPort, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#uY", Client(cArrayLoc).fFile, 1, , vbTextCompare)
        
        '// Server reply.
        mMessageLine = Replace(mMessageLine, "#sQ", frmMain.lblUsers.Caption, 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#sW", GetFromIni("General", "MaxConnections", App.Path & "\data\settings.conf"), 1, , vbTextCompare)
        mMessageLine = Replace(mMessageLine, "#sS", GetFromIni("SiteInfo", "ShortSiteName", App.Path & "\data\settings.conf"), 1, , vbTextCompare)
        
        '// Check if we have reached the end of the file.
        If EOF(mFile) = True Then
        
            '// Check if we should end this.
            If IncludeEnd = True Then
                Call SendResponse(cArrayLoc, MessageType & "  " & mMessageLine)
            Else
                Call SendResponse(cArrayLoc, MessageType & "- " & mMessageLine)
            End If
            
        Else
            Call SendResponse(cArrayLoc, MessageType & "- " & mMessageLine)
        End If
        
        Loop Until EOF(mFile)
    Close mFile

End Sub


Private Sub SendResponse(sckArrayloc As Integer, Data As String)

    '// Send a command or response to the client, a carriage return and line feed
    '// must be added to the end of the string to singnify the end of this command/response.
    frmWinsock.CommandSock(sckArrayloc).SendData Data & vbCrLf
    DoEvents

End Sub

Private Sub SendData(sckArrayloc As Integer, Data As String)

    '// Send data.
    frmWinsock.DataSock(sckArrayloc).SendData Data

End Sub

Public Sub SendComplete(sckArrayloc As Integer)

On Error Resume Next

    '// After sending data on a data connection to the client
    '// via the DataSock control on frmWinsock, this sub will
    '//run after all the data has been sent.
    
    With Client(sckArrayloc)
        Select Case .Status
            Case stat_GETTING_DIR_LIST
                '// Send complete means the directory list is done sending
                '// close the data connection and alert the client.
                frmWinsock.DataSock(sckArrayloc).Close
                SendMessageFile sckArrayloc, App.Path & "\data\messages\site-chdir.msg", "226", True
    
                '// Change client status
                .Status = stat_IDLE
                RaiseEvent ClientStatusChanged(.Id)
    
            Case stat_DOWNLOADING
                If .cFileTotalBytes = .cTotalBytesXfer Then
                    '// Reset variables used for the file transfer.

                    '// Close the open file.
                    Close #.fFile

                    '// Close the data connection.
                    frmWinsock.DataSock(sckArrayloc).Close
                    .DataPort = 0

                    '// Tell the client the transfer is complete.
                    Call AddLogItem("User Download (" & .CurrentFile & ") complete", .UserName, frmMain.lViewLog)
                    SendResponse sckArrayloc, "226 Transfer complete."
                    
                    '// Remove the credit.
                    Call RemoveCredit(.UserName, FileLen(.CurrentFile), False)
    
                    .cFileTotalBytes = 0
                    .cTotalBytesXfer = 0

                    '// Change client status
                    .Status = stat_IDLE
                    RaiseEvent ClientStatusChanged(.Id)
                Else
                    SendFile sckArrayloc
                End If
        End Select
    End With

End Sub

Private Sub LoginClient(cArrayLoc As Integer, Password As String)

    '// Get some essential user settings we will use later.
    Dim tmpAllowAnonymous As Boolean
    tmpAllowAnonymous = False
    
    If GetFromIni("General", "AllowAnonConnections", App.Path & "\data\settings.conf") = True Then tmpAllowAnonymous = True
    
    '// Check if the user who logs in is using anonymous.
    If UCase(Client(cArrayLoc).UserName) = UCase("anonymous") Then
        If tmpAllowAnonymous = True Then
            GoTo LogInUser
            Exit Sub
        Else
            GoTo LogOutUser
            Exit Sub
        End If
    Else
    End If
    
    '// Check if the user tries to use the default user file.
    '// If so just log the user out immediately.
    If Client(cArrayLoc).UserName = "defaultuser" Then
        Call AddLogItem("User tried to log in using the default file!", "Default", frmMain.lViewLog)
        SendResponse cArrayLoc, "550 Not logged in - user '" & Client(cArrayLoc).UserName & "' does not exist."
        LogoutClient cArrayLoc
        Exit Sub
    Else
    End If
    
    '// Check if the user actually exist in the user files.
    If CheckUser(Client(cArrayLoc).UserName) = False Then
        Call AddLogItem("User tried to log in, but account does not exist!", Client(cArrayLoc).UserName, frmMain.lViewLog)
        SendResponse cArrayLoc, "530 Not logged in - user " & Client(cArrayLoc).UserName & " does not exist."
        LogoutClient cArrayLoc
        Exit Sub
    Else
    End If
    
    '// Check if user is already connected.
    If ConnCheckUser(Client(cArrayLoc).UserName, frmMain.tvConnections) = True Then
        Call AddLogItem("User tried to log in, but is already logged in!", Client(cArrayLoc).UserName, frmMain.lViewLog)
        SendResponse cArrayLoc, "530 Not logged in - user " & Client(cArrayLoc).UserName & " is already logged in."
        FTPRemoveUser = False
        LogoutClient cArrayLoc
        Exit Sub
    Else
    End If
      
    '// Check if the username and password the user supplied matches
    '// with your user file. (does not apply to the anonymous users, which
    '// you see are either allowed, or disallowed above.)
    If Password = GetFromIni(Client(cArrayLoc).UserName, "password", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr") Then

        '// Check if the user have been marked as deleted.
        If InStr(1, GetFromIni(Client(cArrayLoc).UserName, "flags", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr"), "6", vbTextCompare) = "0" Then
            GoTo LogInUser
            Exit Sub
        Else
            '// User have been marked as deleted.
            Call AddLogItem("User tried to log in, but account have been marked as deleted!", Client(cArrayLoc).UserName, frmMain.lViewLog)
            SendResponse cArrayLoc, "530 Not Logged in - you have been deleted."
            LogoutClient cArrayLoc
            Exit Sub
        End If

    Else
        GoTo LogOutUser
        Exit Sub
    End If
    
LogOutUser:
    '// Login not allowed.
    Call AddLogItem("User tried to log in, but was rejected!", Client(cArrayLoc).UserName, frmMain.lViewLog)
    SendResponse cArrayLoc, "530 Not Logged in, user " & Client(cArrayLoc).UserName & " are not allowed to log in."
    LogoutClient cArrayLoc
    Exit Sub

LogInUser:
    '// Fill in user data
    With Client(cArrayLoc)
        .HomeDir = GetFromIni(Client(cArrayLoc).UserName, "HomeDirectory", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr")
        .GroupName = GetFromIni(Client(cArrayLoc).UserName, "Group", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr")
        .Flags = GetFromIni(Client(cArrayLoc).UserName, "Flags", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr")
        .Logins = GetFromIni(Client(cArrayLoc).UserName, "MaxLogins", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr")
        .Ratio = GetFromIni(Client(cArrayLoc).UserName, "Ratio", App.Path & "\data\users\" & Client(cArrayLoc).UserName & ".usr")
        .CurrentDir = .HomeDir
        .HomeDir2 = .HomeDir
        .Status = stat_IDLE
        .MountDir = False
        Call ConnUpdateUser(.UserName)
        Call SendMessageFile(cArrayLoc, App.Path & "\data\messages\site-welcome.msg", "230", True)
    End With
    
    '// Add the user to the userlist.
    Call ConnAddUser(Client(cArrayLoc).UserName, frmMain.tvConnections)
    Call AddLogItem("User logged in.", Client(cArrayLoc).UserName, frmMain.lViewLog)

    RaiseEvent ClientLoggedIn(ByVal Client(cArrayLoc).Id)
    RaiseEvent ClientStatusChanged(ByVal Client(cArrayLoc).Id)

End Sub

Public Sub LogoutClient(Optional ByVal Id As Long, Optional cArrayLoc As Integer)

    '// Either pass ID or cArrayLoc of the client to log them out.
    '// This Sub will disconnect the client,
    '// and free system resources used by this client.

    On Error Resume Next

    '// Exit if the ID/Arrayloc is 0 (can't happen.)
    If Id = 0 And cArrayLoc = 0 Then Exit Sub
    
    '// Div. Declares
    Dim ArrayPos As Integer
    Dim tmp As Long
    Dim I As Integer
    
    '// Get ID
    If Id = 0 Then
        ArrayPos = cArrayLoc
    Else
        ArrayPos = GetClientArrayLocByID(Id)
    End If

    '// More precautionary code.
    If Client(ArrayPos).Id = 0 Then Exit Sub
    If ArrayPos < 1 Then Exit Sub
    
    '// Remove the user from the user list.
    '// First check if we should remove the user.
    If FTPRemoveUser = False Then
    Else
        Call ConnRemUser(Client(ArrayPos).UserName, frmMain.tvConnections)
    End If

    '// Update the log.
    Call AddLogItem("User logged out.", Client(ArrayPos).UserName, frmMain.lViewLog)

    '// Reset the user array.
    With Client(ArrayPos)
        '// Close the connection
        frmWinsock.CommandSock(ArrayPos).Close
        frmWinsock.DataSock(ArrayPos).Close

        Unload frmWinsock.CommandSock(ArrayPos)
        Unload frmWinsock.DataSock(ArrayPos)

        If .fFile <> 0 Then Close #.fFile

        '// Reset all client info for next use.
        .ConnectedAt = ""
        .ConnectMode = 0
        .cTotalBytesXfer = 0
        .cFileTotalBytes = 0
        .CurrentDir = ""
        .CurrentFile = ""
        .DataPort = 0
        .fFile = 0
        .HomeDir = ""
        tmp = .Id
        .Id = 0
        .IdleSince = ""
        .IPAddress = ""
        .Status = stat_IDLE
        .TotalBytesDownloaded = 0
        .TotalBytesUploaded = 0
        .TotalFilesDownloaded = 0
        .TotalFilesUploaded = 0
        .UserName = ""
        .InUse = False
    End With

    '// Decrement the connected users count.
    If FTPConnUsers > 0 Then FTPConnUsers = FTPConnUsers - 1
    frmMain.lblUsers.Caption = FTPConnUsers
    frmMain.TrayForm1.ToolTip = "RedFTPd (u:" & FTPConnUsers & "/" & FTPMaxUsers & ")"

    '// Client is now officially logged out.
    RaiseEvent ClientLoggedOut(ByVal tmp)

    '// Set the default value.
    FTPRemoveUser = True

End Sub

Public Sub DisableUserAccount(UserName As String)

'// Mark a user as deleted.
On Error Resume Next

Dim tmpFlags As String

tmpFlags = GetFromIni(UserName, "flags", App.Path & "\data\users\" & UserName & ".usr")
tmpFlags = tmpFlags & "6"

Call SaveToIni(UserName, "flags", tmpFlags, App.Path & "\data\users\" & UserName & ".usr")
Call AddLogItem("User account: " & UserName & " was disabled.", "Administrator", frmMain.lViewLog)

End Sub
Public Sub KickUser(UserName As String)

'// Kick the selected user.
On Error Resume Next

For I = 0 To FTPConnUsers Step 1
    If Client(I).UserName = UserName Then
        Call SendResponse(I, "550 You have been kicked from the server.")
        Call LogoutClient(I)
        Call AddLogItem("User: " & UserName & " was kicked.", "Administrator", frmMain.lViewLog)
    Else
    End If
Next I

End Sub

Private Function GetClientArrayLocByID(ByVal Id As Long) As Integer

    '// This function returns the array number where the information
    '// of the client of the specified ID is located in the client() array.
    Dim I As Integer

    For I = 0 To UBound(Client)
        If Client(I).Id = Id Then
            GetClientArrayLocByID = I
            Exit Function
        End If
    Next

    '// ID not found in client array, return -1 error
    GetClientArrayLocByID = -1

End Function

Public Sub ProcFTPCommand(ByVal sckArrayloc As Integer, ByRef raw_data As String)

    '// If an error occure, then for now just skip over it.
    On Error Resume Next

    '// Declares used to determine different tokens.
    Dim tmpArg1 As String
    Dim tmpArg2 As String
    Dim tmpArg3 As String
    Dim tmpArg4 As String
    Dim tmpArg5 As String
    Dim tmpArg6 As String
    Dim tmpTotalLinks As String
    Dim mI As Long
    Dim mDisplay As String
    Dim mPath As String
    Dim mCount As Long
    Dim mtmpTotalLinks As String
    
    '// This sub processes FTP commands sent by the client.
    Dim Data
    Dim FTPCommand As String
    Dim FTPArgs As String

    '// Remove carriage return and line feed.
    Data = Replace$(raw_data, vbCrLf, "")
    
    '// Command has no arguments.
    If InStr(Data, " ") = 0 Then
        FTPCommand = Data
    
    '// Get the command and arguments.
    Else
        FTPCommand = Left$(Data, (InStr(Data, " ") - 1))
        FTPArgs = Right$(Data, (Len(Data) - InStr(Data, " ")))
    End If

    RaiseEvent ClientSentCommand(Client(sckArrayloc).Id, FTPCommand, FTPArgs)
    Client(sckArrayloc).IdleSince = Now

    '// Get the different tokens supplied with the site command.
    tmpArg1 = ExtractArgument(1, FTPArgs, " ")
    tmpArg2 = ExtractArgument(2, FTPArgs, " ")
    tmpArg3 = ExtractArgument(3, FTPArgs, " ")
    tmpArg4 = ExtractArgument(4, FTPArgs, " ")
    tmpArg5 = ExtractArgument(5, FTPArgs, " ")
    tmpArg6 = ExtractArgument(6, FTPArgs, " ")

    '// Spesified SITE commands.
    '//////////////////////////////////////////////////////////////////////////////////

    If UCase(Mid(FTPCommand, 1, 4)) = "SITE" Then
        '// Check what commands are supplied. (SITE <cmd>)
        Select Case UCase$(tmpArg1)
        
            '// ADD SITE OP //////////////////////////////////////////////////////////
            Case "ADDSITEOP"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE ADDSITEOP'")
                    Exit Sub
                Else
                End If
        
                '// Check if enough values supplied.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-addsiteop.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Check if the user exist.
                If CheckUser(tmpArg2) = False Then
                    Call SendResponse(sckArrayloc, "550 User '" & tmpArg2 & "' does not exist!")
                    Exit Sub
                Else
                End If
                
                '// Check if the user have the flag.
                If CheckUserFlag(tmpArg2, "1") = True Then
                    Call SendResponse(sckArrayloc, "550 User '" & tmpArg2 & "' is already a SiteOP can't add Flag 1 again.")
                    Exit Sub
                Else
                End If
                
                '// Mark the user as siteop.
                Call AddFlag(tmpArg2, "1")
                Call SendResponse(sckArrayloc, "220 User '" & tmpArg2 & "' have been added as SiteOP (Flag: 1)")
                Call AddLogItem("User '" & tmpArg2 & "' have been added as SiteOP (Flag: 1)", Client(sckArrayloc).UserName, frmMain.lViewLog)
        
            '// DEL SITE OP //////////////////////////////////////////////////////////
            Case "DELSITEOP"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE DELSITEOP'")
                    Exit Sub
                Else
                End If
        
                '// Check if enough values supplied.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-delsiteop.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Check if the user exist.
                If CheckUser(tmpArg2) = False Then
                    Call SendResponse(sckArrayloc, "550 User '" & tmpArg2 & "' does not exist!")
                    Exit Sub
                Else
                End If
                
                '// Check if the user have the flag.
                If CheckUserFlag(tmpArg2, "1") = False Then
                    Call SendResponse(sckArrayloc, "550 User '" & tmpArg2 & "' is not a SiteOP, can't remove Flag 1.")
                    Exit Sub
                Else
                End If
                
                '// Remove the user as siteop.
                Call DelFlag(tmpArg2, "1")
                Call SendResponse(sckArrayloc, "220 User '" & tmpArg2 & "' have been removed as SiteOP (Flag: 1)")
                Call AddLogItem("User '" & tmpArg2 & "' have been removed as SiteOP (Flag: 1)", Client(sckArrayloc).UserName, frmMain.lViewLog)
                
            '// UNDUPE ///////////////////////////////////////////////////////////////
            Case "UNDUPE"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "C") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE UNDUPE'")
                    Exit Sub
                Else
                End If
        
                '// Check if enough values supplied.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-undupe.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Check if the file exist in the dupe database.
                If GetFromIni(tmpArg2, "Info", App.Path & "\data\redftpd.files") = "" Then
                    Call SendResponse(sckArrayloc, "550 The file '" & tmpArg2 & "' does not exist in the dupe database.")
                    Exit Sub
                Else
                End If
                
                '// Remove the file from the dupe database.
                Call SaveToIni(tmpArg2, vbNullString, vbNullString, App.Path & "\data\redftpd.files")
                Call SendResponse(sckArrayloc, "220 File '" & tmpArg2 & "' was successfully removed from DUPE database.")
                Call AddLogItem("File: '" & tmpArg2 & "' was removed from the dupe database.", Client(sckArrayloc).UserName, frmMain.lViewLog)
                        
                
            '// MOUNTS ///////////////////////////////////////////////////////////////
            Case "MOUNTS"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE MOUNTS'")
                    Exit Sub
                Else
                End If
            
                '// Send part 1/3
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-mounts_top.msg", "220", False)
                
                '// Get the current number of links
                mtmpTotalLinks = GetFromIni("General", "TotalLinks", App.Path & "\data\redftpd.link")
                
                For mI = 1 To mtmpTotalLinks Step 1
                
                    Dim tmpCount As String
                    If mI < 9 Then tmpCount = "0" & mI
                    
                    mDisplay = GetFromIni("Link" & mI, "Display", App.Path & "\data\redftpd.link")
                    mPath = GetFromIni("Link" & mI, "Path", App.Path & "\data\redftpd.link")
                    Call SendResponse(sckArrayloc, "220- | " & tmpCount & " " & mDisplay & MakeSpace(mDisplay, 21) & mPath)
                Next mI
                
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-mounts_bottom.msg", "220", True)
        
            '// MOUNT DIRECTORY //////////////////////////////////////////////////////
            Case "MOUNTDIR"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE MOUNTDIR'")
                    Exit Sub
                Else
                End If
                
                '// Check if enough values supplied.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-mountdir.msg", "220", True)
                    Exit Sub
                Else
                End If
        
                '// Check if enough values supplied.
                If tmpArg3 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-mountdir.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Get the current number of links.
                tmpTotalLinks = GetFromIni("General", "TotalLinks", App.Path & "\data\redftpd.link")
                tmpTotalLinks = CDbl(tmpTotalLinks) + CDbl(1)
                
                '// Check if directory exist.
                If Dir(tmpArg3, vbDirectory) = "" Then
                    Call SendResponse(sckArrayloc, "550 The directory '" & tmpArg3 & "' does not exist.")
                    Exit Sub
                Else
                End If
                
                If Right(tmpArg3, 1) <> "\" Then tmpArg3 = tmpArg3 & "\"
        
                Call SaveToIni("General", "TotalLinks", tmpTotalLinks, App.Path & "\data\redftpd.link")
                Call SaveToIni("Link" & tmpTotalLinks, "Path", tmpArg3, App.Path & "\data\redftpd.link")
                Call SaveToIni("Link" & tmpTotalLinks, "Display", tmpArg2, App.Path & "\data\redftpd.link")
                Call SendResponse(sckArrayloc, "220 The new mount have been added: '" & tmpArg2 & "/" & tmpArg3 & "'")
                Call AddLogItem("New mount have been created: " & tmpArg2 & "/" & tmpArg3 & ".", Client(sckArrayloc).UserName, frmMain.lViewLog)
                'GetDirectoryList (sckArrayloc)
        
            '// REMOVE MOUNT /////////////////////////////////////////////////////////
            Case "MOUNTCLEAR"
            
                '// Check if enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE MOUNTCLEAR'")
                    Exit Sub
                Else
                End If
                
                '// Check if enough values supplied.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-mountclear.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Get the current number of links.
                tmpTotalLinks = GetFromIni("General", "TotalLinks", App.Path & "\data\redftpd.link")
                tmpTotalLinks = CDbl(tmpTotalLinks) - CDbl(1)
                
                For mI = 1 To tmpTotalLinks Step 1
                    mDisplay = GetFromIni("Link" & mI, "Display", App.Path & "\data\redftpd.link")
                    
                    If UCase(mDisplay) = UCase(tmpArg2) Then
                        tmpTotalLinks = CDbl(tmpTotalLinks) - CDbl(1)
                        Call SaveToIni("General", "TotalLinks", tmpTotalLinks, App.Path & "\data\redftpd.link")
                        Call SaveToIni("Link" & mI, vbNullString, vbNullString, App.Path & "\data\redftpd.link")
                        Call SendResponse(sckArrayloc, "220 Mount '" & tmpArg2 & "' was successfully removed.")
                        Exit Sub
                    Else
                    End If
                Next mI
                
                Call SendResponse(sckArrayloc, "550 Mount '" & tmpArg2 & "' was not found.")
                'GetDirectoryList (sckArrayloc)
        
            '// NEWEST UPLOADS ///////////////////////////////////////////////////////
            Case "NEW"
            
                '// First send part 1/3 to the user.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-new_top.msg", "220", False)
                
                '// Send the file that holds the new entries.
                Dim mMessageLine As String
                Dim mFile As Integer
                Dim mContent As String
                Dim mOwner As String
                Dim mGroup As String
                Dim mRelease As String
                Dim mDate As String
                Dim mStart As Long
   
                mFile = FreeFile
    
                '// Read the file, and put the content into an array.
                '// Then send the content.
                Open App.Path & "\data\incoming.log" For Input As mFile
                    Do
                    Line Input #mFile, mContent
                    mMessageLine = mContent
                    mOwner = ExtractArgument(1, mMessageLine, ",")
                    mGroup = ExtractArgument(2, mMessageLine, ",")
                    mRelease = ExtractArgument(4, mMessageLine, ",")
                    mRelease = strStrip(mRelease, "\")
                    mStart = Len(mRelease)
                    mRelease = Mid(ExtractArgument(4, mMessageLine, ","), mStart, Len(ExtractArgument(4, mMessageLine, ",")))
                    
                    mDate = ExtractArgument(3, mMessageLine, ",")
                    
                    '// Make sure none of the strings are longer than
                    '// they can be.
                    If Len(mOwner) > 8 Then mOwner = Mid(mOwner, 1, 7) & "+"
                    If Len(mGroup) > 10 Then mGroup = Mid(mGroup, 1, 9) & "+"
                    If Len(mRelease) > 46 Then mRelease = Mid(mRelease, 1, 43) & "..."
                    
                    Call SendResponse(sckArrayloc, "220- | " & mOwner & MakeSpace(mOwner, 9) & mGroup & MakeSpace(mGroup, 10) & mRelease & MakeSpace(mRelease, 46) & mDate & " |")
                    Loop Until EOF(mFile)
                Close mFile
                
                '// Last send part 3/3 to the user.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-new_bottom.msg", "220", True)
            
            '// GROUP ////////////////////////////////////////////////////////////////
            Case "GROUP"
                
                '// Return a users group.
                Dim tmpCheckOwnGroup As String
                tmpCheckOwnGroup = GetFromIni(Client(sckArrayloc).UserName, "group", App.Path & "\data\users\" & Client(sckArrayloc).UserName & ".usr")
                SendResponse sckArrayloc, "220 Your current group is: " & tmpCheckOwnGroup & "."
                Call AddLogItem("User checked what group he belonged to.", Client(sckArrayloc).UserName, frmMain.lViewLog)
        
            '// CHANGE GROUP /////////////////////////////////////////////////////////
            Case "CHGRP"
        
                Dim tmpCheckCHGrpFlag As String
                Dim tmpCHGrp As Boolean
                
                '// Check if the user have enough flags.
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Then
                    tmpCHGrp = True
                Else
                    tmpCHGrp = False
                End If
                
                '// Check if the user is allowed to do that command.
                If tmpCHGrp = False Then
                    Call AddLogItem("User tried to change his group, but was rejected!", Client(sckArrayloc).UserName, frmMain.lViewLog)
                    SendResponse sckArrayloc, "550 You are not allowed to do this command!"
                    Exit Sub
                Else
                End If
                
                '// Check if enough values.
                If tmpArg2 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-chgrp.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Check if enough values.
                If tmpArg3 = "" Then
                    Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-chgrp.msg", "220", True)
                    Exit Sub
                Else
                End If
                
                '// Check if the user exist.
                If CheckUser(tmpArg2) = False Then
                    SendResponse sckArrayloc, "220 The user '" & tmpArg2 & "' does not exist!"
                    Call AddLogItem("User tried to change the group of user: '" & tmpArg2 & "' to '" & tmpArg3 & "'", Client(sckArrayloc).UserName, frmMain.lViewLog)
                    Exit Sub
                Else
                End If
                
                '// Check if the group exist.
                If CheckGroup(tmpArg3) = False Then
                    SendResponse sckArrayloc, "220 The group '" & tmpArg3 & "' does not exist!"
                    Call AddLogItem("User tried to change to group: '" & tmpArg3 & "' but group doesnt exist!", Client(sckArrayloc).UserName, frmMain.lViewLog)
                    Exit Sub
                Else
                End If
                
                '// Change the user's group.
                Dim tmpCHGrpOld As String
                tmpCHGrpOld = GetFromIni(tmpArg2, "Group", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SaveToIni tmpArg2, "Group", tmpArg3, App.Path & "\data\users\" & tmpArg2 & ".usr"
                SendResponse sckArrayloc, "220 Changed user '" & tmpArg2 & "' group from '" & tmpCHGrpOld & "' to '" & tmpArg3 & "'"
                Call AddLogItem("User was updated with new group: '" & tmpArg3 & "'", Client(sckArrayloc).UserName, frmMain.lViewLog)
        
            '// SITE USERS ///////////////////////////////////////////////////////////
            Case "USERS"
            
                Dim tmpCheckUserFlags2 As String
                Dim tmpCheckUser2 As Boolean
    
                '// Check the flag of the user who performs this command.
                If CheckUserFlag(Client(sckArrayloc).UserName, "H") = True Then
                    tmpCheckUser2 = True
                Else
                    tmpCheckUser2 = False
                End If
            
                '// The user is not allowed to view the user list.
                If tmpCheckUser2 = False Then
                    SendResponse sckArrayloc, "550 You are not allowed to do this command!"
                    Exit Sub
                Else
                End If
            
                '// Send part 1/3 of the reply to the user.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-users_top.msg", "220", False)
                
                '// Send part 2/2 of the reply to the user. (user list)
    
                '// Declares
                Dim FSO As New FileSystemObject
                Dim Drive As Drive
                Dim File As File
                Dim SubFolder As Folder
                Dim Folder As Folder
                Dim uI As Integer
                Dim UserIndex As String
                Dim strDrive As String
                Dim strFolder As String

                '// Set the start values.
                uI = 0
                strFolder = App.Path & "\data\users\"
                Set Folder = FSO.GetFolder(strFolder)

                For Each File In Folder.Files
                
                    '// Get the user info.
                    Dim uUser As String
                    Dim uGroup As String
                    Dim uTagline As String
                
                    uUser = Mid(LCase(File.Name), 1, Len(File.Name) - 4)
                    If uUser = "defaultuser" Then
                        GoTo NextUser
                    Else
                    End If
                    
                    uI = uI + 1
                    UserIndex = uI
                    
                    '// Make sure the strings are correctly formatted.
                    If CDbl(UserIndex) < CDbl(9) Then
                        UserIndex = "0" & uI
                    Else
                    End If
                    
                    uGroup = GetFromIni(Mid(LCase(File.Name), 1, Len(File.Name) - 4), "Group", App.Path & "\data\users\" & Mid(LCase(File.Name), 1, Len(File.Name) - 4) & ".usr")
                    uTagline = GetFromIni(Mid(LCase(File.Name), 1, Len(File.Name) - 4), "Tagline", App.Path & "\data\users\" & Mid(LCase(File.Name), 1, Len(File.Name) - 4) & ".usr")
                    
                    Call SendResponse(sckArrayloc, "220- | " & UserIndex & " " & uUser & MakeSpace(uUser, 13) & uTagline & MakeSpace(uTagline, 35) & uGroup & MakeSpace(uGroup, 19) & "|")
NextUser:
                Next
                
                '// Set the values to nothing
                uUser = ""
                uGroup = ""
                uTagline = ""
                Call SendResponse(sckArrayloc, "220- | " & "  " & " " & uUser & MakeSpace(uUser, 13) & uTagline & MakeSpace(uTagline, 35) & uGroup & MakeSpace(uGroup, 19) & "|")
                
                '// Set the total string
                uUser = "Total:"
                uTagline = UserIndex
                Call SendResponse(sckArrayloc, "220- | " & "  " & " " & uUser & MakeSpace(uUser, 13) & uTagline & MakeSpace(uTagline, 35) & uGroup & MakeSpace(uGroup, 19) & "|")
                
                '// Send part 3/3 of the reply to the user.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-users_bottom.msg", "220", True)
                       
            '// SITE USER <username> /////////////////////////////////////////////////
            Case "USER"
                Dim tmpCheckUserFlags As String
                Dim tmpCheckUser As Boolean
               
                '// See if enough values is supplied.
                If tmpArg2 = "" Then
                    SendResponse sckArrayloc, "220 Not enough values: SITE USER <user>"
                    Exit Sub
                Else
                End If
            
                '// See if the user exist.
                If CheckUser(tmpArg2) = False Then
                    SendResponse sckArrayloc, "220 User '" & tmpArg2 & "' does not exist."
                    Exit Sub
                Else
                End If
            
                Dim tmpAllowPassword As String
            
                '// Only show password if the one who executes the command
                '// is the "owner" of the user.
                If UCase(tmpArg2) = UCase(Client(sckArrayloc).UserName) Then
                    tmpAllowPassword = GetFromIni(tmpArg2, "password", App.Path & "\data\users\" & tmpArg2 & ".usr")
                Else
                    tmpAllowPassword = "**********"
                End If
            
                '// Send part 1/3 as the reply.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-user_top.msg", "220", False)
                
                '// Send part 2/3 (user info) as the reply.
                SendResponse sckArrayloc, "220- |Login       : " & GetFromIni(tmpArg2, "Username", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "Username", App.Path & "\data\users\" & tmpArg2 & ".usr"), 23) & "Password   : " & tmpAllowPassword
                SendResponse sckArrayloc, "220- |Group       : " & GetFromIni(tmpArg2, "Group", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "Group", App.Path & "\data\users\" & tmpArg2 & ".usr"), 23) & "Flags      : " & GetFromIni(tmpArg2, "Flags", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |Logins      : " & GetFromIni(tmpArg2, "Logins", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "Logins", App.Path & "\data\users\" & tmpArg2 & ".usr"), 23) & "Ratio      : " & GetFromIni(tmpArg2, "Ratio", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |Last online : " & GetFromIni(tmpArg2, "LastLoggedIn", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "LastLoggedIn", App.Path & "\data\users\" & tmpArg2 & ".usr"), 23) & "Num.online : " & GetFromIni(tmpArg2, "NumLoggedIn", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |                                                                        "
                SendResponse sckArrayloc, "220- |Statestics  : " & GetFromIni(tmpArg2, "TotalUPKb", App.Path & "\data\users\" & tmpArg2 & ".usr") & " KBup / " & GetFromIni(tmpArg2, "TotalDNKb", App.Path & "\data\users\" & tmpArg2 & ".usr") & " KBdown / " & GetFromIni(tmpArg2, "Credits", App.Path & "\data\users\" & tmpArg2 & ".usr") & " KBcredits"
                SendResponse sckArrayloc, "220- |-=-------------------------------------------------------------------=-|"
                SendResponse sckArrayloc, "220- |IP's registered:                                                       |"
                SendResponse sckArrayloc, "220- |-=-------------------------------------------------------------------=-|"
                SendResponse sckArrayloc, "220- |IP0 : " & GetFromIni(tmpArg2, "ip0", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "ip0", App.Path & "\data\users\" & tmpArg2 & ".usr"), 28) & "   IP5 : " & GetFromIni(tmpArg2, "ip5", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |IP1 : " & GetFromIni(tmpArg2, "ip1", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "ip1", App.Path & "\data\users\" & tmpArg2 & ".usr"), 28) & "   IP6 : " & GetFromIni(tmpArg2, "ip6", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |IP2 : " & GetFromIni(tmpArg2, "ip2", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "ip2", App.Path & "\data\users\" & tmpArg2 & ".usr"), 28) & "   IP7 : " & GetFromIni(tmpArg2, "ip7", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |IP3 : " & GetFromIni(tmpArg2, "ip3", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "ip3", App.Path & "\data\users\" & tmpArg2 & ".usr"), 28) & "   IP8 : " & GetFromIni(tmpArg2, "ip8", App.Path & "\data\users\" & tmpArg2 & ".usr")
                SendResponse sckArrayloc, "220- |IP4 : " & GetFromIni(tmpArg2, "ip4", App.Path & "\data\users\" & tmpArg2 & ".usr") & MakeSpace(GetFromIni(tmpArg2, "ip4", App.Path & "\data\users\" & tmpArg2 & ".usr"), 28) & "   IP9 : " & GetFromIni(tmpArg2, "ip9", App.Path & "\data\users\" & tmpArg2 & ".usr")
                
                '// Send part 3/3 as the reply.
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-user_bottom.msg", "220", True)
            
            '///////////////////////////////////////////////////////////////////////////
            'Add an IP
            Case "ADDIP"
        
            Dim tmpCheckAddIP As Boolean
            Dim tmpCheckAddIPAll As Boolean
    
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "2") = True Then
                tmpCheckAddIP = True
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Then
                    tmpCheckAddIPAll = True
                Else
                    tmpCheckAddIPAll = False
                End If
            Else
                tmpCheckAddIP = False
                tmpCheckAddIPAll = False
            End If
            
            'See if you are allowed to do SITE DELIP
            If tmpCheckAddIP = False Then
                SendResponse sckArrayloc, "550 Not allowed to do SITE ADDIP!"
                Exit Sub
            Else
            End If
        
            'If no username spesified
            If tmpArg2 = "" Then
                SendResponse sckArrayloc, "220 Not enough values - SITE ADDIP <user> <ip>"
                Exit Sub
            Else
            End If
            
            'If no ipnum spesified
            If tmpArg3 = "" Then
                SendResponse sckArrayloc, "220 Not enough values - SITE ADDIP <user> <ip>"
                Exit Sub
            Else
            End If
            
            'Check if user exist.
            If GetFromIni(tmpArg2, "login", App.Path & "\users.conf") = "" Then
                SendResponse sckArrayloc, "550 That user '" & tmpArg2 & "' does not exist!"
                Exit Sub
            Else
            End If
            
            'See if allowed to del from all, or just from own group
            Dim tmpMyOwnGroup2 As String
            Dim tmpIPi As Long
            Dim tmpAddedIP As Boolean
            tmpMyOwnGroup2 = GetFromIni(Client(sckArrayloc).UserName, "group", App.Path & "\users.conf")
            tmpAddedIP = False
    
            If tmpCheckAddIPAll = False Then
                If tmpMyOwnGroup2 = GetFromIni(tmpArg2, "group", App.Path & "\users.conf") Then
                    
                    For tmpIPi = 0 To 9 Step 1
                        If GetFromIni(tmpArg2, "ip" & tmpIPi, App.Path & "\users.conf") = "" Then
                            If tmpAddedIP = True Then
                                tmpAddedIP = True
                            Else
                                Call SaveToIni(tmpArg2, "ip" & tmpIPi, tmpArg3, App.Path & "\users.conf")
                                tmpAddedIP = True
                            End If
                        Else
                            tmpAddedIP = False
                        End If
                    Next tmpIPi
                    
                    If tmpAddedIP = True Then
                        SendResponse sckArrayloc, "220 IP number " & tmpArg3 & " added to user " & tmpArg2 & "."
                        Exit Sub
                    Else
                        SendResponse sckArrayloc, "550 IP number " & tmpArg3 & " could not be added to the user " & tmpArg2 & " - no more IP slots."
                        Exit Sub
                    End If
                    
                Else
                    SendResponse sckArrayloc, "550 You are only allowed to do ADDIP on users in your group."
                    Exit Sub
                End If
            
            'Allowed to addip for everyone that is on the site.
            Else
            
                For tmpIPi = 0 To 9 Step 1
                    If GetFromIni(tmpArg2, "ip" & tmpIPi, App.Path & "\users.conf") = "" Then
                        If tmpAddedIP = True Then
                            tmpAddedIP = True
                        Else
                            Call SaveToIni(tmpArg2, "ip" & tmpIPi, tmpArg3, App.Path & "\users.conf")
                            tmpAddedIP = True
                        End If
                    Else
                        tmpAddedIP = False
                    End If
                Next tmpIPi
                    
                If tmpAddedIP = True Then
                    SendResponse sckArrayloc, "220 IP number " & tmpArg3 & " added to user " & tmpArg2 & "."
                    Exit Sub
                Else
                    SendResponse sckArrayloc, "550 IP number " & tmpArg3 & " could not be added to the user " & tmpArg2 & " - no more IP slots."
                    Exit Sub
                End If

            End If
    
            '//////////////////////////////////////////////////////////////////////////
            'Del an IP
            Case "DELIP"
        
            Dim tmpCheckDelIP As Boolean
            Dim tmpCheckDelIPAll As Boolean
    
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "2") = True Then
                tmpCheckDelIP = True
                If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Then
                    tmpCheckDelIPAll = True
                Else
                    tmpCheckDelIPAll = False
                End If
            Else
                tmpCheckDelIP = False
                tmpCheckDelIPAll = False
            End If
            
            'See if you are allowed to do SITE DELIP
            If tmpCheckDelIP = False Then
                SendResponse sckArrayloc, "550 Not allowed to do SITE DELIP!"
                Exit Sub
            Else
            End If
        
            'If no username spesified
            If tmpArg2 = "" Then
                SendResponse sckArrayloc, "220 Not enough values - SITE DELIP <user> <ipnum>"
                Exit Sub
            Else
            End If
            
            'If no ipnum spesified
            If tmpArg3 = "" Then
                SendResponse sckArrayloc, "220 Not enough values - SITE DELIP <user> <ipnum>"
                Exit Sub
            Else
            End If
            
            'Check if user exist.
            If GetFromIni(tmpArg2, "login", App.Path & "\users.conf") = "" Then
                SendResponse sckArrayloc, "550 That user '" & tmpArg2 & "' does not exist!"
                Exit Sub
            Else
            End If
            
            'See if allowed to del from all, or just from own group
            Dim tmpMyOwnGroup4 As String
            tmpMyOwnGroup4 = GetFromIni(Client(sckArrayloc).UserName, "group", App.Path & "\users.conf")
    
            If tmpCheckDelIPAll = False Then
                If tmpMyOwnGroup4 = GetFromIni(tmpArg2, "group", App.Path & "\users.conf") Then
                    Call SaveToIni(tmpArg2, "ip" & tmpArg3, "", App.Path & "\users.conf")
                    SendResponse sckArrayloc, "220 IP number " & tmpArg3 & " removed from user " & tmpArg2 & "."
                    Exit Sub
                Else
                    SendResponse sckArrayloc, "550 You are only allowed to do DELIP on users in your group."
                    Exit Sub
                End If
            Else
                Call SaveToIni(tmpArg2, "ip" & tmpArg3, "", App.Path & "\users.conf")
                SendResponse sckArrayloc, "220 IP number " & tmpArg3 & " removed from user " & tmpArg2 & "."
                Exit Sub
            End If
            
            '///////////////////////////////////////////////////////////////////////////
            'Add a user
            Case "ADDUSER"
        
            '// Get the group of the user who performs the
            '// adduser command. This is because the user
            '// will be added to the group of the user who
            '// performs the command.
            Dim tmpMyOwnGroup As String
            Dim tmpAddUser As Boolean
            Dim tmpUserPassSame As Boolean
            
            '// Set or Get the values that are going to
            '// be used.
            tmpMyOwnGroup = GetFromIni(Client(sckArrayloc).UserName, "Group", App.Path & "\data\users\" & Client(sckArrayloc).UserName & ".usr")
            
            If GetFromIni("PassWord", "SameAsLoginAllowed", App.Path & "\data\settings.conf") = "1" Then
                tmpUserPassSame = True
            Else
                tmpUserPassSame = False
            End If
            
            '// Check to see if the user is allowed to
            '// access this command.
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "2") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                SendResponse sckArrayloc, "550 You are not allowed to do 'SITE ADDUSER'"
                tmpAddUser = False
                Exit Sub
            Else
                tmpAddUser = True
            End If
                   
            '// If the user haven't supplied a user to
            '// the adduser command, then spesify that.
            If tmpArg2 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-adduser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// If the user haven't supplied a password for
            '// the user then spesify that.
            If tmpArg3 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-adduser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// Check if the user already exist.
            If CheckUser(tmpArg2) = True Then
                SendResponse sckArrayloc, "550 User: '" & tmpArg2 & "' already exist."
                Exit Sub
            Else
            End If
            
            '// Check if username is too long, max is 10 chars.
            If Len(tmpArg2) > 10 Then
                SendResponse sckArrayloc, "550 Username is too long, max is 10 chars."
                Exit Sub
            Else
            End If
                 
            '// Check if the login and password is the same, and
            '// eventually if that is allowed on the server.
            If tmpUserPassSame = False Then
                If UCase(tmpArg2) = UCase(tmpArg3) Then
                    SendResponse sckArrayloc, "550 Username and Password can NOT be the same."
                    Exit Sub
                Else
                End If
            Else
            End If
                
            '// Create a new user.
            Call AddNewUser(tmpArg2, tmpArg3, tmpMyOwnGroup, tmpArg4, tmpArg5)
            
            '// Give some reply back to the user.
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-adduser_top.msg", "220", False)
            Call SendResponse(sckArrayloc, "220- Added the user: '" & tmpArg2 & "/" & tmpArg3 & "' to the group: '" & tmpMyOwnGroup & "'")
            
            If tmpArg4 = "" Then
                Call SendResponse(sckArrayloc, "220- No IP's added.")
            Else
                If tmpArg5 = "" Then
                    Call SendResponse(sckArrayloc, "220- This IP added: '" & tmpArg4 & "'")
                Else
                    Call SendResponse(sckArrayloc, "220- These IP's add: '" & tmpArg4 & "' and '" & tmpArg5 & "'")
                End If
            End If
            
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-adduser_bottom.msg", "220", True)
            Call AddLogItem("New user: " & tmpArg2 & "/" & tmpMyOwnGroup & " was added to the site.", Client(sckArrayloc).UserName, frmMain.lViewLog)
            
            '//////////////////////////////////////////////////////////////////////////
            'Add a user (group based) only flag 1 + 7
            Case "GADDUSER"

            '// Get the group of the user who performs the
            '// adduser command. This is because the user
            '// will be added to the group of the user who
            '// performs the command.
            Dim tmpGMyOwnGroup As String
            Dim tmpGAddUser As Boolean
            Dim tmpGUserPassSame As Boolean
            
            '// Set or Get the values that are going to
            '// be used.
            tmpGMyOwnGroup = GetFromIni(Client(sckArrayloc).UserName, "Group", App.Path & "\data\users\" & Client(sckArrayloc).UserName & ".usr")
            
            If GetFromIni("PassWord", "SameAsLoginAllowed", App.Path & "\data\settings.conf") = "1" Then
                tmpGUserPassSame = True
            Else
                tmpGUserPassSame = False
            End If
            
            '// Check to see if the user is allowed to
            '// access this command.
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                SendResponse sckArrayloc, "550 You are not allowed to do 'SITE GADDUSER'"
                tmpGAddUser = False
                Exit Sub
            Else
                tmpGAddUser = True
            End If
                   
            '// If the user haven't supplied a group to
            '// the gadduser command, then spesify that.
            If tmpArg2 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-gadduser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// If the user haven't supplied a username for
            '// the user then spesify that.
            If tmpArg3 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-gadduser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// If the user haven't supplied a password for
            '// the user then spesify that.
            If tmpArg4 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-gadduser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// Check if the user already exist.
            If CheckUser(tmpArg3) = True Then
                SendResponse sckArrayloc, "550 User: '" & tmpArg3 & "' already exist."
                Exit Sub
            Else
            End If
            
            '// Check if the group exist.
            If CheckGroup(tmpArg2) = False Then
                SendResponse sckArrayloc, "550 Group: '" & tmpArg2 & "' does not exist. Add it first."
                Exit Sub
            Else
            End If
            
            '// Check if username is too long, max is 10 chars.
            If Len(tmpArg3) > 10 Then
                SendResponse sckArrayloc, "550 Username is too long, max is 10 chars."
                Exit Sub
            Else
            End If
                 
            '// Check if the login and password is the same, and
            '// eventually if that is allowed on the server.
            If tmpGUserPassSame = False Then
                If UCase(tmpArg3) = UCase(tmpArg4) Then
                    SendResponse sckArrayloc, "550 Username and Password can NOT be the same."
                    Exit Sub
                Else
                End If
            Else
            End If
                
            '// Create a new user.
            Call AddNewUser(tmpArg3, tmpArg4, tmpArg2, tmpArg5, tmpArg6)
            
            '// Give some reply back to the user.
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-gadduser_top.msg", "220", False)
            Call SendResponse(sckArrayloc, "220- Added the user: '" & tmpArg3 & "/" & tmpArg4 & "' to the group: '" & tmpArg2 & "'")
            Call SendResponse(sckArrayloc, "220- These IP's add: '" & tmpArg5 & "' and '" & tmpArg6 & "'")
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-gadduser_bottom.msg", "220", True)
            Call AddLogItem("New user: " & tmpArg3 & "/" & tmpArg2 & " was added to the site.", Client(sckArrayloc).UserName, frmMain.lViewLog)
          
            '//////////////////////////////////////////////////////////////////////////
            'Del a user
            Case "DELUSER"
        
            Dim tmpHoldFlags As String
            Dim tmpDelUserFlags As String
            Dim tmpDelUser As Boolean
            Dim tmpDelUserGroup As String
            Dim tmpAllGroups As Boolean
            Dim tmpDelOwnGroup As String
        
            '// Check if the user have enough flags to do this.
            '// The user needs either 1, 2 or 7 flag
            '// to edit user.
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = False And CheckUserFlag(Client(sckArrayloc).UserName, "2") = False And CheckUserFlag(Client(sckArrayloc).UserName, "7") = False Then
                Call SendResponse(sckArrayloc, "550 You are not allowed to do 'SITE DELUSER'")
                Exit Sub
            Else
            End If
            
            '// Now see if the user is siteop/cositeop, and if
            '// so he/she should get permission to del users
            '// from all groups and not just his/hers.
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Then
                tmpAllGroups = True
            Else
                tmpAllGroups = False
            End If
            
            '// See if the user have spesified a user to
            '// delete, if not, let the user know about that.
            If tmpArg2 = "" Then
                Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-deluser.msg", "220", True)
                Exit Sub
            Else
            End If
            
            '// Check if the user exist.
            If CheckUser(tmpArg2) = False Then
                Call SendResponse(sckArrayloc, "550 User: '" & tmpArg2 & "' does not exist.")
                Exit Sub
            Else
            End If
            
            '// Get the group of the user that the user
            '// is trying to delete and the user self.
            tmpDelUserGroup = GetGroup(tmpArg2)
            tmpDelOwnGroup = GetGroup(Client(sckArrayloc).UserName)
            
            '// See if the groups are the same.
            If UCase(tmpDelUserGroup) = UCase(tmpDelOwnGroup) Then
            Else
                
                '// See if the user is allowed to delete people outside
                '// the users groups.
                If tmpAllGroups = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to delete people outside your group.")
                    Exit Sub
                Else
                End If
                
            End If
            
            '// Check if the user is already deleted.
            If CheckUserFlag(tmpArg2, "6") = True Then
                Call SendResponse(sckArrayloc, "550 User: '" & tmpArg2 & "' is already deleted.")
                Exit Sub
            Else
            End If
            
            '// Mark the user as deleted.
            Call DeleteUser(tmpArg2)
            
            '// Give some feedback to the user.
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-deluser_top.msg", "220", False)
            Call SendResponse(sckArrayloc, "220- User '" & tmpArg2 & "' of the group '" & tmpDelUserGroup & "' was deleted.")
            Call SendResponse(sckArrayloc, "220- Do 'SITE READD " & tmpArg2 & "' to readd the user with all info intact.")
            Call SendMessageFile(sckArrayloc, App.Path & "\data\messages\site-deluser_bottom.msg", "220", True)
        
            '///////////////////////////////////////////////////////////////////////////
            'Purge ALL users marked as deleted
            Case "PURGE"
            
            'Check if the user have enough flags to do this.
            'The user needs either 1 or 7 flag
            'to purge user(s).
            Dim I As Long
            Dim tmpPurgeUserFlags As String
            Dim tmpPurgeUser As Boolean
    
            If CheckUserFlag(Client(sckArrayloc).UserName, "1") = True Or CheckUserFlag(Client(sckArrayloc).UserName, "7") = True Then
                tmpPurgeUser = True
            Else
                tmpPurgeUser = False
            End If
            
            If tmpPurgeUser = False Then
                SendResponse sckArrayloc, "550 You are not allowed to do this command!"
                Exit Sub
            Else
            End If
            
            'M4 "2") = False And C4EeopUserF Now see if the user is siteop/cositeop, and if
   ee if the user have spesifie/ Mark the user  is stmpPurgeUser = TruH================gnd the user sel=0oraylosckArrayloc    Esy5Dw", "220", TrsckArr, "p       If tmDw",    'M4 "2") = False And C4EeopUserF Now see if the user is siteop/cositeop, and if
   ee if the user ha    S"p         End Ifp to
      se
      If tmDw",    'M4 "2") = False And C4EeopUserF Now see if the ohe user ha    S"p    nruH======oa        uTaglidrs.
      S"p  see if the ohe user h if oglide if thgorF 4w",
      se
 =======   'M4iteop, and if
   ee if the user ha    S"p         End Ifp to
      se
      If tmDw",    'M4 "2") = False And C4EeopUserF Now see if the ohe user ha    S"p       Call SendResponse(sce C4EeopUse if 4 = Tum If Eeo3 if Eeo3 &ndMesxo(a DELIP
            If tmpCISosit   nd C4EeopUserF Now olidrs.
  _aResponse(sce C4EeopUse if 4 = Tum If Eeo3 if Eeo3 &ndMesxo(ae ThenResponscie(scer = Ftgse tmpArg5 & "' and '" & tmpAUxo(ae Then   nd C4EeopUserF Now olidrs.
  _aResponse(sce C4 if Eeo3 &ndMesxo(ae            Iffser.       if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if Eeo3 &ndMesxo(ae     PAll = TruD "DELUSER"
        
 User::::::lmrrayEeo DEonse(sce::::::dMesif
  espientnse(sce IP5 :are not althe same.
            If UCase(tmpDelUserGroup) = UCaf
  etmpDelUserGroup) = onse(skArr, "p       If tmDw",    'M4 "2") = False And C4EeopUserF Now sev If CheckU= onse(skArr,         t the  if p         End Ifp to
      se
      If tmDw",    'M4 "2") =), 1,  & tmpArg2 & USER"
       se
      If tmDse
     3 & "' already exist."
        aAnd CkrDEonse(scthe usr- |IP2 : " & GetFromIni(t"220 : " & Get&3 & "'e(scthe usr- |IrTrue Then
      '// Check if the lEet&3 & "     Iffser.       if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if Eeo3 &ndMesxo(ae     P)Mesxo(ae     Now see iss/PurgeUser = wd C"1") = True     Iffser.       if g5 & " = wdW & "' already exist..usr(scthjs    if g5 & " = wdW & "' already exsllesxo(ae     P)Mesxo(r 7 flag
            'to purge user(s).
    UserFPurgeUserF   wdW th & "\data\users\" & Client(sckArrayloc).U "' alrags As String.
mlrai"v       1     P)Meln1     P)M            1     P)Meln1     P)M            1     P)Meln1     P)M            1     ser2 & "lyse(sc     1     P)Meln1     P)M        (skArr,            wdW th & "\da               If tmpAllGroups e sckAr        S"p         End I     
            '// Give some feedback to the user.
            eleteUser(tmpArg2)
       'i1     P)M            1    oups7eUser(tp: '" & tmpMyOutf tmpAllGr.rnf g5 & "   Else
 eedback       '// Cet             "     '// Cet 'to pur// Cet 'tIuame = False
            End If
          "LastLoggedIn", App.Path & "\data\users\" & tmpArg2 & ".usr"), 23) & "Num.online : " & GetFromIni(tmpArg2, "NumLoggedIn", App.Path & "\data"Num.online : " & GetFromIni(tmpArg2, "NumLoggedI(    D    Ie       pArp>
            End If
            
   ftom.msg", "220", True)
'// Cet             "   //
            'Purge nd ")
   Meln1   1     P)Meln1     P)M        (skArr,            wdW th & "\da               If tmpAllGroups e sckAr        S"p         End I     
            '// Give some feedbaye feedbalready Brye fserName, "7") =, TrondEnd I     If
  (    D    Ie       pArp>oa feedback5, tmpAg usr- |IrTrue Then
// CheckondEn
                Usto
  DeUser = wd C"1") = True     Iffser.   "7") =,
            Else
rnr    If
       fseel=0oraylosckArr   '
   ftom.msg", "220", Tip 
   ftom.msg", "  pArp>oa feedback5, tmpAg usr- |IrTrue Then
// Che/ SAg usr- |IrTrue Then
W-deluser_ '//mpArg5 & "' and '" & tmpArg- |IrTrue 
mlrai"v  kondEn
   >oa f P)Mel Case "Pto do this.
            '// The user needs mpArg2)
   (    D    Ie  //mpArg5 & "' andl Case "Pto do tilse
            End If
  d0ps e 4sR & ".usrdy deleromIni(tmpArg2, "NumLoggedIn", App.Path & "\data"Num.online : " & GetFromIni(tmpArg2, "NumLoggedI(    D    Ie 
    "                Exit Sub
o4m tmpCheckAddIPAll As Boold, then s 
       AeWtIf
       fsusers\" & Client(schen
                        If tmpAddedIP = True Then
        yo          se    End If
  d0ps e      s 
ite-gad     
ite-gad     
ite-gad .   (>oa feedback5, tmpAg usr- |IrTrue Then
ad .   (   End If
  d0ps e      s 
it, 23) & "Num.online : " & GetFromIni(tmpArg2, "NumLoggedIn", aKraylourgeUserTrue4Eeoprp>
     IMr\" & Client(sckArraylne : " & GetFromIni(tmpArg2, "NumLoggedIn", aKraylourgeUserTrue4Eeoprp>
     IMr\" & Client(sckAroc, "Boold Se).UserName,er  ispAg usr- |IrTrue Then
/FromIni(tUiiiiiim.msg", MLoggedIn", & tmpArg2rthen          tilsFromI& "' already exist.y >1tom.m|ne tmpArg5 & "' and '" &                                                                    =          
      , "220m tmpD             fsusers\" W, 23) & "Num.onl=t >1tom.m|ne tmpArgc & "      m
  aib     (tmpArg3, ta C4E", App.Path & rath & c"CkrDE1         fsusers\" W, 23) & "N      =                           
  d0ps e 4sR 9om.  P, 23) & "iye feedbalready Brye fsEAD tm23) .org2 = "" Then
         fsusers\" W, 23) & "N      =                           
  d0ps e 4sR 9om.  P, 23) & "iye feedbalready Brye fsEAD tm23) .org2 = "" Then
         fsusers      '/       y= "" Then
xit Sub
o4m tmpCheckAddcthe usr- |rg2 =n    Ie   o.PFPurg if g5 & " = wdWbS          Else
              y= "" Then
xit Sub " & Ge/ "" Then
lse
   TromI& "' 
              y= "" Then
xit Sub " &          d If
 3) & "Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"Nu"           lNu"Nu"        
          Lashready Brye fsEAD tm23)Leedban        isr- |Ire 
mlrai"v  kondEn
   >oa f P)Mel T>0            y Brye fsEAD tm2            y= "" TheEAD tm2     AD tm2  'ESrai"v  kondEn
   >oa f P)Mel T>0       ) & "N      "" Th flag 1 + 7
     "" TArg2, "() &           En>1tom.m|ne tNn       AD tm2 lN      b     (rue o  S"p    nruH======oa        um2     AD tm2  'ESrai"v  kondEn
   >oa f P)Mel T>0       ) & "N      "" Th flag 1 + 7
     ""            (>oa feedback5, tm         y= "" Then
xit Sub " & Ge/ "" Th   y= "" rai fee,word is the samit S    (A(tmB1    nrcuite-gad .   (>e-addng
 xit Sub " & Ge/ "" Th   y= "" rai fee,word is the samit S 
    7 f   ayloc, "220- |IP1 : "'tm2  'ESrai"v  kona          y= "" The Ge " & Ge/ "" Th   y= "" rai fee,user ha    S"p       Call SendResponse(sce C4Eeha    S"py= "" Thr) & "N      "" Th fWl Send    t S   deckT"" Then
                Call SendMe.m|nis  The Ge "r End Ifp to
      se
      If tmDw",    'M4 "2") = False And C4EeopUserFm         y= "" Then
xit Sub " &    (skArot 2tmpAgeUserF  3at are going to
           the oheftmB1   " L") = Fa 1False And C4EeopUserFm         ySub tNumLoggedIn", App.Path & "\data"N>oa f,        y= "" Then
xit Su9IDsps e et  (s& " re Ips7eUs///////// (s& " 3                 
  d0ps e 4sR 9om.  P, 23)6X   If tmDw",    'M4 "2") = False And C4EeopUserFm       e etmpDelO   nnbhe oheftmB1   " L") = Fa 1Fal   the oh   E.
   "Nudhec     f  'M4 "2") = False And C4EeopUserF NowprpIps7eUs/l   the   u"Nu"NusnT  Usto
  DeUser = wd C"1") = True     Iffser.    u"Nu"NusnT  Usto
  DeUser = wd C"1") = True     Iffser.    u"Nu"NusnT  Usto
  DeUser = wd C"1") = True     Iffser.    u"Nu"NusnT  Usto
  DeUser = wd C"1") = True     Iffser.    u"Nu"NusnT  Usto
  DeUser = wd C"1") = True     Iffser.    u"Nu"NusnT  Usto
  Des t       lient(sckArraylD1") = True     Iffser.    u"Nu"NusnT  Usto
  Des trDIYkv  kEraaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaag   rName,ArggggggggggUserF NowprpIps7eUs/l   t---------------------=-|"




 ed "Nu"N  S"trDIY'l = s site   sckArrayloc).UserName, "group", App.Path & "\users.conf")
            tmpAddedIP = False
    
            If tmpCheckAddIPAll = False Then
                If tmpMyOwnGroup2 = GetFromIni(tmpArg2, "group", App.Pat tmpMyOwnGroup2 = GetFromIni(tmpArg2, "group", Rt SjArp>Zg", "220", True)
'// Cet     ue     Iffser.    u"Nu"vMesxo(serFlagDs As Boolean
           te-adduser_bottom.msg", "220", True)
      EIP
            If tmpCheckDeer.
            If tmpUse   e etY'l = s site   sckAF           Else
     performs the
   aaaaag   rName,ArggggggggggUserF Nolp220roo
  DeUser = wd C"1") = True     Iffser.    u", "1nmomDw",    'M4 "2") = False And C4EeopUserFm         y= "" Then
xit Sub " &    (skArot 2tmpAgeUserF  3at he oc    Esy5Dw", "220", TrsckArr, "p       If tmDw",    'M4 "2") = False And C4EeopUserF Now see if the user is siteop/cositeop, and if
   ee if the user ha    S"p         End Ifp to
      se
      If tmDw",    'M4 "2") = Fa    SendResponse sckArtmpAd'M4 "2")    Iffser.    u"Nu"NusnT  Usto
  DeUser = wd l      eUser = wTusers\" & tmpArg2  xit Sub " & Ge<Met     ue
 eedbm         r- |IrTrue Then
W-delidIn", aKrnd Ifp to
      se2wp) = True     Iffser.    u"Nu"NusnT  Usto
  Des t       lient(sckArraylD1") = True     Iffser.    u"Nu"NusnT  Uf tm R3 u"Nu"NusnT  UdylD1") = True    c |IP1r.    u"Nu"NusnT  Uf tm R3 u"Nu"NusnT  UdylD1") = True    c |IP1r.    u"Nu"N" & Ge<STnbhe ohen
xit Surnd If
   UdylD1") do
      se
      If tmDw",    'M4 "2") = Fa    SendResponse sckArtmpAd'M4 "2")    Iffser.    u"Nu"NusnT  Usto
  DeUser = wd l rs3) eopUserF Now see if the user is siteop/caSub " & Ge/ "" Then
lse
 GnserF NowxLpm   Ily ezpArg5 & Then
lse
v   s=ezpA eedbm   r  UnT  UdylDd'M4 "2")dbmue
 ee Then
W-delidIn", aKrnd Ifp to
      se2wp) = True     Iffser.    u"Nu"NusnTrER'"e/ ""  D    Ie       ptse
 Iff use  ptse
 fR'"e/ ""  D    Ie       ptse
 IffxLpm   Ily ezpArg5 & T Ie       ptse
 Iff use  ptse
 fR'"e/ ""  D    Ie       ptse
 IffxLpm   Ily ezpArg5 & T Ie       ptse
 Iff use  ptse
 fR'"e/ o
      se2    orIff se  tom    se
      If tmDw",    'M4 "2") = Fa   ent(sckArhen
                Call SendMe.m|nis  The Ge "r End Ifp to
      se
      If tmDw",    'M4 "2") = False And C4EeopUserFm         y= "" Then
xit Sub " &    (skArot 2tmpAgeUserF  3at are going to
           the oheftmB1   " L") = Fa 1False And C4EeopUserF C4EeopUserF C4EeopuIf tmpUse   e etY'l = s site   sckAB
    
    mue
 ee Then
W-delidIn
 ee Then
W-dFdf tmDw",    'M4 "2"iy Ifp to
  mnnd C4Ee "220- |IP1 : "'tm2  'ESrai"v  kona          y= "" The Ge 4Ee PurgeUserFla"e/ ,    'M4 "2sps = False Then
                    Call SendResponse(sckArrayloc, "550 You are not allowed to delete people outside your group.c     9
    NusnT  U     Cala|ne tmpArgc e "l SendReDw",    'M4 "2"nd C4EeopUserFm Usto
  DeUser = wd l rs3) eopUserF Now see i  fsusers\" W, 23) & "N      =  r = w    'M4 "2") = Fw     fsuserst      se2    orIfferF  3at he oc    Esy5Dw", "220", TrsckArr, "p       If tmDw",    'M4 "2") = False And tmpGAddUser As Boolean
            Dim tmpGUserPassSame duser command. This is because the user
      yYou are not allowed to do 'SITE DELUSER'")
   'SITE " & wAddU w oR'")
   'SITE " & wor  wAddU w oR'",    'M4 "2") = False And tmpGAddUser As Boolean
            Dim tmpGUserPassSame dus : "' user commandser = wddUser ARa C"1") = True     Iffserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\userslTE "5, MLoAFalse And(t      Elseevie DeUser = wd l rs3) eopUserF Now see i  fsusers\" W, 23) & "N      =  r = w    'M4 "2") = Fw  supplied a passworgggggUserF Nolp220roo
2 ptsese(sce C4 if Eeo3 &nd Now see i  fsuR.cona/l  tOifsLU tOii2")    Iffser.    u"Nu"Nusrs\" W, 23) & "N      =  r = na/l  tOifserto
 "\users.cona/l  tOifserto
 "\  tOifsertMuS
eers.cona/l  tOifserto
 "\  tOifsertMuS
eers.cona/l  tOifserto
 "\  tOifsertMuS
eers.StOifserto
 "\users.cona/l  tOifserto
 ", & tmpArg2rthen          tilsFro2ifserto
 "\users.cona/l  tOifserto
 ", & tmpArg2rth'ertoc going to
aK                     ' Iffser.    u  "LastLogget  &nd(t       IrPassSame duser command. This is because theUs/////yOwnG               _3TAddUser As Boolean
   E " & wors Boolean
   E " & woCwnG  mBoolean
  i         
            'Check if the user have enough fF Now tG     Call SendM Boolean
   V theUs/////yOwnG        u     ser.    u"an
   V theUs//// theUs/TAddUser As B      Call Seolean
  i      o Iffsupplied aBI     _3TAddUser As Boo  Cal   o Iffserto
 """"""""""so"Nu"NilsFro2222  ElseDoa feedback5serto
 ", & tmnt   
     r     u     ser.     (skA   user ha    S"p         End Ifp to
      se
      If tmDw",    'M4 "2") = Fa    SendResponse sckaaaaag   rrrrrrrrr     "erto
 4   Iffser.    u"Nu"vMesxo(serFlagDs As Boolean
    hhhhhhhhhhhhhhhhhhhhhhhhhhhh1feck if the user have enough fF Now tG     Call Se = wdow tG     Call Se = wdow tG     Call Se = wdow tG     Call Se = wdow tG     Call Se = wdow tG     Call Se = wdow Kely ezpArg5 & Then
lse
v   s=ezpA eedbm   r  UnT  Udbaceedbm oi  UdfUser = wd C"hy5Dw", " e user.a2if g5 & "' and 'Cw olidrs.
  _aRespotxhhhl    D   (skA   user ha    S"'Ln
  i      o Iffsupplie Ily ezpArg5d 'Cw olidrs.
  _aRenough fF NowAddUser As Boolea23) & "N      =                           
  d0ps e 4sR 9om.  P, 23) & "iye feedbalready Brye fsEAD tm23) .org2 = "" Then
         fsusers\" c ing to
FromIarafF Now tG     Call SIarafF No if the user have enough fF  _aRenough fF NowAddU feedbalready Brye fsEAlean
   E " & wors Booleaser  fFelErp>
     IMr\" & Client(sckAroc, "Boold Se).User     tC4EeopUserFm  sfF NowAddU feedbalready Brye fsEAlean
   E " & wors Bhe ohen
xit Surnd If
   UdylvS3o  Cal          SendResponsCal    fF  _aRenough fw/"220"    Call SendResponse(sckArrayloc, "220- These IP's add: '" &   Udfese IP's   I wors Bhe l SendResponse(sckArrayloc"Num. wors Bhe l sckAroc, "Boold Se).Usto
      se
      If tmDw"sg", "220", Tr "220- Added the user: '" & tmpArg3 & "/" & tmpArg4 & "'d tmDw",   iSlD1")     Case
 ", & tmnt   
     r     u     e lDw",   iSlD1")      ready Brye fsEAlean
  eventually if that is allowed on the server.cT,   iSlDlR 
    mue
 ee Then
W-delidIn
 ee Then
W-dFdf tmDw",    'M4  iSlDlR 
    mue
 ee T  muR 
    mue
 ee T  mu   SlDlPooln
   V tuR 
    mue
 ee T  mu   SlD   SlDlPooln
   V tuR 
    mue
 ee T  mu   SlD   SlDlPooln
 
  d0ps edf tmer = l      =                    u  = Fa  e lDw",   iSlDk if the lEet&3 & "     Iffset&3 & "         mue
 ee T  mu   SlDlPooln
   V tuR 
    muiUsto
      se
f g5 & "' and 'Cw olidrs.
  _aRespotxhhhl    D   (hl    D   Arg4 & "'doln
   n g5 & "txhh tmp "7") = False Then
    e
 ee TTrue     Iffser.    u"Nu"NusnTrER'"e/ ""   n g5 & "txhh tmp "7rER'"e/ ""    oupe
 ee e    k if the user have eno_ee Th7rER'y sify that.
          p220 fy that.
     lDlR 
 euiUsto
sps.
                "e/ ""    oupe
 eerF C4Eeopuahe group of t"    
 "\users.cona/l  tOifsertS    DD ptsue
 ee T  muR 
    mue
 ee T  mu   SlDlPooln
   V tuR 
    mue
 ee T  mu   SlD   SlDlPooln
   nuR 
    mSlDlPooln
   nuR 
 h2sps = False TorsgUserGrslDlPooln
   nuR 
 h2sps = L 
 ee 
  
   3  u"Nu"N" & Ge<STnbhe oheoCc4EeopupoSirst."
       all Se = wdou h2sst."S
    mue
 ee T  mu   SlD   SlDlPooln
   nuR 
    mSlDlPooln
   nuR 
 h2sps = False TorsgUserGrslDlPooln
   nuR 
 h2sps = L 
 else Ips kuser have eno_ee Th7rER'y sify that.
          p220 fy that.
  & "txhh tmp "7rER'"e/rto
 ",g      all Se = wdou h2sst."S
    mue
 ee T  mu   SlD   SlDfy that.
  & "txhh tmp "7rER'"e/rto
 ",g      all Se = wdou h2sst."S
    mue
 ee T  mu   SlD   SlDfy that.
  & "txhh tmp "7rER'"e/rto
 ",g      all Se = wdou h2sst."S
    mue
 ee T  mu   SoughrtMuS
eer"e/ ""    oupe
 eerF C4Eeopuahe group of t"    
 "\users.con "7rER'"e/rto
 ",g    n" rai feoup of t"   aendRect7rER'"e/rto
 ",e   n tmp "7rEReTEonse(sce:c   (hltooln
at.
rai                   escCtooln
at.
rai                   escCtooln
at.
rai                   escCtooln
at.
rai    ty"220- These IP's add: '_rd is the samit   tycc""2") n
   nuR 
  n" rendRe eerPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPassSaPasOArto
(of t" ssssse "DELs.StOpArg2rth'ertoc going to AddLogItem("Ne             pArgc e "l SendReDw",    '     usr- |IrTrue Then
/FromIni(tUiiiiiim.msg", MLoggedIn", & tmpArg2rthen          tilsFromI&( I,hhhhhhhhhh1feck if IP's add: '_rd is     tils                   escCtooln
at.
rai                   escCtooln
at.
rai    ty"220- Thess"550 sSaPassSaPrER'"e/ ""   n g5 & "txhh tmp "sr- |IrTrue   ty"ser exist.ue Or CheckUserFlag(Client(sckArertMuS
eers.SorsedIns   .ue"a new usee         fsuooln
at.
ra  E "uiAFalse An               escCtooln
at.
rai         gef the group exist.
            If CheckGroup(tmpArg2) = False Then
                SendResponse rER'"e/rmF  3aisDser ARa C"R'"0", e Then
                SendResponse rER'"Lashready Brye fsEAD tm23)Leedban ct7rER'"e/rto
 ",e   n tmp "7rEReTEonse(sce:c   (hltooln
a'"Lashready tm23)hSGThenPAD txrs are g.
ra  E.Bryato
  a    hhhh Then   nd C4EeopUseaEeedban ct7rER'"e/rto
 ",e   n tmp "7rEReTEoNdReD
 m23.Bryato
  ava (ckoNdReD
 m23.Bryato
  ava (ckoeSOt.
raiyOwnGroup = GetFromIni(Client(sckArrayloc).UserName, "Group", App.Path & "\data\users\" & Client(sckArrayloc).UserName & ".usr")dArray   0TrEReTEof tmpGy   0TrEReTEof tmpGy   0TrEReTEof tmpGy   0TrEReTEof tmpGy   0TrERu"e/recf tm  Uf tmIM0rray   0TrER             escCtooln
at.
rai   5 & "txhh tmp "7") = .
rai    ty"220 - |IrTrue Then
/FromIni(tUiiiiiim.msg", MLoggedIn", & tmpArg2rthenme & _NdReD
 m23.Bryato
SaPasIn", = wd C"1"Pai .Path & "220 - |Iim.msserFsagemessagesER',  o    uo",e Name     Iffser.       if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if EeSaPasIn", = wd C"1"Pai .Path & "220    B.    u"if E, = "7 olidrs.
  _aResponse(sce C4 if EeSaPasIn", = wd C"1"Pai .Path & Respotxhhh            p "7"Di .Path & (tbomIni(Client(sckArraylln
at.
rai   5 & "txhh    
SaPasIn",asIn",asIn",asIn",asIn",asIn",asInc l    'M4 "2") =t(sckArertMuS
eers.SorsedInsn", emessagesER',  o    uo",aInc l sagessr As Boole "2") = False And C4EeopUserF Now see if the user is siteop220 - |Iim.msserFsagemessagesER',  o    uo",e NamedIn
 ee Then
W-dFdf tmDw",    'M4     escrnGrof.r ath & "\data\users\" & Client(sckArrayloc).UserName & ".usr")dArray   0TrEReTEof tmpGy   0TrEReTEofrsedInoupe
 ee e    k if ti    ty"220- These IP'sng to
           the oheftmB1   " L") = Fa 1False And C4EeopUserFm         ySuis  TIf tmDw",    'M4 "2") =), 1,  & tmpArg2 & USER".sm n.lViewLog)
          i.           1 Ie       ptse
 Iff use  ptse
oughrtMuS
elr"\use. ) =t= wd C"1In
 erTrue s    'M4 "2edIs"2") =), 1,  & tmpArg2 & USER".sL     Iffser.   USER user haLog)
          i.           1 Ie       pts o    uo",aInc l sagessr As Boole "2") = False And C4oup", Ap      pts o   ie&3 & "    ,aInc l sfp to
Ipts  & "    ,"   i    ty"220- These IP'sng to
 ,"   n
  i      o IffsuppliedromInilea:  CaughrtMuS
elr"\use. ) =t= wd C"1iDng t rs3) eopUserIn
 erTrue slr"\ao
   eff use  p.
rai   5 & "txhh tssSa0esp& tmpArg2 & USER".sL     Iffser.  pn",asIn",asIn",a20- |IP1 : "'tm2  'ESrai"v  kona          y= "" ThLoggedInr"\use. ) eSaPasIn", = wd C"1"Pai .Path & Respotxhhh            p "7"Di .Path & (tbomIni(Client(sckAOt'/       y= "" Then
xit Sub
o4m tmpCheckAe2wIn",asIn",a20- |IP1 Sennt(s
 ee e    k if the user d onask ifO:c iArraasIn",a     if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if EeSaPasIn", = wd Csame, "7") = False Then
        e lDw",   iSlDk if the lEet&3 & "     Iffs4m=      If tmDw",    'M4 "2") = Fa   ent sfp to
Ipts  & "    ,"   i    ty"2  'toh    'N onask -" ThI    rrayloc, ApRCheckAe2wIn",asIn",a20- |IP1 Sennt(s
P            N SaPasIn", = wd = False Then
         en
     isIn",scCtooln
at.
rai      a20- |IP1 cotxhhtStnartmpAese. )& t",scCDi            If CheckUserFlag(Client(sckArrayloc).UserName, "1") =_bottom.msg", n"                  escC is siteop220 - |Iim.msserFsagemessagesER',  o    tmDw",    'M4OChecknartmpAese. )& t",scCDi            If CheckUserFlag  o3   Iffser.  ;hh1feDRrang5 & o   Log)
          i.     i)thtdescC is siteop220 :hh   rnqeck                  escCtooln
t& "tmpAestmpAestm  escCtoolt",scCDi            If CheckUserFlag  o3   Iffser.  ;hh1feDRrang5 & o   Log)
          i.     i)thtdescC is siteop220 :t Trueg   i)thRRts  e      s 
ite-gltmpAestm  escCtoolt",:t Trueg hh   rnqeck                  escCtooln
t& "tmpAestmpAestm  escCtoolt",scCDi            If rteopOnIf rteop          If rteopOnIf rteop          If rtand 'Cw olidrs.
  _aRespotxhhhl    D   (skA   usescCtoolt",scCDi          y p220 :t Trueg   i)thRR      i)thtdescC is siteopl      If CheckUserFlag(     rteop   s 
ite-g0- These IP'sng to
           the oheftmB1   " L") = Fa 1False And C4EeopUserFm         toospot"g to
           thslse And C4 & "'"e20 :hh   be-g0a   p   oolt",scCDiC4EeopUsery   0TrEReTEof tmpGy   0TrERu"e/recf tm  Uf tmIM0rray   0TrER             escCtooln
at.
rai   5 & "txhh tmp "7")   if g5 & "      E- |IP1 : "'tm2  'ESrai"v  kona          y= "" ThLoggedInr"\use. ) eSaPasIn", = wd C"1"Pai .Path & ResnnnnnrName &    0TrERelPooln
 C"1"Pai .Par&    0TrERelPoSalk "7rEReTEoNdReD
t"g to
  eerr3"7rEaaaaaaaaaaaaaaaaaaal  1 Ie       pts o    uo",aInc l o    unGroup) Then
            Else
               hFe=Ft And C4EeopUserF C4EeopUseLoupe

            '// Check if the login and       '// Check if the login   Log)
,_    uIf CheckUserFlag(     rteop   s 
ite-g0i,  .4rFs.b   " t.|IrTrue Then
/FromIni(tU   " t.|IrTrue Then
/FromIni(tU   " t.|IrTrue Then
Ei    - |IP1 : "'tm2  'ESrai"v  kona          y= "" ThLoggedInr"\use. ) eSaPasIn", = wd C"1"Pai Ucona/l  tOifserto
 i3
 eerF C4EepDhe       ohef.Path & D |IP1 : "'tm2  F Cath & Due     Iffser.    u"Nu"NusnTser is s   e login   Log)
,_  ",a    Else
               hFe=Ft An aeukona     opUserFm         toospot"g to
           thslse And C4 & "'"e20 :hh   be-g0a   p   oolt",scCDiC4EeopUsery   0TrEReTEof tmpGy   0TrERer.  mee e    k if the user d onask ifO:c iArraasIn",a     if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if EeSaPasIn", = wd Csame, "7") = False Then
        e lDw",   iSlDk if the lEet&3 & "     Iffs4m=      If tmDw",    'M4 "2") = Fa   ent sfp roo&3 & "  f the lEet&3 & "    3 & "     Iffs4m=    
        e lDw",   iSlDk if the   nuR 
  n" rendRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs.
  (up2 = GetFromIni(tmpArgDi rCD           W lDiCsame, "f Ee"7rER'"e/rto
 ",g    n" rai feoaRes.lf EeSaPae9ckAe2wIn",e, "fMesxo(serFlaget&3 &wRe effs4m=ggedthe lEet&3 & "  drs.
  _aReswRe effs4m=ggedthe lEet&3 & "  drs. 
  _aReswRe effs4m=ggedthe laReswRe effs4m=gg effs4m=ggedthe lEet&3 eswRe effs4m=the the laResg2 & C4 &r.cT,   Ls4m=oNtmpArsg2 s       eswRe effs4m=g eswRe ef   p"2"iRe effs4m=g olEe<t
   Melnfs4m=gg effs4m=ggedthe lEet&3 eswRe effs4m=the the laResg2 & C4 &r.cT,   Ls4m=oNtmpArsg2 s       eswRe effs4m=g eswRe ef   p"2"iRe effs4m=g or eff=g eswRm  sfF=ggm=ggedthe lEet&3 ee ef   p"han4m=ggedthe lEet&3 & "  drs.
  (up2 = GetFromIni(tmpa          y= "" =ggr(sce C4 if        End Ifp to
      sR & "/" & tmpArg4 & "'d tmlmuser d if
            '// so he/she should get pe" wd eThen
hhhr = p2 = GeR/she should get pe" wd eThen
hhhr =ihen
hhhP1 cot/Ina     opUserFm         toospot"g toS
 m tmlmuser d if
R   if g5 & "' and 'Cw olidrs.
  _aResponse(sce C4 if EeSaPasIn", = wd Csame, "7") = False Then
        e lDw",   iSlDk if the lEet&3 & "    
 ee T  mu   SlDlPE 'ESrai"v f tmpArg2 = "" Then
                C
  rERendReDw",    im.msReswRrswRm  sfoggedInr"\4C
  rERendReDw",    im.msRe   E;,   me lEet&3 ee e   E;,   me lE") = False Then
 to ee e rsResponse;,   me(sceif
R RRRRRxctsponse(sce C4 if EeFm   _aResponse(sceN onaskaren
 to ee e rsResponse;,   t&3 & "  #s4m=g olEe<t
   Melnfs4m=gg effs4m=ggedthe lEet&3 eswRe effs4m=the the laResg=ggeNE 'ESrai"f4 if EeFm   _aResponse(sceN ongongot SlD  tmDw",    'M4 "2") ut&3 aM4 =ggec"L   MelaM4 "2") = Fa  0TrERelPln
t& "tmpAestmaM4 "2") =the lmuser d if
R   et =      v  k _aRecDponse(sceN0D  ef Ee  _aG0ps  oolt",sc   Mel",    'M4 "2") = eaReswRe effs4m6E 'ESrai"rtat Boo&3 & "  f tRe effs4mrt
   Melnfs4m=gg effsg2 & C4 &r.cT,   Ls4sc        ac    c        ac  2wIDlPoolng0a   p   hn
at.
rai         gef the group exist.
  IDlc    c      ll Se = wd     Iffser.  pnv    '/rRRxctsponse(sce    E;,   me lE ac  2wIDlPoolng0a   p   hn
at.
rai         gef the group exiSif
s.
  (up2 = GetFromIni(tmpO        .    f
  d0pse fsEAD tm23) .org2 = tnpO  lng0a   p   hn
at.
rai         gef the group exi2Srai"v f tmpArg2 = "" Then
               (i       MelaMSrai"v  kona          y= "" The Ge " & Ge ooln
   V tuR 
    mue
 ee T  mu   SlD   SlDlPooln
   V tuR 
   =the lmuser d if
R   et =      v  k _aRecDponse(sceN0D  ef Ee  _aG0ps  oolt",sc   Mel",    'M4 "2") = eaReswRe effs4m6E 'ESrai"rtat Boo&3 & "  f tRe effs4mrt
   Melnfs4m=gg effsg2 & C4 &r.cT,   Ls4sc        ac    c        ac  2wIDlPoolng0a   p   hn
at.
raiThen
teopOnIf rteop          If rteopOnIf rteop          If rtand 'Cw olidrs.
  _arai"rspAg usr- |IrTrue Then
/FromIt                nse;,   _aG0ps  oolt",s ,    'M4 "agesER',  o   SlDlPooln
") =the lmus_      Else
e ,    'M4 "agesEm=     '// Thue Then
/F     sm  ef Ee     p"2"iRe4m=    
   tetFromIni(tmpArg2, "group", App.ltetFromInimExiiu"Nu"Nux"Then
                C
  rERendReDw",  tbo.UsersSaPa
  _am tmlmuser d if
R   iT
       'M4 "2") = eaReaS ni(tmpne slr"\ao
   eff use  p.
rai   P)Meroup", App.ltetFromInimExiiu"Nu"Nux"Then
                Cto
 ",g  in
at. nse;m=    
   tetFeaS nep", App.ltetFromInim  
   tetFromIni(tmpArg2, i            ;m=    
   tetFeaS nep", Ap(nse(sbm("Ne   si.
  IDlc    c      ll Se = wd     Fa  deTEof tmpGy   0TrERu"e/recf tm  Uf tmIM0rray   0TrER   N0D  ef Ee  _aGaIM0rray   0TrER   N0D  ef Ee  _aGaIM0f tmpGy  M0rray   0TrE(V tcp", pAnd C4EeopUserF C4EeopUseLo4m6E 'ESrai"rtat Boo&3 & "  f tRe effs4mrt
   Melnfs4m=gg effsg2 & C4 &r.cT,K      UseLo4m6E 'ESrsseLo4m6E 'ESrss4 &r.cT,K  o4mmmmasg2mSaPae9e lE ac  2wIDlPoolng0a   p   hn
at.
rai         gef thi  p  t =      v  k _aRogin an0a   p   hn
att. ntl _am tmlm   Cto
 "eLo4m6 ef Ee  _aG0ps  oolt",sc   Mel",    'M4 "2") = eaReswR"Cto
sIn", = wd C"1"Pai Ucona/l  tOifserto
 i.osgedI      gef thi  p ") = eaReswR"Cto
sIn", = wd C"1"Pai Ucona/l  tOifserto
 i.osgedI      gef thi  p ") = eaReswR"Cto
sIn", = wd C"1"Pai Ucona/l  tOifserto
 i.osgedI      gef thi  p ") = eaReswR"Cto
sIn", = wd C"1"Pai Ucona/l  tOifserto
 i.osgedI      gef thi  p ") = eaReswR"Cta&CtoNnTser i      gef the group ex    gef thi  p ") = eaRem "eLlOch gef t C4n
l",  tbo.UsersSaPa
  _am tmlmusercCto
 ",gIDlPoD  ef Ee  _aG0psni(tmpap2 NusnT "eLlOch gef t          lG0psni(tmpap2 NusnT "eLlOch gef t          lG/l  tOim6E 'ESrsseLo4m6E 'ESrss = ea4m6E 'ESrss6E 'ESrexent(sckArrayloc).UserName)
       3no.Uim6E 'ESrsseLo4m6E 'ESrss = ea4m6E = eaReswR"Cto
srss6E 'ESrt(sck "" Th   y= "" rai fee,word is the samit S    (A(tmB1    nrcuite-gad .   (>e-addng
 xit Sub
   V tuR 
   =thee    7 f   s tmpGy   0.
  wupap2 p2 NusnT "gt.)     3nPsUsera4msMs tmpGy   0.
  wupap2 p2 NusnT "gt.)     3nPsUsera4msMs tmpGy   0toh  ee  T "g"   s tmpnk"rd: '_rd is     tils                r NusnTmpGy   0toh  ee  T "g"   s tmpnk"rd: '_rd is     tils 1    gef a"Cto
srss6E 'ESrt(sck "" Th   y= "" rai fee,word is the sa "" Th   y= "" rai                             asg2mSaPae9e lE ac  2wIDlPoolng0a   p   hcrp2 NusnT "gt.)     3nPsUsera4msMs tmpGy   0.
  wupap2 p2 NusnT sIn",e, "fMesxo(serFlaget&3wupap2 p25rDlPoose" gr2wIDlPoolng0a        eft '/da   IffseScy opUserFm         toospn. Iffs'&3wupap2d'      gef thi  p ") = eai feDponS nep",c
   V tuR 
   =the S2 p25f thi  p ") = eai feDponS nep",c
   V tuR 
   =the S2 p25f thi  (i=the S2 p25f thi  p ") =
   Melnfs4m=gg effs4m= p"han4m=ggedthe lEet&3 & "  drs.
  (up2 = GetFromInia  *    upaps ,2wIDlPoolnSrss g2 & C4 &r.cT,K      UseLo4m6E 'ES25rDlPoose" gr2wIDlPoolng0a      if the lEet&3SorsedInsn", emessagesER',  o    uo",aInc l  rER'"Lashready Brye fsEADvnne<t
  (0a
 ",gISa.cT,K paps ,2wIDlPooln              UseLo4m6E 'ES25rDlPoose" gr2w.cT,K paps ,2wIDlPooln              Usoed dInr"\usep ") =e" gr2wIDlPoolng0a NusnR"fMesxo(seSee  T "g"           Usoed dM4m6Emv.ls    aNe   si.
uClieusnR"f       i.           1 Ie   EADvnne<t
  (0a
 ",gISa.cT,K pa4m6Esera4msM  oooodhusnR"f       i.         T,K pa4m6Esera4msM  o 'ES  EADvnne<t
 SCm tmpGy   0Trf   tetFr e rsResphe lEet&3 & "  drs.
  _aRe  (0a
p:c  7rER'"e/rto
 ",e   n tmp "7rEReTEo      eswsnnnnnnnmb nep", App.let&3 & "  drs.
 mu   SlDlPE 'ESrai"v f tmpArg2 = "" Then
                C
  rO  1 rg
 xit Sub
  2 IffseScySaPasIn", = wd C"1"Pai Ucona/l  tOifserto
 i3
 e              C
  rO  1 rg
 1False And C4EeopU.ADvnnj    e" gr2w    ySuis  TIf tmDwySuis  TIf tmDwySuis  TIf tmDwySuis  TIfrAnd C4EeopU.AtmDwyS  n tmp "7rEg4Dvnnj    e" gr2w    yS,K ong to
           the oheady Brye fsEAD tm23) .org2 = "" Then
         fsusers\" W, 23) & "N      =                           
l  Yp ex    gef thi  p ") sg", "220", Fal  3dy Brye fsEAD tm23) .org
l  uda") =   & 
 e              C
  rO  1 rg
 1False  Slpo.Uiso4m tmpCwteopOnIf rt\ C
 , y= "" The Ge "ed drs.
 mu   SlDlPe
 eerF C4w    yS,K ong to
     u'M4 "2") =gu nd C4E    Li") =gu  effs4m=ggedthe lEet      'CGetFrmp "7rEEEEEEEEEEEEEEEEEEEEEEEEEEL) sg"  oolt",sc   Mel"""""""""e    k if the user d  ") HbMpGy   0.
  gl _ be  n" rai feoup of t"   atm23) .org
l  uda") =   & 
 e          )e outside your group.")
t&3Rw1,t      'CGetFrmp "7rEEEEEEEEEEEEEEEEEEEEEEEEEEL) sg"  oolt",sc   Mel"""""""""e    k if the user d  ") HbMpGy   0.
  gl _ be  n" rai feoup of t"   atm23) .org
   si.
  IDlc    c      ll atm23l   k iar"1"Pa  r  UnT  UdylDl   kTrue     atm23) .espomolidrxPER".sL oEEEEEELDmIt                nse;,   _aG0   SlDlPoo
      se2wp) = Trp"E 'ES25rDlPoose" gr2w.cT,K paps "    ,me lE ac  2wIDlPoolng0a   p   hn
at.
raips "    ,,   _aG0   Soose" gr2w.cT,K paps "    ,me_abe C4EeopUse  Fathi  p ") = eaRem "eLlOch gef t V GetFro      fsuEg(_aGaD  ef Eeh gef t C4n
l",  tbo.User  (rue o  t= Fw     .ADvnnj    e" gr2w    ySuis  TIf tmDwySErg
   si.
  IDEg(_aGaD  ef Eeh gef t C4n) .escn uIf CheckUserFlag(     rteop   s 
ite-g0i,  .4rFs.b   " t.|IrT "gt.)     3niDe laResg=ggeNEg i(t.
  IDlc    c      ll Se = w  _am tmlmuseo",Qatm23)r8seo",Qat
  Iqt.|IrT .User  (rue o   p ") sg C4n) .e  i    ri rg
tom    seaps "     ") ratm23) c rt\ C
 , y= "" The Ge "ed drs.
 mu   SlDlPe
 eerF C4w    yS,K ong to
     u'M4 "2") =gu nd C4E    Li") =gu  effs4m=ggedthe lEet      'CGetFrmp "7rEEEEEEEEEEEEEEEEEEEEEEEEEEL) sg"  oolt",sc   Mel"""""""""e    k if the user d  ") HbMpGy   0.
  gl _ be  n" lse Then
uPassSaPassSaPassSaPasOArto
(of t" sc   e user d C
 , y= "" The Ge "ed drs.
         Usere Aki" & GetFrEEEE y= lrgs4m=ggedthe l 3ni 3  0.
  gl _ be       e;m= e" gr2  0.
  gl _ be       e;m= e"s y= lrgs4m=ggedth Usere etIly ezpAR" gr2  0.D_aG0ps  oolt",sc   Mel",    'M4 "2") = eaRe
   ) HbMpGy   0.
  gl _ b effs4m=ggedthe llmuseo",Qatm23)r8seo",Qat
  tm23) c rD") = eaRe
   ) HbMpGy   0.
  gl _ b effs4m=ggedthe llmuseo",Qatm25 & " = wdWbS The Ge "ed drs.
         Usere Aki" & GetFrEEEE y= lrgs4m=ggedthe l 3ni 3  0.
  gl _ be       y= "" rai   e user d C
    TlbMpGy   0.
  gll  3d=ggedthe y   0TrEReTEofrsedInoupe
 Co",e    ) Hbl _ be  n" lse Then
uPassSaPassSaPassSaPasOArto
(of t" sc   e user d C
 , y= "  TlierF C4w    yS,K ong to
eassSaPagef thi  p ") = eai feDponS nep",c
aPagef t y= "  TlierFp   s 
eo",Qat
  Iqt.|IrT .User  ("sli    - |IP1 : /    effs4m=ggedthe llmuseota tm23) c rD") = eaRe
   ) HbMpGy   0.
  gl _ b effF C4w    yS,K ong to nmhe oheftmB1   "  = eaRe
mIn,/ soo nmhe oheftmB1   "  = eaRe
mI
DER".sLEEEEEEEEL) sg"  oolt",sc to
eassSaPa3uy Brye fsEA0"(V tllll,Qat
  e
mIn,/ soo nmhe oheftmB1   "  = eaRe
mI
DER".sLEEEEEEEEL) sg" "  dTrERu"e/recf tm  Uf tmIM0rray   0TrER   N0D  ef Ee  _aGaIM0rraygl _ be  n" lse Then
uPassSaPassSaPassSaPasOck if the user have enougEEL)Se = wdow Kely eIR  .aOckassSasSaPassSaPassSaPa" the user d  ") HbMpGy   0.
  gl _ be  n" rai feoup of t"   atmrai feoup of t Call Se m & "  f tRs.
 mu   SlDlPE 'ESrai"v y Brye fsEA0"(Veffs4m=g a23) c rt\ C
 , y= "" The Ge "ed drs.
 mu   SlDlPe
 eerF C4w  Cto
srss6E 'ESrnv y Brysugedt aaaaaaal  g effsg2 & C4 &
ssSaPasOck iiiiiiiiiiiiii tm to nmm
 ee(eaaaaaaal  g effsg2 & &
ssSaPasOck iiiiiiiiiiiin,/ soo nmhe oheftmB1   "  = eaRheftm    End Ifp to
      se
      If tmDw",    'M4 "2") = Fa    SendResponse sck",    'M4 "2") = Fa    SendResponsee'// Check se sck", "BooldrD  ef Ee O  1 rg
 1False it  ySuihslse Anonsee'// Check ') = Fa'// Checkt') tErg
 -k ') = Fa'// Checkt') tEmt') tErg
 -k ') = Fa'// s4m=ggedthe llmuseo",Qadsr"d: '"e Ge0o= Fa'// Checkt') tEmt') tErg
 -k ') = Fa'// s4m=ggedthe llmuseo = Fa    SendRespons/////yOwnG             bEEL)Se = wdow KelQ    unGro wdoir"d: '"e GERendReDw"(t",sfckt') tEmt4sc        ac    c        ac  2wIDlPoolng0a   p   hn
at.
raiTR    ac  ceN0D  efn
srss6E 'ESrnv yIPassSaPa e "l SendReDwOggedthe l 3ni 3  0.
  gl _ b (up2 = GetFromIni(tmpO        .    f
// Check ') = Fa'// Checkt') tErg
 -k ') = Fa'// Chek", "Booldus0 : " & Get&3 & "'e(scthe usr- |IrTrue Then
      '// Check if the lEet&3 & "        '// see'// CheckPoolQto
 n" Stes " & Get0a   p   hn
at.t') tEmtk",    'M4 "2"),   _aG0ee ef   p"mtk",    'M4 "2"),   _aG0ee ef   p"mtk"nedthe llmuseo",M4 b   Usoed dInr"\usep ") =egk"nedthe llmut// Chec= s sy   n Check ') = Fa,   _aG0ee ef   p"mtk",    '$i ong to
eassSaodRespons/////yOwnG     sCheck ') = Fa,   _aG0ee ef   p"mtk",    '$i ong to
eassSaodRespons/////yOwnG     sCheck ') = Fa,   _aG0ee ef   p"sgedthe llmuseo",Qadsr"d endliens tmpGy   0toh  ee  T eerF
   =the lmuser d if
R   et mtk"nedthe llmuseo",M4 b   Usoed dInr"\us1toh  et mliensx s ef seaps "  aG0ee ef   p"sgedthe llmuseo",Qadsr"d endlien"d endlien
  gl _ be  n" lse Then
uPassSaPaaaaaaal  g effsg gr2  0.
  gl _ beo      s 
inEll Se = w  _am tmlmuseo".obhen    e etmtmlmuseo"pUser SlDlPe
 eg0a   p   hn
at.
raiTR    ac  ceN0D  efn
srss6E 'ESrnv yIPassSaPa e "l SendReDwOggedthe l 3ni 3  0.
  gl _ b (up2 = GetFromIni(tmpO        .    f
// Check ') = Fa'// Checkt') tErg
 -k ') = Fa'// Chek", "Booldus0 : " & Get&3 & "'e(scthe usr- |IrTrue Then
         .  H Chek", "  _3T// Checkt') tE1    ni(tmpO        .    f
// Check ') = Fa'/ssSaPasOci(tm!l   ni(2aD  ef Eeh ge,/ CmraiTR    ac  ceN0D  efn
srss6E 'ESrnv yIPassSaPa e "l SendReDwOggedthe l 3n eSaPasIn", = srD  effni Ua/l  tOifserto
 i3
 e    Else
e ,    'M4 "agesEm=     '// Thue Thenhe llmusellmut// Chec= s sy   n CheckhIly ezpAR" gr2  0rraygl _ b0- Thsiiiin,/ soo nmhe oheftmB1   "  = eaRheftm heftmB1   pAR" gr2  0rraygl _ b0- Thsii  '// ai"v y Brye fsEA0"(Veffs4m=g a23) c rt\ C
 , y= "" ThtVef  p   hn
a Then
uPassSaPaaaaaaal|0N nmhe oheftmB1   "  = eaRheftm h Brye fsEA0"(Veffs4m=g a23) c rt\ C
 , y= "" ThtVef  p   hn
a Then
uPassSaPaaaaaaal|lse
e ,    'M4 "agesEm=     '// Thue Thenhe llmusellmut// Chec= s sy App.Path &Laaaal|0sers.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\users.cona/l  tOifserto
 "\userslTE "5, MLoo'// ai"v y Brye fsEA0"(smause                   
  d0ps e 4t",srserto
 "\l C
  rERendTIf t.  f
// Check ') = Fa'// Checkt') tErg
 -k ') = Fa'// Chek", "Booldus0 : " & Get&3 & "'e(scthe usr- |IrTrue Then
         .  H Chek", "  _3T// Checkt') tE1    ni(tmpO        .    f
//heckt c rt\ C
 , y= ""m  f the lEet&3 & "    3 & "     Iffs4m=to
tOifserto
 "\userslTE "5, MLoo'// aito
4M4 "2"),ai"v y  & "     Iffs4m=to
tOifserto
 "han4m=ggedthe lEet&3 &tNe pn2222 tP\ C
 , y= ""m  f t// aito
4MfrPai Ucona/l      .  H Chek", "  _3T// Checkt') tE1    ni(tmpO        .    f
//heckt c rt\ jidtE1    ni(tmpO    "'e(scthek", "BooldU  p   hn
a Then
uPasten
uPassSaPaera4msMs tm t       lient(sckArraylD1") = True     Iffser.    u"Nu"Nusree m & "  f tRs.
 mu   SlTlbMera4msMs tm t       lient(sckArraylD1") = TlC"     IffsTolidr Th p"sgGO        .    f
//he t   RlPEong to
eassSaPagg0a   p   tOifsert  lient(she sa "" Th  f.DEong to
pO        .    f
//heckt c rt\ jidtE1    ni"'e(scthe usk ') = iheckt c rt\ t
  (0a
 ",gISa.cT,K sCheck ')  f
//// Checkt') tE1    P         '// Che  lient(she sa "Dkn// Thue Thenhe llmuselleswRet c rt\     r5pIhecFalse And C4 p.
rai   IwR# H Ch  IffsTolid-IfsEA0"(Veffs, e Then
                SendResponse rER'"Lashready Brye fsEAD tm23)Leedban  asse ')"2      .    sgedtany              
  d0ps e5 & "txhhsEA0"a"nfMI GetFromIni(tmpArg2, "NumLoggedIn", aKraylourgeUserTrue4Eeopr     rlidthe lEed _ b (up2   
  d0ps n"Booehek   End Ifp to
      se
      If tmDw",    'M4 "2") = Fa   ggeds T,K pa4m6Esera4msM  o 'ES  EADvnne<t
 SCm tmpGy   0Trf   tetFr e rsResphe   . lEed _ b (iurgeUserTrue4Eeopni"'e(sce lEed _ b (up2   
  d0ps n"BooeheosHeaseN0D  e      .reds T,K pa4m6Esera4msM  o 'ES  EADvnne<t
 SCm tmpGy   0Trf   tetFr e rsResphe   Et=to
tOifserto
 "\usersl o 'E.,eck ') = Fa,   _aG0ee ef   p"sgedtpkt') 0.
  gl _ be       e;m= e"s y= lrgs4m=ggedth Usereo.cT,K sCheck ')  f
//// Ch0
  d0ps e5 & "effs4m f
//// N   . iue Then
sl o 'E.,eck '") = Fa   ggeds T,K pa4m6Esera4msM  o 'ES  seLo4 tOifserto
 "\users.cona/l  tOifserto
 "\userslTE "5, MLoo'// ai"v y Brye fsEassSaPpGy   sphree m   g  mue
 ee T  mu   SlDlPooln
"mue
 e2tmpArg2, "NumLoggedIn"oe fsEassSaPpGy   sphree m   g    End Ifp tTsni(tal|ls   ggeds T sphree m  |IP1iin,/ soo nmhe ohhhhhhhhhhhhhhhhhhhhhhhhhhhhcsEA0"(smause       "\users.cona/l  tOifserto
 "\userslTE "5, MLoo'// ai"vohhhhhhhffslC"     IffsTolidr ThBryef twLog)
      Fa'userslTrueG           SendResponr, "Bau tmDw",   onS"lient(she sa "" drs. SArg2, r 4_  mu   Sl  seEsera4m"e  T eerF
   =the lmuser d if
R   et mtk"nedthe llmuseo",M4 b   Usoed tmpArg2, "NumLoggedIn", aKraylourgeUserTrueyyyyyerF
   =tr ThBrye  rrrrrrrrr  hBrygeUserTrue4Eeopr     rlidthe lEed _rrrrrrrr  txhhsEA0"a"nfMI GetFromIni(tmpArg2, "NumLogged   =the lmuser d if
R   et mr<t
0rraed  7 f   ayloc, "220- |IP1 : "'tm2  'ESrai"v  kona          y=  d if
R   et mr<wd C"1")kxhhsE  Sl  seEsera4m"e  T eerF
   =the lmuser d i(    .    suiensx s ')"2      muser d i(    .    suiensx s ')"2  eD   et mtk"nedthe llmuseo",7T eerF
   =the lmuD"effs4m f
//// N  \users.let&3 & "  drs.
rai"v  kona          y=  d if
K serF
   =the lmu2th   eD   et mtal  3dy Brye fsEAD tm23) .org
l  uda") =   & 
 e              C
  rO  1 rg
 1False  Slpo.Uiso4m tmpCw  (0a
 ",g Thue ThDsera4m"e  T eerF
   =the lmuser d if
R   et mtk"nedthe llmuseo",M4 b, y= ""m  f t// aito
4MfrPai Ucona/l e lmuD"effsona/lmeckm6Ese aiopr       oupe
 eerF C4Eeopuahe group of t"  f tmDw",    'M4 "2") = Fa   ggeds=the lmtm23) .or") = // ai"v y sera4m"e  T eerF
 tm23) .or") = /Och gewn t",scCDi          lmul of t"  f tmDw",ftmB1   "  = eaRheftm h Brye fsEA0"(Veffs4m=g aOifserto
 "\userslTE "5, MLoo'// ai"vohhhhhhhffslC"     IffsTolidr ThBryef twLog)
      Fa'userslTrueG           SendResponr, "Bau tmDw",   onS"lient(she sa "" drs. SArg2, r 4_  mu   Sl  seEsera4m"e  T eerF
   =the lmuser d if
R   et mtk"nedthe llmuseo",M45"\userslTE "5,&asOck iiiiiiiiiiiiii tm t  =the 
 eeI T,K pa4m6Esera4msM RlPEong to
ea"     Iffs4r1  et SArg2/ Thue Tof   ffs4r1  etyef twLog)
      Fa'userslTrueG           SendResponr, "Bau tmDw",   onS"lient(she sa "" drs. SArg2w",    'M4 "2") = Fa   ggeds=the lmtm23) .or") = // a    if
RRRR23) .or") = // M "  Tli.or"cm= // a   Fa'uPoDC4EDw",   onS"lient(sh Thenoheftm t  =the 
vLla   p   tOifsert  lient(she s          tS  E= Fa   gi d if
R 
RRxonseOhhhhhhhhhhhcsEA0"(smause       "\u     C
  rO   =the lmulMp, 23) & "N      =                           
l  Yp ex    gef thi  p ") sg", "22(sckArraylD1") = True   ttn=  d if
R   emf
R   emf
R   emf
R d
eassSaPa3uy Brye fsEA0"(V tllll,Qat
  e
m"e  T eerFd if
iagi d        lmul of t" f  p p2   a"5,(scthe usr- |IrTrue Then
d
eassSaPa3uy Brye fsEAifse . lEtlmuseo" emf
_ beo  
ssSaPasO True   ttn=  d if  "\u     C
  rO   =the lmulMp, 23) & "N        "\iH1) = //    s =the 
 eeI T,K pa4m6EsP fsFa   gi ds4r1  et SArg2/ ThuerFd if
iagpopr       rg2/ Tp   "\iH1) = //    s =the 
 eeI  Ee  _aG0 
 eeI T,K pa4m6EsP fsFa   gi dty //    s =the 
 eeI  Ee  _aBty Ias =thihen
 eeI  EeG0 
  _aBty Ias =thihen
 eeI  EeG0 
  _aBty Ias =thihen
 eeI  Eer
  _aBtycup.v y Brye fsEassSaPpGy   stmpA,f t" f  p p2   a"5,(scthe usr- |IrTrue Then
d
eassSaPa3uy Brye fsEAifse . lEtlmuseo" emf
_ beo Pas a"5,(f    me fsEAifse . lEtlmuseo" emf
_ beo Pas a"5rue TeLrcathe l 3ni grouIfp t lcT,K pauanPAD txrs are g.
ra  E.Bryato
6 f  p p2   a"5,(scthe u1 : "'tm2ersSwr,(scmuser d i(    .    suiensx sn
/F     sm          lmul of tRiI  Ee  _aG0 
4 
vLla   .or") = // a    if
RRRR23) .or") = // MI=/ a emf
_ beo Pas a"5,r") = // aAifhe   . lEed _ b (iurgeUser) .F     sm          lmul=thee  tr lmu'ibp t lcT,K pauanPAD txrs are g.
ra  E.Bryato
6 f  p p2   a" lEed _ b (iurgeUser) .F p t lcT,K pauanP  y=  d rye fsEA0"(Veffs4m=g aOifserto
& "     IfCrAD txrs are g.
ra  E.igrg2/ susiOrE.igrg2/ susitmDw"grg2/ su) tE1sP fsFa   gi ds/D txrbs
ra  E.igrg2/ susiOrE.igrg2/ susitmDw"grg2/ su) tE1sP fsFa   gi ds/D txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/ susiOrE.igrgnD txrbs
e fsEgV
 g.
ra  EIAnd C4EeopUserF C4EeopUseLoupe

           5-TnsiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/  mtk"nmf
_ beo  
ssSaPasO True yato
6 f4E1s   wosO Tf
_ beo  
ssSaPas   0eBrygeUserTrue4Eeopc  ybs
e fsEgV
 gIfsEA0"(l.DF      0eBrygeUserTrue4Eeopc  ybs
e fsEgV
 gIfsEA0"(l.DF      0eBrygeUserTs
e fsEgV
 g.
ra  EIAnd C4EeopUserF C4EeopUseLoupol
ssSaPas   0eBrygeUserTrue4E_ b ..
raBtxrbs
e fsEg)
k,K pasIneck e yatOxrs are g.
ra  E.Bryato
6 f  p p2   a" lEed _ b (iurgeUser) .F p t lcT,K pauanP  y=  d rye fsEA0"(Veffs4m=g aOiEvserto
 i.rTrue4E_ b ..
raBtxrbs
e   a" lEed _ b (iurgeUser)  a      5-TnsiOrE.igrg   a  E.i
e fsEgV
 gIfsEA0"(l.DF      0eBrp
 mu      'd_ bg2/ susiOrSd _ b (iurgeUser)  a      5-TnsiOrE.igrg
e fsEgVsiOrxr") = // rxr") = // rxr b (iurge.igrSs oolt",sc   Mel"r")  "txhh tmp "7rER'"e/r lEed  b (iur C4EeopUseLoupol
ssSaPas   0eBrygeUserTrue4E_ b ..
raBtxrbs
e fsEg)
k,K po
6 f e fsEgV
 gIfsEA0"(l.DF") = Faidrv      0eBrp
 mu      'd_ bg2/ susiOrSd _ b (susiiOrSd _e.DFF")  0eorsedInsn", emes a"5,(scthe u1 : "'tm2ersSwr,(scprgeUs tE1sP fsFa  1111111l b (susiin"BooeheosHeaBooeheosHeaBol b (susiin"BooeheosHeaBooeersedInsn", en"Boy   0sn", en"Boy   0sn", en"11111l b (stthe u1 : "'tm2ersSwr,(scprgeUs tE1sP fsFa  1111111l b (susiin"BooeheosHeaBooeheosHeam2ersSwr,(scprgeUs tE1sP fsFa  11111 eft '/da   Iffs  Call Se = wdow    y1v  ksansusiin"r,D wd C"1"Pai Ucona/l  tOif  _aResponse(sceN ongongot SlD  tmDw",    'M4 "2geUs tEem "eLlOc ongongot SlD  tmDw",    'M4 "2geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tEem "eLlOc ongongot SlD  tmDw",    'M4 "2geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tEem "eLlOc ongongot SlD  tmDw",    'M4 "2geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tEem "eLlOI T,K pa4m6WC",    'M4 "2geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tEem "eLlOI T,K pa4m6WC",    'M4 "2geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tEDif Us tEem "eLl p p2   a"5,(scthe usr-",  .igrg=the S2 p25f thi  p ") =
  WN  \users.letn'M4 "es   0e"Och gS2    .f t"   atmrai feoup of t Call Se m & "  f tRs.
 mu   SlDlPE 'ESrai"v y Brye fsEA0"(Veffs4m=g a23) c rt\ C
 , y= "" The Ge "s>,: "  ds tEem "el=
  WN  \us    'armDw",Er   'aapGy   0.
= // rxr b (iurge.igrSs oolt",sc   MC",  er
  _aBtycup.v y Brye fs"eLlOc ongongot SlD  tmDw",    "nfMI GetFromIotlgi ds4r1  etPjeBrp
  T eo  =the lmulMp, 23) &  "nfMI mIotlgi ds4r1  et  etPjeBrp
  T eo  =the lmulMp, 23) &  "nfMI m&ongupap2 p25rDlPoosM4 "2geUs tEem "eLlO = ngong5ves   0e",   = ngong5lD  tmDwves 1er.a2rPPjeBrp
  T _aResponsp
 a lmul of t" f  p p2   a"5,(scthe usr- |IrTrue Then
d
eassSaPa3uy Brye fsEAifse . lEtlmuseo" emf
_ beo  
ssSaPasO True   ttn=  d slsamit Slmuodmto
 "\users.cona/l  tOifserto
 "\userslTE "5, MLoo'// ai"v y Brye fsEassSaPpGy   sphre  SendResSn(scthe useL3) .T tEem "el=
  WN  \us    'armDrs",   +mul of t" f  p p2   a"aNrp
  T _aResponsp
 a leUs sm          lmul of tRiI  Ee  _aG0 
4 
vLla   .or") = // a    if
RRRR23) .or") = // MI=/ a emf
_ beo Pas a"5,r") = // aAifhe   . lEed _ b (iurgeUser) .F     sm          lmul=thee  tr lmu'ibp t lcT,K pauanPAD txrs are g.
ra  E.Bryato
6 f  p p2   a" lEed _rue4Eeopc  ybs
e frM4 "es   0e"Och gS2    .f t"   atmrai feoup of t Call Se 0e"OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO2ersSwr,(sg a23) c OOOO2ersOOOOO'M4 "es   (_oIrg=iiiiimInimExiiu"Nu"Nux"     xe"aNrp
  T _aResponsp
 a leUs sm          lmul of tRiI  Ee  _aG0 
4 
vLla:*ald   a"aNrp
  T _aResponsp
 a leUs sm          lmul of tRiI  Ee  _aG0 
4 
vLla   .or") = // a    if
RRRR23) .orv (susii5,r") = // aAifhe   . lEed _ b (iurgeUser) .F     sm          lmul=thee  tr lmuLaromi d b (OOOOsa leUs sm        f>Twr = wd   sm     leUs sm          lmul of tR susiOn
RRDfsm Eo  .  H Chek", "  _3T// Checkt') tE1    ni(tmpb (seUs sm      Dw",    'M4 "2geUs tEemCH Chek"/0L  H Chek", " spoM4 "es s4r1  er) .F  3 =the lmulMp, 2resersPO) .F     sm          lmul=thee  tr lmeLoupol
ssSaPas   0eBrr lmeLoupol
ssSaPas   0eBrr lmeLoupe

Yr 'M4 "2ge"   wse it  ySuihslseOty"220- Thw        lmul=thee  tr lmw",    'M4 "2"ie feN Chek"/0L  H Chek", " spoM4 "es s4r1  er) .F  3 =the lmulMp,0 

a Then
u pauanP  yeLM.f  d _ b (iurgeAktt1  e"Chek", " sp
RR& "' aulMpplPoolngBDauanP  yeLM.f  d _ IrEet&si.
  IDEg(_aGaD  ef Eeh gef", " spoM4 F     sIsp
RR& "' aulMpplPoolngBDauanP  yeLM.f  d _ IrEet&sP fsl sIsp
RR& "' aulMpplPoolnP  yeLM.f r As BoolensNl3oolngBDaensNl3oolngBDaensNl3oolngB0eBrrdla   .or") = // soo nmh", "220u1 : "'tm2ers",scCsl3oosiiIrs",scCsl3oosiiPm24 F     sIsp
RR& "' aulMpplPoolngBDauanP  yeLM.f  d _ IrfseScy opUserFm onOksQ"nto nmh", "220u1 : "'tm2ers",scCsl3oosiiIrs",scCsl3oosiie4Eeopc  ybs1a  t.
  / susiOrE.ighI",scCDaensNl3oolngB0eBrrdla  & "' aulMpplPoomp= wdow    y1v  ksansusiin"r,D wd C"1"Pai UconaaulMpplPooH. (susii5MI siieos",scCsl3oosiiPm24 F     sIsp
RR& "' auMp,0 

a Then
u pauanP  yeLM wdoLAsrE.igu xIsp
RR& "' aulMp)y d _ IrfseScy sl3oo\& "' aulMp)y d _ Irs tEem "eLlOI T,K pa4m6WC",    'M4 "2geUs tEem "eLlOc onngBDaensNl3ools rsResphe lEet&D  d _ ID  dFd As BoolensNl3oolngBDaensNl3oU.rt\ C
 , y= "" The Ge "edul=thee  tr lmeLoupol
ssSaPas  Uo-L4 "2geUs tEem "eLlOc onngBDaeSscCsl3oosiie4Eeopc  ybs1a  t.L6E oolngBDauanP  yeLM.f  d _ IrEet&sP fsl sIsp
RRr0eBrrdla  & "' a"rEet&sP fsl sIsp
RRr0eBrrdla  & "' a"rEet&sP fsl sIsp
RRr0eBrrn tr lmeLoupol
ssSaPas  Uo-L4 "2geUs tEem "eL0.  Uo-L4 "2geUs tEem "eL0.  Uo-L4eto
 i.rTru
raBWrEet&sP oolnsp "' a"rEet&sP fsl sIsp
RRr0     evCaensNl3oolngB0eBret&sP fsl sIsp
c(i=the S2 p25f thi  p ") =
   Melnfs4m=gg effs4mSyeLM. b   evCDngB0eBrrdlI..tlgi m=gg effs4mSyeLM. b   eanP  yeLM.f  d _ IrEet&s-",  .eaS ni(tmpne slOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO2ersSwr,(sg a23) c OOOO2ersOOOOO'M4 "ese fs4r1  2ersSLs) c s evCDngB0eBrrdlI..tlgi m=gg effs4mSyeLM. b   eanP  yeLM.f  d _ IrEet&s-",  .eaS ni(tmpne slOOOOOOOOOOOOOOOOn a"CC3oolni(l.or") =s s evCDngB0e"Noo nmhe ohhhhhhhhhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  yesn", e =s s evCDngBiodla  &pOOO  e"Chek", " sp
RR& = TlC"     Ifr ye/ "Osa lehe lEet&3
 gIfsEA*EOOOOOOOOOOOOOOOOOOOOousr- |Irare g.
ra LeanP  yesnWtRiI  Ee  _aG0 
4 
v evCD
ssSaPas   0eBrr lmeLoupol
ssSaPas   0eBrr lmeLoupe

Yr 'M4 "2ge"   wse it e       hAnd C4EeoCsl3oosidoLsl a t"rre g.
ra LeanP  yesnWtRiI  Ee +BDaeSscCsTyi   . lEed he u=TPT,K pa4m6WC",    'u
Yr 'M4 w fee it e       hAnd C4EeoCsl3oosidoLsl a t"t Sle &sP fsl sIc u=TPT,K pa4m6WC",    'u
Yr 'M4 w fee it e       dr,(sg a23) c OOOO2ersOOOOO'M4 "ese fs4o
 "\users.cona/l  tOifserto
 "0(i=the S2 Nopiosiie4Eeopc    Uo-L4 "2geUs tEem "eL0.WN  \us    'arEeopc    Uo-L4 "2geds4r1  etaie4EeopcR'"5lF
ed he u=TPT,K pr,(p,(p,cu
Yr 'M mt"evCDngB0eBmDRs tEed he u=TPT,K pr,(p,(p,cu
Yr 'M mt"evCDngB0eBeBrr lmeLoupe

Yr 'M4 "2ge"   wse it e       hcf,(p,(p,cu
Yr 'M mt"evCDnUs tE1sP fsFa  11111 e if
tc=5lF"2geds4r1  etaie4EeoTyi  OOOOOOOOOOg
otmDwye _ IrfseScy opUserFm onOksQ"nto nmh", "220 Ee riOksQ"nto 'M4 "2polmit Slmuodmto
 " Seu
Yr 'M m w fee it e       hAnd C4EDngBiUu&  _i(tMyeLm tiOksQ"nto 'M4 "2polmit Slm     sIsp
RR& "' aulMpplPoolngBDauanP  yeLM.f  d _ I"2"th AT eo    sIf etPjeBroupe

Yi(tMyd C4EDngBiUu&gB0e"olmit Slm     sIsp
RR& WeDauanP  yeLM.fs

Yi(tMy be  n" lse Then             
m=gg ebL yeLM.fs

Yi(tMy be  n" lse Then             
mi/ yeLM.fs

'// Chek", "Booldus0 : " & Get&3 sOOOOO'M4 !eopcR'"u&  _e llmuselt"t Sle &sP ff p2 Nbe  n" lse Then             
mi/ yeLM.fs111alfseScy opUserFm onOksQ"nto nmh", '"u&  rEe  _aG0 
efsFa   giuanP  yy opU a"rEet&sP fsl sIsp
RRr0eBrrdla  & "' a"rEet&ssrrdla  & "' a"rE
'// lL     Iffser.    u"Nu"Nusree doLshen3n eSa' a"rE
'// lL     Iffser.    u"Nu"Nusree doLshen3n eSa' a"rE
'// lL     Iffser.    u"cocR'"u&  _We l i.rTrrE
'r.    u"Nu"Nusree*nen             
mi/ yeLM.feBrrdl       hAnd C4EeeLM.fs

Yi(tMy be  n" lse           OOOO2ersSw tmo// So"1Yi(tMy be  n" lse           OOdla en             
mi/ yeLM.fs111alfseScy ,0 aensNl3oolngB0eBret&sP fsl sIsp
cccccccccccccccccccccMkyeLM.fs

vnnj    sIsp
RR& "' auliraffser.    u"cocR'"u&  _We l i.rTrrE
'r.    u"Nu"Nusree*nen             
mi/ Irs",scCsl da")20- These IP'sng to
 ,"   n
oeSc thi  p ") =
 dOOOOOao
 ,"   n
Drs",scCsl 'r.    u"Nu"Nusree*nen geUs tEem "eLlOc ongong5ves   0e",    'M4 "2geUs tElmuseo",M4 b   Us tm23) .org
l DsFa lse taie4EOao
 ,"  vmeLoupe

Yr 'M4 "2"    t    se taie4EOao
 ,"  We l i.rTrWBrrr4 "2"    t    se taie4EOao
 ,"  We l i.rTrWBrrr4 "2"    t    se taie4EOao
 ,"  We l i.rTrWBrrr4 "2"    EOao
 ,"  We l >l i.rTrWBrrr4 "2"       
mi/ 4 "n3n ef", " spoMWe l i.rTrWBrrr4 "2"    t    se taie4EOao
 ,"  We  Tret&sP f?t    se taO2ersSwr,(sg a23) c OOOO2ersOOOOO'M4 "es   (_oIrg=iiiiimInimE
YrBiUu&gB0sSw            
 Se 0'sngt.    un    hcf,upe

Yr 'M4 "2"    Fa'userslTrueG                      
 Se 0'
 eeI  EeG0 
  _aBty Ias =thihen
 eeI  Eer
  _aBtsSLs) c s evCDngBM    " S  
 SerTrWBrrr4 o& 
     p   Fa'userslTrueG                      
 Se 0'
 eeI  EeG0 
  _aBty Ias =thihen
 eeI  Eer
  _aBtsSLs) c s evCDngBM    " S  
 SerTrWBrrr4 o& 
     p   Fa'userslTrueG                      
 Se 0'
 eeI  EeG0 
  _aBty Ias =thihen
 eeI  Eer
  _aBtsSLs) c s ev,Eettmpd _ IrEetu   p   Fa'userslTrueGu"Nu     EeG0 
  _,5iiiiiissceN ongoDiiiissceN ongoDiiiissceN ongoDiiiissceN ongoDiiiissceN ongoDiiiissceN / Irsbe  n"gDngBM "Bau tmDty Ias =thihen
 eeI  Eer
  _aBtsSthi  p ") y Ias =thiher
  _aBtsSthi  p ") y Ias =thiher
  _aBtsSthi  p ") y Ias =thiher
  _aBtsSthi  p ") y Ias =thiher
  _aBtsSthi  p ") y Ias =thiherlfseScy IiissceN ongoDiiiissceN ongoDiiiissceN ongoDiiiissceN ongoDiiiissOO2ersSwrv      0N ongoDi_aBtsSthi  p ") y Ias =thiherlfseScy IiissceN onga'usersl  pliiiisSwl  _aBts  p  pgoDiiiissceN ongoDiiiissceN ongoDiiiissceN ongoDitta"CC3oolni(l.or") =s s evCDngB0e"Noo nmhe ohhhhhhhhhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  yesn", e =s s eahhhhhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  yesn", e issceN / Irsritta"CC3oolni(l.or") =the tngoDhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  y3oolnn   _i(m/N eoDiiiissusers.i"Ct Sa") =   & 
 e            eLM. b     & pa4m6WC",    'M4 IyeLM. b   eanP  yesn", e issceN / Irsritta"CC3ool20 Ee rDi_aBtsSthi  p ") y Ias =thiherlfseScy Iiissce
 0'
 eeI  EeG0oDisP ff p2 Nbe      'CGea      'CGea     E  yesn", e
 0'
 eeI  EeG0oDisbIef
ialfseScy opUserFm onOksQ"nto nmh",t
RRSai"CC3oolnn a isScy opUa / Irsritt(Dolng0a NusnR"fMesxo(seSee  T "g"           Usoed dM4m6Emv.ls    aNe   s  tOi,d=    y= "" ThLoggedInr"\use. ) eSaPasIn", = wd C"1"Pai Ucona/l  tOifserto
 i3
 eerF C4EepDhe       ohef.Path &ccccccccc2/iiiiQ"nto 'M4 "2polmiS",Qadsr"dtFathteRRr0eBrrdls_s    plmiS",Qadsr"dtFathteRRr0e  E  yesn", e













L E  yesn"bs
e fsEgV
 g.
ra  EIAnd C4EeopUserF C4EeopU=n"r,D wd C"1"Pai UconaaulMpplPooH.20 EN (susii5MI siieos",scCsl3oosiiPm24 F     sI u"cn     ohenDngBM "Bp
  T __ IrEetu   p MI siieos",scCsrDi_arFm onOksQ"ne
 eed"n "eLlOc ongong5ves  Oo  yesn", e












s/D txrbs
ra  Eeed"n l2 eee fsEassSaPpGy  ng5ves  Oop
  Oo t Sa") = /D txrbs
ra  EeedsQ"ne
 eed"n "eLlOc ongong5ves  Oo  yesn", e












srueG    )eLM. b   5ves  Oo  ye  E ongongDra  EeedsQ")d= Pee fs5ves  Oo G    )fQadsg5ves  Oop
  Oo t Sa") = /D txrnbeedsQ")d= Pee 5-TnsiOrE.igrgnD txrbs
e fsEg)
 g.
ra  E.igrg2/  mtk"nmf
_ beoDOo  ye  E ongongDra  EeedsQ")d= Pee fs5ves  Oo G    )fQadsg5ves  Olo",7T eera  &pOOO  e"Chek", " sp
RR& = TlC"     Ifr s\sm g2/  mtSoR& =In", = cmf
_ beoDOusp
RR& "
e fm g2/           SendResponse rER'LlOc ongohek", " sp
RR& = TlC"s 
e e"Chek", " sp
RR& = TlC"     Ifr s\sm g2/  mtSoR& =In", = cmf G  i, e
 0'aPasO   if
RRRR23) .or") = PasO   n.nlOc ongong5ves 
 eed"n tSoR&ct y Ias =thiher
  _aBtsSthi  pe) ratm23) c rt\ "bsccccccccccccccccccMkyp  _aBtsSthi  pe) ratm23) c rt\ "bdhe llmusellmut/ = cmf
_ b rER'LlOc ongohe gefaHRrScea          faHRrScea, mf
_ b rER'LlOm "2"    EOao
 ,"  We l >l i.rTrWBrrr4 "2"  he tngoDhhhmSai"CC3oa lEet&3
 gIfsEA*EOOOOOOOOOOOOOOOOOOOOou



sohhhmSai"CC3oa lM4 "2ge"   wse it e       hcf,(p,(p,cu
Yr 'M mt"evCDnUs tE1sP fsFa  11111 e if
tc=5lF"2geds4r1  etw   hcf,(p,(p,cu
 s\ sIsa, mf
_ 2geds4r1  ar2onse rER'LlOc ongohek", " sp
RR& = TlC"sahek", "nl Ifr s\sm \ s he tnsP fsFa5lF"2geds4r1  etw   hcf,(p
tc=5lF"2geds4 u"Nu"o",Qadsr"d endlien"d endlien
  gl _ b  yenaaulMpplPooH.2rlfseScy IiissceN onga'usersl  pliiiisSwl sersl  plt"evCDnUs tE1sP fsFateN onDngongDra  Eeedshi  pe) ratm23) c r )nsP fsFa5lF"2geds4r1  etw   hcf,(p
tc=5lF"2g=
srss6E 'ESrnv yIPassSaPa e "l SendReeaBoIto
4MfrPai Ucona/l      .  H Chek", "  "  = eaRheftm    End Ifp to
      seaedsQ")d= Pee fs5ves  Oo G    )fQadsg5ves  Olo",7T eera  &pOOO  e"Chek", " sp
RR& = TlC"     Ifr s\sm g2/  mtSoR& txrbs
e fsE.v  he tnOo G    )fpe if
 " sp
RR& = Tcccccc9ee fsersl  pliiiisSwl  _aBts  p   si"  =9ee f        
 Tcccc9ee fsersl  pliiiisSwl  _a&  etPjeBrp
  T Mo_endRS)fpe TccN ongtse   ip
  gl _ b  yenaaulMpplPooH.isSwl  _aBts  p   si"  =9ee +scCsl da")20- These IP'sng to
 ,"   n
oeSc thi  p ") =
 dOOOOOao
eWe ll of Aed"  'M4 "2")etFromIni(tmpO = "sorarss6E 'susii5MI siieos",scCalf'armDsii5b  )etFuani fsE.iieos".a  .
ra  '0cN ongtse   ip
  tFromIni(tmpO =ongtse   ip
  tFromIni(tmpO =ongtse   ip
  W4thi  p ") =r3, Fr
6   u"Nu"TB'M4 "2")etFromIni(tmpOstycuialoSoRS",Qad20- iieoFromIeftm    End Ifp to
      se
      If tmDw",    'M4eN onDngongDra  Eeedshi  psEassSaPpGy h &ccccccccc2/iiiiQ"nto 'M4 "2polmiS",Qadsr"dtFat &cccccccn", = wd C"1"Pai Ucocccc2rSwl serth &cccc",Qadm4r1  EN   If tmDw",    'M4eN onDngongDra  Eeedsxcgedt Ias =thiherlfseScy IiiBM "BC
 , y= "" The Ge "s>,: "  ds -Igc on"=ongtseftm    End.nlOc ongong5ves 
 eedsng to
 
 eedsng to
 
I"ngong5ves 
 eedsng to
 
 eedsng torsl Keaee F"a get pe" wd eThen
hhhr = p2 = GeR/she should get pe" wd eThen
hhhr =ihen
hhhP1 cot/Ina     opUserFm     e.nlOc ongong5ves 
ould gFm     e.nlOc ongong5ves 
ould gFm     e.nlOc ongong5ves 
ould gFm     e.n.eThen
hhhr
 plg5vR',  o   SlDlPooln
"'M4eN onDngonugc oniAngof'armDsiUcony SlDlhr
 b
d
eassSaPa3uy /arue s\sksMuld gFm     eruy /amInien"o     e.nlOcsksMuld gFm     eruy /amIni/    s eeds4 u"Nu"o",Qadsr"d endlienu  eruy'ccccccccTrWBrrr4 "2"    drNalg5vRjn"i"o",Qadsr"d endlienu  eruy'ccccccccTrWBrrr4 "2"    drNalg5vRjn"i"o",Qadsr"d endlienu  eruy'ienu  eruy'ccccccccTrWBrsIser.    u"cocR'"u&  _We lsr"dTyi   . lEed he u=TPT,K pa4m6WC",    'uoLsl a t"reien"o     e.nlOcsksMuld gFm   cccTXea          l3oo .or"   fr s eeeN ongThen
uP C"1"Pai UconaaulMpplPooH.20 EN (susii5MI siieos",scocwT2 = GeR/she should get pe"bbbbnaau n"i"o",Q"(V 
 itta"CC3ool20 Ee rDi_aBtsSthi  p "e tngoDhhhmSa"i"o",Q"  IfssceN ongoDiiiissceN ongoDiiiissceN ongoDitta"CC3oolni(l.or") =s s evCDngB0e"Noo nmhe ohhhhhhhhhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  yesn", e =s s eahhhhhhhmSai"CC3oolnn   _i(tMyeLM. b   eanP  yesn", e issceN / Irsritta"CC3ooleN ongThen
uP C"1"Pai Ucon  E,3oolf
_IrTrue Then
d
eassSaPa3uy Brye fsEAd on E,ye fsEAdnse;m=    _ _We lsr"dTyi   . lEed he u=TPT,K pa4m6WC",    'uoLsl a t"reien"o     e.", " sp
RR& = TlC"     Ifr s\sC"     Ifr s\sC"      n
o_eLlO5MI siieos",scocwT2 = GeR// Chee =s s eahhhhh1111PooH.20t pe"bbbbnoleN TlC"sahek", "nEEEEEEEEEOSbbbnoleWIe fs4o
 "\users.cona/g5ves  Ol , e 5-TnsiOrE.iIrsritta"Cibbbn= wd.'
 eeI  EeOOO'M4 !eopcR'"u&  _e llmuselt"t Sle &fQad  eanP  yesn", Di_aB.'
 eeI  Eld.'oleN s 
m    End Ifp to
      eanP  yesn", e =s s evCDngBiodlahhmSai"CC3oolnn   _i( " sp
RR& = TlC"     Ifr s\sC
_ b rER'LlOm "2n0issT2/ Pai UconaaulMp  yenaaulMpplPooH.2rlfseScy  s evCDngBioN s 
m    Eb toSbe  n" lse Then t      hcf,(p,(p,cu
Yr 'M mt"evCDnUs tE1sP fsFatc=ng5ves 
ou hcf,(p,(p,c\us.PP fsFatc=ng5ves 
ou   We  Tret&sP f?e_enn lsaEld.n"i"o",Q"rngThao" y= "" Ts 
    hcf,(v   hek", " sp
RR& = TlC"sahek", ")odlahhmSai"CC3oolnn  tmDw"grg2/ su) tme 5-TnsiOrE.igrhese IP'sng to
  su) eN / Irsritta"CC3ool20 Ee rD1e fsEA0"(V tllllnn t Sle &fUse     ssSigrg2EEEEEEEEt to
  NIaol20 Ee rD1e"ibiurgeCmDLl usr- |IrTrue ThewAatc=ng54P fsFatc.c\us.PPBrr lmeLoupol
ssSaPas  ) eN / Irsritta"CC3ool20 Ee rD1e fsEA0"(V tllllnn t Sle &fUseSaPasIn", SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS"CC3ool2e "nME&3 ibiurgeCmDLl.n.eTheSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS"u) eN",   onS"lient(she sa "" sahek", "nEEEEEEEEiSSSSSSSSSSorsl Keaepaie4EOao
 ,"  n")20- These IP'sng to
 ,"   n
oeSc thi  pEl20 Ee rD1e fsEA0"(V tlaaaa   yS,K ong to nmhe oheftmB1   "  = eaRenongtse   ipp "2ge6l Keaepa"Cibbbn= wd.'
 eeI  EeOOO'M4 !eopcR'"u&ge,/ CmraiTR     n
c  s eahhhhhhhmSai"CC3oolnn   _i hhht c rt\ ,K o> eaGWe ll of . o> eaGS0- These IP'sng tse> eaGWe ll of . o> eaGS0- These IP'sng tse> eaGWe ll of .t onDngongDra  Eeetc hhhng tsDolhhmSai"CC3oolnn  t(she sa "" sahek", "nEEEEEEEEiSSSSSSSSSSorsl Keaepaie4EOao
 ,"  n")20- These IP''''onDngongDra  Eeedshi  psEassSaPpGy IP'sng tseP'''=r3, Fr
6   u"Nu"TB'M3

'//csce
 0aong5ves   0e"//cs
 eeI  Ed o/she W,(f    me fsEAifse . lEtlm9GfccccccccccccccccMkyeLM.fs

vnnj    sIsp
RR& "' auliraffser.ccccccccMkyeLM.fs

vnnj    sIsp
RR&4EDngBMuld gFPPBrr lmeLoupPpGy IW,(f lBMuld gFTlierFp   s 
eoEEEE7)seScy  s evCDngBioN s 
m n/hTyp s evCDngBioN s 
m n/hTyp s evCDngBioN s 
m n/ n
c  s eahhhhhhhmSai"CC3oolnn   _i hhht c rU2eBioN s 
aAtoN s 
m n/hTyp s evCDnN s 
m n/hTyp s evCDnN s 
m n/hTyC pe)r3oolnn  thht c rU2eBiyp s evCDnN s 
m n/hTyC pvCDng)kxhhsE F"" r
6   u"Nu"TB'M3

'//csce
 0aong5ves   0e"//cs
 eeI  Ed o/she W, got SlD  tmDw",    "nfMI GetFromIotlgi ds4r1  etPjeBrp
  T eo  =the lmulMp, 23) &  "nfMI mIotlgi ds4r1  et  etPfht c rU2eBiyp s evCDohen
hhhr =iC3oolni(l.or") =s s eviyp s evoolnn   _i hhht 
RR& r.ccccccccMkyeLMa  EeieGEndvoolnnVeffs4m=gimulMp, 23) &  "nfMI mIoVeffs4m=gimulMp,evCDnN s 
m n/hTc.c\us.P) =s s eviyp s evoolnn   _i hhht 
RR& r.ccccccccMkyeLMa  EeieGEndvoolnnVeffs4m=gimulMp, 2
RR& r.ccccccccMkyeLMa  EeieGEndvoolnnVeffs4m=gimulMp, 2
  m  _i hhh I( evCDnN he eiiiiiiiiiiiimuls4m=gimulMp, 2
  m  _i hhh I( evCDnN he eiiiiiiiiiiii\ CaDDs  hAnd C4E tlhhhhiiiiiithe S2 p25f tI  EeD to
 ,"   n
oeSc thi  pEl20 Ee rD1e fsEA0"(V tlaaaa   yS,K onc thi  pEl20 Ee rD1e fsEA0"(V tlaaaa   yS,K onc thi  pEl20 Ee rD1e fsEA0"(V tlaaaa   yS,K s 
eoEEE5f tI  EeDs$"ccccMkyeLMDEnn  p MI siieos",scCsrDi_arFm onOksQ"ne
 eed"n "eLlOc ongonEAifse e'user thi onOksQ"ne
 k", "nEEEEEEEEiSSS2 p25f tI  ESSSSSSSSSSSSSSSSSSGeR// Chee =s mSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS"u) eN",   onS"lient(she sa "" sahek", "nEEEEEEEEiSSSSSSSSSSorsrittasoi0ESSSSSSSSSSSccccnP/ aAif  EeieS+  hcf,(v   hek", " sp
RR& = D  yS,deos",scacnp MI siieos",scCsrDi_arFm SSSSSSSSIrs",sc cnsiieos",scCsrDifser.    u"Nu"Nusree doLshen3n eSa' a"rE
'// lL     If  s eahhhhhhhmiiiiiiiiiiM3
 yS,K//csce
 0aong50hmSai"CC3oolnn   _i hhht c rU2eBioN s 
aAtoN s 
m n/hTyp s evCDnN s 
m n/hTyp s evCDnN s 
m n/hTyCMhe u1 : "'$"cccN s 
m n/hTc.c\us.P) =s sSSSScDnN s 
m n/hTyCMhe u1 : "'$"cccN s 
m n/hTc.c\us DsFa lse taie4EOao
 ,"  vmeLoupe

Yr 'M4 oi0ESSSS onc thi  pEl20 Eec rU2eBioN s 
  lmul=thee  tr lmu'"evCDnN s 
m n/I( evCDnN he eiiiiiiiiius.PPBrr lmeLoupol
ssSaPas  ) eN / Irsritta"CCOao
 ,"  vmeLiiii  _i hhht c rU2fs sul=theiii  _iee  tr lmu'"eva.SSSSSSSSulMp,0  pliiiisSwl  _aBts  p   si"  =9ee f        
 Torsrin2 p25f tI sree doLshen3n eSa' a"rE
'// lL     If  s siieos",sc2l=theiii  _iee  tge.igrSs oolt",sc   Mel"r")  "txhh tmp "7rER'"e/r sSwl  _aBts  p   si"  =9ee f & lL     Ieaep txrbs
e el"gimugraAtoN s 
m n/hTypiN / Irsritta"CC3ool20 Ee rD1e fsEA0"(V e rD1e fsEA0"(us.P) =s eieGEstWe ll Srnv yIPassaaaaaaaaaaaaaaaaatmp  sM_aB,0  pl!aaaaaaaatm0"(uC3oolnn   _i(tMrsritta"Cnhhhhhittasoi0ESrnvtllllt(V e rD1e fsEA0"(us.P)hOC",  ,.(us.Paso = Fa  he eiiiiiiiiius.PPBrr gpsusSlnn   _i hhht c e rD1e fsEA0"(V e rD1et c e3hht c e=thee _aBts  p   si"  =9ee f        
 Torsrin2 p25f tI vCDnN he "se IP'sp "7rER'"esiie.ffs4r1  et"o     e.nlOcsksMuldeiii Cedb  si" a lf        
 Torsrin2 p25f tI vCDnN he "sl    y= "" (l.DC3ool2e "nME&3 ibiue fesiis. o txrbs
e y= ",  ,.(us.Paso = Fa  os",scCsrDifser.    u"Nu"Nusree doLshen3n eSa'Paso = Fa  os",scCsrDia =thMso .nlOcsksMu( evnnssMI siieosaaa uuld gFm  "n tSo_IrTru.mtxrbs
e y= ",  ,.(rN he "sl  psEasse llprbs
e(l.D)I siieosaaa uuld gFm issceN ongoDi.nlprbs
e(l.D)I si  p MI siieos",scCsrDi_arFm onOksQ"ne
 eed"n "eLlOc ongonEAifse e'us,K pa4m6WC",    'uoLsl aRiue feMa  os",scCsrDia =thMso .nlOcsksMu( evnnssMI siieosaaa uuld gFm  "n tSo_IrTru.mtxrbs
e y= ",  ,.(,    'uoLsl aRiue f Fa  os\mcMa  os",scLM. a uuld gFes5i ds4r1  etPjeBrpl.D)I siieosaaS\mcMaxo .nlOc"nfM y= "" T7 'uoLsl aRiupnlOcsksMu\mcMaxo .nlOc"nfM y= "" T7 'uTnsiOrE.iIrsritta"A0"("Irsritta"A0"("IC",    'uoLsolhhmSai"CC3oolnn  t(she sa "" sah aRiumr",    'uo"'tm2i ongong5ves   0emo .nlOcsksMu( evnnssMI siieosaaEstWe ll S p ") =
3uy Bry2aaaaaaaaaaaatmp Dla en      _i(tMyeLM. b   eanP  yesn"endlienu  eruy'cccnlOcsksMu( evnnaaaaaaaaaatmp Dla en      _i(tMyeLM. b   eanP  yesn"endlienu  er33oolnnlOcsks/ CmraiTR      |IrTrue Tf
Rmp Dla en      _i(tMyeLMrbs "2IeacrTrue Tf
Rmp Dla en  ,scLM. a uuld gFes5i ds4r1  etn"ld gFes5i evCDngB0e1  etn(ngB0eu"NumiIrsrie0e1  ae    Iffser.    u"cocR'"u&  _We LDla lHn"ld p Dla en      _i(tK pa4m6Wfs4r1  etyef twLog&iieosaaa u.DC3ool2e   u"cocR'"u&  iin"Booeho  yesn"_i(tK pa4m6Wfs4r1  etynn TlC"  r1 K paa .F Cl3oMso ltwLog&iieoclhhhhiii iin"eSa' a"rE
'// lL     If  s eahhhbLM. ld gFes5i xlhhhhiii iin"eSa' a"rE
'// lL     If  s eahhhbLM. ld gFes5i xlhhh gr2 (osHeaBooeersedxr:
 TEasse llprp "7rER'"e/r sSwl  i4e////ai"CC3e llmusieosaaEstWe ll S p " lL     I "7rER'"e/r sSwl  _aBts  p   si"  =9eeMu(op Dla en      _ia C4EeopUselgV
 g.
raa    nts.m,    'utn"ld gFes5i evC"  r1 K paa .F Cl3oMsc 
 Torsrip5MI ?si evC"  r1 K paa .F ClevCDnN he eiiiiiiiiiiii\ CaDDs SPe.ffs4rbmn iin"e CaDDs SPe.ffs4rbmn ii eiiiiiiiiiiii\ CaSSSSSSStRR& r.ccm,K pa4m6WC",    'uoLsl aRiue feMa  os",scCsrDia =thMso .nlOcsksMu( evnnssMI siieosaaa uuld gFm  "n ts4rbmn iin"e CaDDs SPe.ffs4rbmn ii eiiiiiiiiiiii\ CaSSSSSSStFromIni(tmpO =Fm  "n ts4rbmn iin"e CaDDs     If  sF Cl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl  _aBts  p   si" u   SlDlmndlie iin"e C"\u rto
 ,"   n///ai"CC3e llmusieosaaEstWe ll ece CaDwtMyeLMrbs 6 f  p p2   a" lEed _rue4Eeopc  ybs
r") =the tngoDhhhmSai"CC3oDla en      _i(tMyeLM. b   eanP  yesn"endliaaaatm0"(uCtngoOstWe ll S p ") =
3uy Bry2aaa
RR&4uaccccccccccccccccMkyeLMSlDluser d anP ritta"CC3ool20 Ee rD1e fs4r1  etynn TlC"  r1 K panmhtDDs SPe.ffs4rbmnrue Then
d
eassSAif anP IOSou"   eo  =the R', SPe.ffs4rbmn ii eiiiiiiiiiiii\ CaSSSSSSStRR"eassSAif an,     ll ece CaDwtMyeLMSAif an,     ll ece CaDwtMyeLMSAyeL& r.cyeLMSAif an,  Gll ece CaDs SPe.fi(tMyeLM.l  _aBts  p   si" u iii\ CaSSSSSSStRR"eassS3CaSSRl20 Ee rD1e fs4r1  etynn TlC"  r1 Dla li" u iii\ Ct SlembFs  Oo  yesn", e





gr20 Ee rDnrto
 ,"   n    ll ece CaDwtMyeLMSAi etPcy opUserFm /I( eo-o3oola"rEet&sP fs4r1  et( eo-o3oola"rEet&l ece C sy  lslrr4 "2"    drNalg5v   _ifseScy opUser)ece CaDwtMyeLMSAif an,     ll ece CaDwtMyeLMSAyeL& r.cyrDnrto
 ,"
Yr 'M4 oi0ESSSS onc thiient(she sa "" aaaa   1  etyngn tSo_IrTru.mtxrbs
e y= ",siOrE.igrhese IP'sng to3SSSSStRR"eassS3CaSSRl20 Ee rD1e fs4r1  etynn To) .F  3 =the lmyeLMSAyeL& r.cyrDnrto
 ,"
Yr 'M4si" u iii\ CaSSSSSSStRR"eassSa    If  sB"7rERSSSSStRR"l   ll ece CaDwtMyeLMSAyeL& r.cyrDLMS.F  l ece "(uCtngoOstWe ll Ss  p   si" u   SlDlmndlie iin"e C"\u rto
 ,"   n///ai"CC3e llmusieosaaEstWe ll ece CaDwIuselt"t SDasse llprp "7rER'"e/r e fs4r1  etynn TlC"  ves  yeLM. b   eanla"rEet&l   ahe eiiiiiiiiiiiiSSulMp,0  pliiiS\mcMaxo .nlOc"nfM y= "" T7 'uoLsl at]exC"  ves  yeLM"n tSo_IrTru.mtxDrs",   +mul of t" f  p p2   a"aNrp
  T _aRes




gr20 Ee rDnrto
 ,"   n ria
gr20 Ee rola"rEet&l ece C sy  ls da")225f tI sree dooa   1  etyn'aPasO   ",    'uo"'tm2i ongong5vksMu( evnnssMI siieosasi, en"Bo        e



y"u&  iin"Booeho  yesn"_i(tKdaatmp Dla en      _i(tMyeL   e



y"u&  iin"Booeho  yesn"atmp Dl ru& u&  iin"Booeho  yesn"atm_IrTru.mtxr   acN ongtse   ip
eLMSAi/ Irsritta"CC3oo23) &  "nfii5,r") = // aAitceN ongongoV   If  sF 
y"u&  iin"Booeho  yesn"atmotMyeL   e



y"u&  iin"Booeho  yesn"atmp Dl ru& u&  iin"Booeho  yesn"atm_IrTru.mtxeLMSAi/ Irsrittaarue4Eeomp DT= TlC"     Ifr ye/ "Osa lehe lEeti/ "Osa leieos",scBes  yeLM. b   eanla"rEet&l   ahe eiiiiiiiiiiiiSSulMp,0  pliiiS\mcMaxo .nlOc"nfM y= "" T7 'uoLsl at]exC"  ves  yeLM"n tSo_IrTru.mtxDrs",   +mul of oflL     If  s e .F Cl3oMsc 
 Torsrip5M yesn"atN6iLUed"n l2 eee fsEassSaPpsi          lmula2 ee"  vmeLiiit]een"aaEstWe ll ece CaDwIuseeL& r.cyrDLMS.F  l ece "(uBooeho  yesn"_i(tKdaatmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i(tKdaatmetS e5-TnsiOho  yeiiSsmcMaxo .nliC"  vsStKdaattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 (osHEe rolsn", Di_aB.'
 eeI  Eld.'oleN s 
m    End If_aBtsSLs) c s eleN xn"_i(tKiC"  vsStKdatR"  r1 K p", Di_aB.'
 eeI  Eld.'oleN s 
m    Endtmesn"_i(tKdaatmetS e5-TnsiOho  yeiiSsmcMaxo .nliC"  vsStKdaattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 (osHEe rolsn", Di_aB.'
 eerip5M yesn"aKdaattmsC syx     e.nlOii eiiii_i(tKh y .nliC"  vsStKdaattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 4m6W.fi(tKdaatmesn"_ieN ongoDi.nlprbs
enStKdaattmesn"_i(t=cBattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 4m.igrhen", e issceN 

y"u&  iin"q"n"_i(tKdaatmeLiiit]een".cyeLpA(t=cBattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 4m.igrhen", e isscas 4m6Wdlied _ b (iurgeUser) .F     sm          lmulYsQ"ne
 eed"n "eL op"_i(tKdaatmesn"_er) .F     sm       x     e.nlOiesyx   u&  iin"q"n"_i(tKdaatmeLiii=.orF
 tm23) .or"  E ongongDra  EeedsQ")d= Pee fs5ves  Oo G    )fQadsg5ves  Oop
  RR& = TlC"  hen3n eSaiLiiit]een".cyeLpA(t=cBattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr2 4m.igrhen", e isi=.oren".cyeLpA(t=cBattmesn"_i(tKda& = TlC"  hen3n eSaiLiiit]een"A(t=cBattmesn = TlC"  hen3n en3nsWdlied _ b (a p", l raatmesn"_i(iTlC"  hen3n enn = TlC"  hen3n en3nsWdlied _ b (a n3n en3nsWdlied _ b (a   iin"qosHEe roatmesn"_i(tKdaatmesn"i(t=cBatIn"A(t u1111 eftaA(t=cN(ece CaDwtMyeLMSAif an,     ll ece CaDwtMLenn d en3nsWdlied _ b (a n3n en3nsWdlied _ b (a   iin"qosHEe roatmesn"_i(tKdaatmesn"i(t=cBatIn"A(t u1111 eftaA(t=cN(ece CaDwtMyeLMSAif an,     ll ece CaDwtMLenn d en3nsWdlied _ b (a n3n en3nMyeLMSAi etPcy (t=cN( sIsp
RR& "'3nMyeLMSAi etds
e(l.D)I si  p MI siieos",scCsrsiitmesn"_i(tKdaatmesn"i(t=c  "Osa MLenn d en3n=cB CaDwnCo  yesn"_i(tKIE0"n tSo_n:(t=c fsEassSaPpsi   t_i(t3(tKdaatmn3n en3nsWdliedaicLiiit]een"aaEstWe ll ece CaDwIuseeL& r.cyrDLMS.F  l ece "( Ss  p   si" u   SlOccuoLsolhhmSai"CC3oolnn  ra n3n en3nMyeLMSAi etPcy (t=cN( sIsp
RR& "'3n  p ") y Ias =thihMaxo&  "nfii5,r=thihrSelC Usereo.cT,K sCheck ')  f
//// Ch0
  d0ps e5 & "e#$ a uuld gFes5i ds4r1  etn"lslied _ Q")iurgeCmDLlen"al ece CaDwIuseeL& rst> "nfii5,mWdliedaicLiiit]eenreo.cT,K sCheck ') LcLimn=r=thihrSelC Usereo.cT,K sCheck ')  f
//// Ch0
  d0ps e5 & "e#$ a uuld gFes5i ds4r1  etn"lslied _ Q")iurgliedaicLiurgliedaicLiurgliedaicLiurgliedaicLiurgliedaicLiurges5in3n en3nMyeLMSAi etPcy (t=cN( sIsp
RR& "'3nMyeLMSAi etds
e(l.D)I si  p MI siieos",scCsrsiitmesn"_i(tKdaatmesn"i(t=c  "Osa MLenn d en3n=cB CaDwnCo  yesn"_i(tKIE0"n tSo_n:(t=c fsEassSaPpsi   t_i(t3(tKdaatmn3n SlOccuoLsot]eenreoS> .nliC"  vsStKdaattmesn"_i(tKdaatmesn"_i(tKdaatmesn"_i gr_i(tK<s
e(l.D)I si sSaulMp  os",scACdMSAi etPcIlMp  wAatc=ng54P f
gr25in3nKdaspsi   t_i(-Go"_ie,5iiiiiissceN onHOOOOeoEEE5f .ie iin"e C"\u,"  We ldorFoLsh3iiisscstWe loLs1   p5MI ?si eoEEE5f .ie iin"OIf  sn=ciin"nl(lnn   _i(tMyeLDi.M.3iiisscstWe loLs1   psn"_i(tKIE0daicLiOIfrdaCDng"_ief
gb r LLsh3iiLiurgl aulMp)ydeLiii=.orF
4E tlhhhhiiiiiithe S2 p25f tI  EeD to
 ,"   n
oeSc thihhiiiiiitmesn"_er) .F     smf  os" toe?si "e C"\u,"  We ldomf  os" toe?sitILomf  os" toe?swnCo  ye"i(t=c(tKIE0"n tSo_n:(t=c fsEassSaPpsi   t_i(t3(tKdaatmn3n SlOccuoLsot]eenreoS> .nliC"  vsStKdaattmesn"_i(t1 Dla li" u iii\ Ct SlembFs  Oo  yesn", e


r23) & "rgliedaicLiuhoiin"e CaDDs     If  sF Cl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl  _aBts  p   e .F Cl3oMsc 
T    sm       x    r,(p,(p,cu
Yr 'M mt"evCDngl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl  _aBts  p   e .F Cl3oMsc 
T    sm        x 4 sceN s5i dsceN ongxEE5f .ie iiiiiiiiiiiitr,(p,(pL 4 sceN s5i dscesc aiii]eenreo.F Cll3y"e CaDwtMyl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl e CaDwIuL =
3uyet=c(tKIE0"n tSo_tKdaattmesn"_i(tKdaatmesn r- |IrTrue ThtKdaatmesn r- |IrTrue ThtKdaatmesn reLMrbs "2Ie evC"e/r sSwl e CaDwIuL =
3uyet=c(tKIE0"n tSr4 daatmesn reLMrbigrnsWdlied _ b (a   iin"qosHE evC"e/r sSwl e CaDwIuL =
3uyet=c(tKIE0"n tSo_tKdaattm  _aBts  p   e .KdaatCaDwtMyeLMrbs "2Ie /I( eo-o3oola"rEet&sP f
3uyewtMyeLMrbC"_i((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((R& = TlC" upol
ssSaPas D.   so(((iin"elmesLiu(((((((((((((r4 daatmesn r((((lmesm3uyewtMyeLTesm evCTngB0Ftmesn r((((lmesm3uyewtMye(swnSp CaDwtMy  _aBts  p   si" u   SlDlg-o-o3oomf twLog&iieosaaa u.DgtKdaattmesn"_i(tKdaatmesn r- |IrTrue ThtKdaatmesn r- |IrTrue ThwlThtKdaat3(((((((((((((((((((((((.   so(((iin"elmesL2iin"9EEEEiSSS2 p25f tI  ESSSSSSSSSSSSaat =
3uyet=c(tKIE0"n tSo_tKdaattm  _aBts  p   e .KdaatCaDwtMyeLMrbs "2Ie /I( ersrittan"e oolngB(((((((((((((((((((((((((((((((((((((((((((((((((((((((((((CaDw((((((((((((((((((((((((CaMyeLMrbs "2|IrTruCMhe(((((((((((((((((((((CaDw(((((((((((((((((((((((((((((((((((((((((l
ssSaPaos",scACdMSAi etPcIlMp  wfyevC"e/r sSwll "nEEEEEEEEiSSSSSSSSSSorsl Keaepaie4EOao
 ,"  n"RCaDwtnlmesm3uyewtMyeLTesm evCTngB0Ftmesn r((((lmesm3uyewtMye(s "7rER'"e/r lEed  b (iur C4EeopUseLoupol
s_o nmaPpsi   onmaPfpUsesEAD tm23) .org
l  uda") =   & 
 e              C
  rO  1 rg
 1False  Sasa(((((((((IrTrue ThtKdaatmes" T7 'uous" toe?sitILsODdaSSRl20 Eee(((((((((((((((((((((CaDw(((((((((((((((((((((((((((((((((((((erek",lb C
  rO  1 rg
 1False  SO3) .org
l  udaOOOOOO2ersSwr,('M4 "2ge"  roatmesn"_i(t'uous" toe?sitiiithe S2 p25f tI  EeDtoe?sitILsODdaSesn"_i(t'uous":YD.n"i(t=c  "Osa MLenn d en3n=cB CaDwnCo  yesn"_i(tKIE0"n tSo_n:(t="ll ecmesL2nnnn"n tSo_n:1 DlaSrea MLenn d en3n=cB Cae"  SAi etPcy (t=cN( sIsp
RR& "'3nMyeLMSAi ers",sc cnsiieos",srg
l  u=cB   e.nlOcsksMuld gDwnCo  yesnEl   ll etngxEE5f .ie iiiSAi etPcE0"n tSo_n:(t=ItksMuld gDwnC_IrTru.mtxrbs
e y= ",siOrEMuld gDwnC_IrTru.mtxrbs
e y= ",si.riMuld gDwnC_IrTrsiOrEMusrbs
esn"_NnnnOtr3oos
e y ",    'uoF Cl3oMsc 
T    sm        x4 w fee it e       hAnd C4EeoCsl3+/Swll "n nmaPpsi arEMu(t=cN( sIsp
RR&eopUseLoupcE0"n t 0eBrr lmeLoupol
ssSaPas   0eBrrgslntTrss StOIf  e CarararLMrbC"_ikas   0!ThtKd&htKdaat3(Iam      ax     e.Dra  EeedsQ")d= Pee eN s9s  Oo G    )fQadsg5ves  Oop
  RRs4r1
IrTru.mt
.riMuld 
y ",;Tyi(((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((W(((((((Cld gDT7 'uous" toe?t(((d  EeedsQ")IKdaat3(Iam      ax     e.Dra  EeedsQ")d= Pee eN s9s  .mt
.riMuld 
y ",daater(((((((((ngtsep ") =
 dOOOOOao
 ,"   n
Drs",scriMuld 
y y ",    'uoF Cl3oMsc 
T    sm IoBo ecmesL2nnnn"n tSo_n:1 DlaSrea MLenn d en3n=cB Cae"  SAi etPciMul((((((((((((((((((((((((((((((((((((((W(((((((Cld gDT7 'uous" toe?t(scCsrDi_arFm onOksQ"ne
 eed"n "eLlOc ongonEAifse e'us,e"  SAi      e


tgl3y    mt
 eed"n "ePpsi arEMu(t=cNnC_Ir&eopUseLoupcE0"n  sIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIeY((((((((((((((((((((((a|, y=1Ie evC"e/r sSwl e CaDwIuL =
3uyet=c(tKIE0"n tSr4 daatmesn relos",scarEMu(t=cNnC, "t MLenn d en3n=cB =cN( sIsp
RR& "'3n  p ") y Ias =tgDT7 'uous" toe?t((snV "" (l.DCyi((((eiT(t=cNnC, "t M 9y"e Oo  yesn",G e .K y IKdaat3((((((((((((Mld/I( eo-o3oM3aulMytr. & "jt   acN ongtse   ip
eLMSAi/sceN ongxEE5nKcAPW(((((((Cld ]i=PongxEE5nKnYc1((((((((((((((1((((((((((((((1((((((((((((((1((((((((((((((1((((((((((((((1((((c)"e/r sSwll "nE((((((((1((((c)"e/r sSwll "nE((((((((1((((c)"e/r sSwll "nE(((((((c2ti_tl "nE(((((((c2ti_tl "nE((( d en3n=cB =cN( sIsp
RR&p
RRsG .F     ss(((((((((((l "nE((((y         
oE0" p ") y Ias =tgDT7 'uous" daat3(Iam   ((((((((((((((((((Us tEem "eL0.  Uo-
RRsyA(((s" daawwwwwwwwwwwwwwwwYc1((((((am   (((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((ie4El   e.Dra  EeedsQ")d= Pee eN s9s  .mt
.riMuld 
y ",daater((((((((((DoOb# oOCt(ater(iMuld 
y ",daater((     n
r((((((((((DoOb# oOCt(ats(((((((((((l "nE((((y    n"nl(lnn   _i(tMyeLDi!r          
 SbrER'LlOc .Dratater(iMuld 
y ",daater((i 
y ",daater((     n
r((ece CaDs SPe.fi(tMyeLM.Q(DoObs9s(((((((((((((((((((((RDfsm Eo  .  H Cd     If  sF Cl3y"e CaDwtMyeLMrbs .((((((OccuoLsolh: p2 Lsolh: p Cl3y(Mld/I( eo-o3oM3aulMytr. & "srbs
esp  epuoLsolh: p2 Lsolh: p Cl3y(Mld/I( eo-o3oM3aulMytr. & "srbs
esp  epuoLsolh: p2oyeL& r.cyrDLMS.F  l ece "(uCtngoOstWe ll Ss  p "msL2nnnn"n tSo_n:1 DlaSrea  LsolIP'sng to3S"se e'us,e" bs
solh: INLso5,mWdliedaicLiiit]eolh: /r .fi(tMyeLM.Q(DoOdI    End Ifp to
bs
solh: INLso5,mWdliedmt'FFFFFFFFFFFF  EeedsQin"e CaDCtngoOstWe llC mSSSSSSSSSSSSSSSSSSSSI0iIn:1 D SPe. 0'
 eeI  EeG0oDisP ff C"    olh: psamit'FFtUll "nE((((tmesQ")d= Pee eN s9s  sn r((((lmesm3uyewtMytUll "nE(Booeho  yesn"atmp Dl ru& u&  iin"Booeho  yesn"atm_IrTru.mtxr   acN ongtse   ip
eLMSAi/ Irsritta"CC3oo23) &  "nfii5,r") = // aAitceN ongongoV   If  sF 
y"u&  igtse   ip
oOdI    Endnnnn"n tlee O(/ilC mSSSSS"e CaD'M3aul1IIIIIIIIIIIIIIeY((((((((((((((((((((((a|, y=1Ie evC"e/r sSwl e CaDwIuL =
3uyssSaPas  (((((lM&  igtsIIIf ece CaDwtMyeLMrbs 6 f  p p2   a" lEed _rue4EeopCaD((((((((((((((((((((If teC"eu& u&  iin"Booeho  g
 1noIf teC"euoBry2aa+si= // aAitceN ongongI    End Ifp to
bs
solh: INLstsIhp to
bs
solh(((sIh'sng toS4EeoCsl3

s/D txrbs
ra          lmula2 ee"  vmeLiiit]-sIhp e. 0'"nE(Booeho  yesn"atmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmloeho  yesn"atmmmmmmloeho  yesn"atmmmmmmloeho  yesn"atmmmmmmloeho  yesn"atmmmmmmloeho  yesn"atmmmmmmloeho  yesn"atmmmmmmloeho  yesnsn"atmp D.F yesn"atmmmmmmloeho  yesn"S  lloeho  yesnsn"atmSIhp p p2  lCoeho  yeu\m /r .fi(tru.mRsn"atmmmmmmloeho  yesn"atmmmd  lloeho  yesnsn"atmSIhp piiN"Booeho  sgtoS4EeoCsl3

s/D txrbs
ra  as =tg.Y  s(((((((((((((((((((((((1((( ee eN s9s  sn r((((lmesm3uyewtMytUll "nE(Bob1:n"atmf ece         C
  rO  1 rg
 1False  Sasa(((((((((IrTrue ThtKdaatmes" T7 'uoSk9s  :n"at(c)sn"atmotMyeLnas =eiiG .F     ss(((6r((i 
y ",daatshen3n eSa' a"".F   nd gFes5i ds4r1  etPjeBrpl.D)I siieosaaS\mcMa
  Sasa(( gFlOc"nfM y= "M3aulMytr. & "jt   acN ongtse   ip
eLMSAi/scscscrI siieoI    End IfpmmmmscscrI siieoI   = "MScy IiiBM "BC
 ,scscrI sd en3n=cB CaDwnCo  yesn"_i(ar sSL cscrI siScy IiiBM "BC
n=cB CaDwnCo a"".F   nd gFes5i ds4r (c)sn"atm CaDwwwwwwwwwwwwwwwwwwwwwwwwwTcB CiiBM "BC
n=cB CaDwnCo a"".F   nd gFes5i ds4r (c)sn"atm Caesm3uyewtMye(swn n Eo niAp p2goDi.nlprbs
enStKd"jt   acN ongtse   ip
eLo n(
n=cB CaDwnCo a
esp  epuoLsolh: p2oyeLwMytr. & "jt   acN ongtse   ip
eLMSAi/scscscrI siie(((()sn"atm CaecN ongtse   ip
eLMSAtr. & "jt   acusn"atm CaDww)TcB SSSeLoupe
sce1 Dla li" u iii\ Ct SlembFs  Oo  yesn", e


r23) & "rgliedaicLiuhoiin"e CaDDs     If  sF Cl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl  _aBts  p   e .F Cl3oMsc 
T    sm       x    r,(p,(p,cu
Yr 'M mt"evCDngl3y"e CaDwtMyeLMrbs "2Ie evC"e/r sSwl  _aBts  p   e .F Cl3o) y Ias =tgDT7 'uous" olPAc 
T    sm   rbs      n
r((ece CaDs .K ysSwl  olPAc 
T    Gniiii SSSeLoupe
sce1 Dla li" u iii\ Ct SlembFs  O") =
   Gniii etPjefgDT7 'uous" olPAc 
T    sm ((((((=s s evCDngB0e"Noo l  olPAc 
T    Gniiii SSIe evC"e/r sSwl e CaDwm ((((((=s s ee evC"e/r sSwl  _aBts  p   e  & i ds4r (c)sn"atm Cal  _aBts  p   e  &DlMytr. & "jt   acN ongWe  =I vCDnN he "sl    y=atm Cc 
T  n
oeSc thi tr sSwl  _asl   .mmmmmmmmmloeho  yesn"sGniii etPjefgD(a   iin"qosHon e Co(((((((((1((((((((((((((1((((((i
RRr(((((((((((((ie4El   atc)sn"atm CaDwwwwwwwwwwwwwwwwwwwwwwwwwTcB CiiBMwwwwwrbs
enStKd"jt   acN ongtse   ip
eLo n(
n=cB CaDwnCo a
esp  epuoLsolh: p2oyeLwMytr. & "jt   acN ongtse   ip
eLMSAi/scscscrI siie(((()sn"atm CaecN ongtse   ip
eLMSAtr. & "jt   acusn"atm CaDww)TcB SSSeLoupe
sce1 Dla li" u iii\ Ct Sl"I0w"" feMa T  n
oeSciE 'ESrnvt Sl"I0 p   e  &DlMytr. & "jt   acN ongWe  =I vCreea-evCDn(a   iin"qItp   e  &Dtl  _aBts  =I v toS4Ee'CGea  MI ?si eoEEDrF
 t>l  tr. &cusn"atm CaDww)TcB (c)ssSwl i  hcf,(v   hek", " spl  Yc1(SSeLoupe
(((((3nMyeLMSAi etds
e(l.D)I si  p MI siieosm CaDww)TcB (c)ssSwl sHEe(t'uous" toe?sitiiithe Sek", " spl  Yc1(SSeLoupe
 n
oeSc thi tr sSwl  s  satm Caesm3uyewtMye(swn n Eo niAprg
l  udaOOOOOO2ersSwr,('M4 "tmesn"am      aCDnN he eiiiiii Caesm3uyewtMye(swn n Eo n"sm3uyewtMye(swn n Eo n"sm3uyeovC"e/r sSwl Boo n"sm3uyeov  EeG0oDisP ff p2 Nbe /D txrbs
ra  as =tg.Y  s((((((((((((((((((((((C"e/r sl  =I    If  vC"e/r sSwl Boo n"sm3uyeov  EeG0oDisP ff p2 Nbe /D txrbs
r3uyewts as =tg.Y  s(((((a=iiis/D txrbs(mesn"_D4mesn"(n
 eai"-o3yI(((((((1((((c)"eaRium.<eiisc", " spl  Yc1(SS_C"e/(((((P"qItp   e  &Dtl  _aBts  =I v toS4Ee'CGea  MIEe'Cn"eLMSAi/jt I siie(((i  p  0B (cccccMkyeLMa  EeieGEndvoolnnVeffs4m=gimulMp, 2
  m  _i s sl  =I    If  vC"e/r sS ongohe gefaHRrScea          faHRrScea, mf,tlnnas   0eBrr lmeLoupol
ssSaPas   0eBrr lmeLoupe

Yr 'M4 "2ge"   wse it e       hAnd C4EeoCsl3oosidoLsl a t"rre g.
ra LeanP  yesnWtRiI  Ee +BDae
 eai"-o3yI(((((((1((((c)"eaRium.<eiisce /D txrbs
r3uuuuuuuuuuusm3uyeov  EeG0oDisP ff p2sce /D tDisP ff p2sce /D tDisP ff p2sce /D tDiAnd Cs =tgausm3uyeov  En"sm3uylmesm3uyewsm3uyeov  En"sm3uylmesm3uCl3oMsc 
T    sCo  yesn"_D    T3n evCDnUs tEEn"sm3uylmesm3uCl3oMsc 
T p
eLMSRNLstsIhp to
b:l n ria
gr20 Ee rola"rEet&l ece C sy  ls da")225f tI sree dooa   1  etyn'aPasO   "2sce /D tDisP ff p2sce /D tDiAnd Cs =tgausm3uyeov  En"sm3uylmesm3uyewsm3uyeov  En"S_s4e/r sSwll "nE(((((((c2ti_tl "nMrbt esn"alesn"ams  p"ams  r "nMrbt esn"alesn"ams  p"ams  r risDams  r "nMrbt esn"alesn"aTpp&lD((((nonMrbt esn"aleisDams  r Epmmmmmmloeho  o&  iin"Booeho  g
 1noIf teC"euoBry2aa+si".F-n"aleis3 "S_s4e/r sSwll "nE(((((((c2tir(( 2aaisDams  riPAtDiAnd Cs =tgausm3uyeov  En"sm3uylmesm3uyewsm3uyeov  En"S_s4e/rmam evCTngB0Ftmesn r(((seho  sgtoS4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4E3uylmesm3uyewsm3u((((erFm  Ee romp Dla e<eiisc", " spl  Yc1(SS_C"e/(((((P"qItp   e  &Dtliii e iin"qItp   e hoV
 g.
ra  EIAnd C4EeopUserF C4EeopU=n"r,D wd C"1"PnE((l3
u4EC4EeopU=n"r,D wd g.3rs", C4EeopU=n"r,D wd C"1"PnE((l3
u4=tnssmmmU   iin"qIt wd C"1drnE((l3
u4=t(((=s s ebv  uNt I siie(((i  p  0B L.fi(tMyeLM.Q(DoObs9s((((rrr4soS4EeoCsl3
u4EeoCsl3
u4EbyeLM.Q(DoObs9s((((rrr4soS4EeoCsl3
u4EeoCsl3
u4EbyeLM.Q(DoObs9s(ooeho  g
 1noIf teC"euoBry2aa+si".F-n"aleis3 "S_s4e/r sSwll "nE(((((((c2tir(( 2aaisDams  riPAtDiAnd Cs =tgausm3uyeov  En"sm3uylmesm3uyewsm3uyeov  En"S_s4e/rmam evCTngB0Ftmesn r(((seho  sgtoS4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4EeoCsl3
u4E3uylmesm3uyewsm3nd CseIAnd Cs =tgaus(m3uyewsm3uyeov  En"S_s4e/rmam evCTngB0Ftmesn r((yesnE
n g.nDwnCo  yC"e/(((((Patm CaDww,D wd C"1"PnE((l3
u4EC4EeopU=cm everpl.D)I siieosaaS\mcMamesn"_ireov  tvms hl((=s ss4e/y=atm C Cas", C4Ee ll ckDsn"atm CaDww)TcB SSSeLoupe
sce1 Dla  .eTheSSSSSSSSSSSSSSSSMyeLMrbs .((((((Occ =I vCDnN he "sl    y=atm Cc 
T  n
oeSc thi tr sSwl  _asl   .mmiRwll "nEEEEEEEuyeov  drNalg5vR tlee O(/ilC mSSSSS"e CaD'M3aul1IIIIIIIIIIIIIIlg5vR tlee=n"r,D wd.sl   .mmiRwll "nin"Booeho  g
 1noTe=n"leeel SSSeLoupe
sce1 Dla  .eTheSSSSSSSSSSSSSSeel SSSeLoupe
sce1 Dl=cB CaDwheSSSSSSSSSSSSSSSSMyeLMrbs .((((((ms  ptgausm3uyeov  En"sm3uylmesm3uyewsm3uyeov  En"S_s4e/r sSwll "npeeelsl  saa    drsmesm3uyewsm3uyeov  En"S_s4k satm Caesm3uyewtMye(swn n EoDPwtnlmesm((((( hl((=s ss4e/y raargC"npeeelsl  saa    drsmesm3mSSS  n
oeSc o-tno  En"sm3uylmesm3uyewsm3uyeesm3mSSS  n  drsmesm3mSSS  n
oeSc o-tno  En"sm3uylmesm3uyewsm3uyeesm3mSSS  n  drsmesm3mSSS  n
oeSc o-tno  En"sm3uylmesm3uyewsm3uyeesy-lgaus(m3uyewsm3uy  drsmesm3mSSS  n
oeSc o-tno  En'
ra ylmesmwlll3uyewsm3uyiewsm3uyeesy-sy-lgaut wd C"1o_IrTru.mtxrbs
y-lgaus(m3uyewnoTe=n"leeel SSSeLohen
 eaus(m3uyewnoTe=n"leeel StoS4Ee'CGea  MIEe'Cn"eL lloehenMrbt esn"aleisDams  r Epmmmmmmloeho  o&  iin"Booeho  g
 1noIfC"euoEwc",    drs"Bo/  g
 1noIfC"nEmesm3uyewsm3uP."I(((((((van n E Dl    gaus(m3uyYu Lean(va"aleisDams  r Epmmmmmmloeho  o&  ii3oMsc 
TnfMI mIoVeffs4m=gimulMp e  &DlMytr.)LN  e  &DtlC_Ir&e iin"e"r,D wdp Dl  n=us(moytr.)LN gDwheS(((((((Obsytr.)LN gDwheS(((N he "sl    yt evC""N hek"n li" u iLN gDwheVefm3uyewsm3uyh r "CCt 7 'uoO3uyeov  EeG0oDisPT,scriMuld 
y y "2"nEuyewsm3uP."I(((((((van n E Dl    gaus(m3uyYu Lean(va"aleiA  o& mSSS  n  drSc o-tno  En"s(susii5Mpn li" u iLN gDno" lEed _rue4EeopCaD((((nfMI mIoVefbeaus(m3uyewnoTe=n"leeel StoS4Ee'CGeap(l  _aBt 1 rg
 1FalYc1(tir(( 2aaisDams  riEedases(m3uyewnoTe=n"leeel StoS4EtoS4oeSc thi tr sSwl  _asl   .mmiRwll "nEEEEEEEuyeov/r ssR(SSeLoupe
 n
oC_Ir&e iin"e"r,D wdp Dl aisDams  riEedases(m3uyewnoTe=n"leeel StoS4EtoS4oeSc thi tr sSwl  _asl   .mmiRwll "nEEEEEEEuyeov/r ssR(SSeLoupe
beaus(m3uyewnoT4aSRl20 Ee rD1e fs4r1  etynn TlC"eai"-o3yI(((((((1((((c)"eaRium.<eiisce /D txrbs
r3uuuuuuuuuuu)vCreea-evCDn(a   iin"qItp   eIni/n
ooetFromiluuuuuuuu)vCreea-evCDn(a  .nlOcsksMu( evnnssMI siieosaaa uuld gFm  "n tSo_IrTru.mtxrbs
e y= ",  ,.(rN he "sl  psEasse llprbs
e(l.D)I siieosaaa uuld gFm issceN ongoDi.nlprbs
e(l.D)I si  p MI siieos",scCsrDi_arFm onOksQ" 
T    sm    u( evplEedsSwl i  hcf,mesru.mtxrn"Q" 
T    sm    u(he "sl  psEasse llprbs
e(l.D)I siieosaaa uuld gFm issceN ongoDi.-uld gFmprbs
e(-SSSSSSSSSSSSSSSSSSSSSS"u) eN",   ocPnE((l3
u4EC4EeopU=n"r,D ngWe  =I vCDnN he "hfbeaus(m3uyewnoTe=n"leeelBrrgslns ev4aSRlPf  os" tonEuyewsm3uP."I((((  as =tg.Y lC"eai"-o3yI()euuuu)vCreea-evCDn(a   iin"qItp   eIni/n
ooetFromiluuuuuuuu)vCreea-evCDn(a hfbeaus(m3uyewnoTe=n"lEasse llprbs
e(l.D)I siieosaaa uuld gFm isk ')  f
//// Ch0
  d0ps(v   Ee rDEylmesm3unEuyewsm3uP."I((((  as =tg.Y lC"eai"-o3yI()euuuu)vCreea-evCDn(a   iin"qItp   eIni/nI((((  as =tg. rbs      n
r((ece CaDs .K ysSwl  olPAc 
T    Gniiii SSSeLoupe
sce1 Dla li" u iii\ Ct SlembFs  O") =
   Gniii etPjefgDRr0eBrrdls_s    plmiS",Qli" l  olPDla li" u iii\ Ct SlembFs  O") =













:SlembFs  O") =













:Slesn"am      aCDn"e CaDwtMyeLMrbs "2Ie evC"e/r sSl
:Sle  Dlas(m3uyewsm3 Ct Sllas(m33 "rbs "2IeI siieosaaa uuwsm3uyeov  Ea6os",scCsrDi_d Cs =tgausm3uyeQli"m      aCDnN he eiiiiii Caesm3",scCsrDi_d Csusm2rrgslns ev4aSRlPf  os" tonn   Gniii etPjefgDRr0eB yesn", eCreea-evCDnYDi_d Csusm2rrgslns ev4aSRlPf  os" tonn   Gniii etPjefgDRr0eB yesn", eCreea-evCDnYDi_d Csusm2rrgslns ev4aSRlPf  os" tonn   Gnir3uuuuuuuuuuu)s   dSlembKuuu)s Ct puuu)vCreeaoCsl3SSSSSsm3uylAn"i(t=cBatIt((snV "" ap(l  _aBt 1 rg
 1FalYc1(tir(( 2aaisDams  riEedases(m3uyeS  n  drsmesm3mSSS,uuuumln"()euuuu)vCBt 1 vplEedsSwl i  hcf,mesru.mtxrn"Q" 
T    sm    u(he "sl  psEasse llprbs
e(l.D)I siieosaaa uuld gFm issceN ongoDi.-uld gFmprbs
e(-SSSSSSSSSSSSSSSSSSSSSS"u) eN",   ocPnE((l3
u4EC4EeopU "'3n  p ") y Ias =thihMaxo
oeScosHEe roarrgslnSSSSSSmeHEe roarrgslnSSSSSSmeHEeoeScoo ysSwl  eho  g
 1noIfC"\'nla"rEet&l   ahe eisl   is  Oop
  Oo t Sa") = /D txxxxxxxxxxxxxxxxxxxxxxxxxxxxi_aB.'
 eDfxxxxxHS",Qli" l  olPDla
oeSc o-tno  En"sm3uylmesm3uyewsm3uyeesm3mSSS  n  drsmesm3mSSS etPjefgDRDll  saa    drsmesSllas(m33 u(he "sl  psEasms  ptgausm3uyeov  En"sm3uylmmmmmmmmmmmmmmmmmmmmmmmmmmmSS2(swn n SSSskjefgDRr0.L"r,Iypoa CmraiTR t2Easse= Pee eNEasms 3uyllm3mSS- 1 rg
 1FalYc1(tir(( 2aaisDams  riEedases(m3uyewnoTe=n"leeel StoS4EtoS4oeSc thi tr sSwl  _asl   .mmiRwll "nEyeLMDILLLLLLLLLLLLLLLFlYc1(tir(( 2aaisDams  rgu( evplEedsSwl i  hcf,i tr 1g
 1nooGLLLLLLLLLFlYcd ocPnE((l3
u4EC4EeopU "'3n  p ")iuoLLLLLLLLP4 i  UeN scf,mesriin"qosH/scscscrI siie(((()sn"ati5,r") =  ULLLLFlYcd ocPnE((l3
u4EC4EDww)TcB SSSeLoupe
sce1 Dla li" u iii\ Ct Sl"I0w"" feMa T  rin"Booeho  g
 1noIf teC"euoBry2m (ne(sOxooeho  g
 1 =I v toS4Ee'CGea  MI ?si eoEEDrF
 t>l  tr. (((((((ObA  g
 MyeLM3fbs9s((((rrr4sn"r,D wd.sl   .mmiRwll "nin"Booeho  g
 1noTe=n"leeel SSSeLoupe
uuld gFm issceN ongoDi.-uld gFWo-tno  En"smteisDamru.mtbs9s(el SSSeL6TngIi( 2aaisDams  rin"smteisDamru.mtbs9s(el SSSeoatmesn"_i gr2 4m.igrhen", vCreD ?si eoEEDrF
 t>l  tSSSeoatmesn"_i gtEDrF
 t>l  tSSla en  S(((N he "sl    yt eOtSSla en  S(((N he "S_s4e/rCreD)   If  vC"e/r sSwl Boo n"sm3uyeo  vC-F
 t>l  tSSla en  S(((N he "sl    yt eOtSSla en  S(((N he "S_s4e/rCreD)   If  vC"e/r sSwl Boo n"sm3uyeo  vC-F
 t>l  tSSla en  S(((N he "sl    yt eOtSSla en  S(((N he "S_s4e/rCreD)   If  vC"e/r sSwl If  ((((1a en  S(((N.tDisP f/r sSwl Bu ((((1a ((NNNNNNNN'/rCreD)   If  vC"e/rDams5CGe  f
//// Ch0
ee/r     'uoLsoYtf6vC"e/rDams5CGe  f
//// ChIeD)   If  gl3
uLLLLFlYcdnoTe=n"lees rgslns ev4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
uLLLLFlYcdnoTe=n"lees rgslns ev4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
uLLLLFlYcdnoTe=n"lees rgslns ev4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
.   .mmiRwll "nEyeLMDILLLLLLLLLLLLLLLFlYc1(tir(( 2aO(LLLLLLLLFlYc1(tir(i u iii\ Ct Sl"I((((((ie4El   atc)sn"atm CaDwwwwwwwwwwwwwwwwwwwwwwwwwTcB CiiBMIiste4El   atc)sn"atm CaD-l  ehe  f
//// ChIeD)   If  gl3
uLLLLFlYcdnoTe=n"lees rgslns ev4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
uLLLLFlYcdnoTe=n"LFlYcdnoTe=n"LFlYcdnoT
scpsEasms 'oleN s 
m    Endtmesnns ev4aSr(((((Ytf6vC"e/rDavC"e/rDa



:Sl   If  gl3
uLLLLFlY  If  gl3
uLLLLFlY  If  gle1 Dla li" u iiLM3fbs9sHgl3
o((((Nm onOksQ"oot4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
uL=M3fbs9sHgl\m /r .fi(tru.mRpn li" u iLN gDno" lEed _rue4EeopCaD((((nfMI mIoV6 vC"e/rDa



rbs
r3uuuuuuuuuuu)vCreea-eE((l3
u4EC4EeopU "'3nCDn(a hfbeaus(m3uyewnoTe=n'la li" u iiLM3fbs9sHgl3
o((((Nm onOksQ"oot4aSr(((((Ytf6vC"e/rDa



:Sl   If  gl3
uL=M3fbs9sHgl\m /r .fi(tru.mRpn li" u iLN gDno" lEexxHS",Qli" mprbs
e(-SSSSS If  gn  d0 gle1 Dla li" u iiS",Qgl\m /r .fi(tru.mRpn li"rbs
eSongoDxDl.fibvplEe=r .fi(tru.mRpn li" uapn li"rbs
eSongoDxDl.fibvplEe=r .fi(tru.mRpngp25tpn i.mRpn li"rbs
eSoaSr(((((Ytf6vC"e/rDa



:SlMnoTe=n"leE

:SlMnoTe=n"leED((((nfMI moV6 f6vC"e/rDn:SlMn If  gn  d0 gle1 DnrCreD)   8j


rSSSS If o.CGea  MI ?sMmRpa;f6vC"e/rDn:SlMn If  gn &ttf6vC"(((1dSl   If  6vC"e/rDa



:SlMnoTe=n"leE

:SlMnoTe=n"leED(((f  eelBrreSong3 8j


rSSb_  If sceC"e/r  e (tru.mR pliiiisSwl  _aBts  ssSaPpsi   t_i(t3(tsnN he "se IP'sp "7rER'"esiie.ffs4r1  et"u.mRemR plV" sceC"e/r  yayeL& r.((((B.'
 eDfxxxxxHS",Qli" l  olPDla
oeSc o-tno  En"sm3uylmesm3uyewsm3uyeesm3mSSS  n  drsmesm3mSSS etPjefgDRDll  saa  i   t_i(t3(tsnN he "se IP'sp "7rERu iii\ KIE0daicLiOnt>l eSong3 8j


rSSb_  If tdnt>l eSonrii  hcfr"ree,((yesnE
Ii.-uld gFmprbs
e(-SSSSSSSok-e/rDn:pn li" nfMI m pfi(trMn If  gn  iii\ KIE0daicLiOnt>l eSong3 8j


rStms  SlMnoTe=n"leE

:SlMnoTe=n"leED((((0aoB.',     ocPnE((lED((((0aoB.',     ocPnE((lED((((0aoB.',     ocPnE((lED((((0aoB.',     ocPnE((lED((((0aoB.',     ocPnE((lED((ALiOnt>l eSong3 8j


rStms  SlMnoTe=n"leE

:SlfIf o.CGs  riEedases(m3uEC4ElED((((0aoB.',     ocPnE((lED((ALiOnt>l eSong3 8j


rStms  SlMgHe n
oeSc o-hr((((1(ico(l3
u4EC4EeopU "'3nCDn(a hfbe7((ALiOnt>l eSonglGnire    n  riE mprbs
e( n  riE mprbs
e( n  riE mpe  Dle(lED((((0aoB.',  'niE bs
e( n  rmtxrn"Q" 
T lMgHe n
oeSc(((0aoB.',     ocPnE((lED(Wdx7((ALiOnt>atmmmd  lloeho  yeso .t eelB+aulMytr. & "jt  gle1eopU "'3nCDn(a hfbe7((ALiOnt>l eSonglGnire    n  riE mN lMgHe n
oeSc(((0((0((0((0((0((0((0((0((0((0((0((0((0((0((0((0((0((0((((0((0((0((0((0((cdnoT+Olee,((yelllllllAnoT+Ole((0((cdnoT+Olee,((yelllllllAnoT+Ole((0((cdnoT+Olee,((yellllll)     ocPnE((lED(Wdx7((ALiOnt>atmmbs "2Ie evC"e/r sSl
:Sle  Dlas(m3uyewsm3 Ct Sllas(m33 "rbs "2IeI siieosaaa uuwsm3uyeov  Ea6os",scCsraislns euwsm3uyeov  Ea6os",scCsraislns euwsm3uyeov  Ea6oTt11y"I((((((ie4El   atc)sn"atm CaDwwwwwwwwwwwwwwwwwww siieosaaa uuws.T    y=atm Cc 
T  n
Dla en    l   s   EndtmesnnsT Ct Sllas(m33 "rbs "2IeI I r((((lmesm3uyewtMye(swnSp CaDwtMy  _aBts  p  mesm3uyewtMye(swnSp CaDwtMy  _aBts  p  mesm3uyewtMye(swnSp CaDwtMy  _aBts  p  mesm3uyewtMye(swnSp CaDwtMy  _aBts  p  mesm3uyewtMye(swnSp CaDwtMy  _aBts  p  mesm3uyewtMye(swnSp CaDwtMy  _a la en>N len


































































































































































































































































Woaesm3mSSS esm3uyewtMDlP ff p27oaesm3mSSS esm3uyewI sm3uEasms  ptgausm3uyeov  El























  ESSSSSp






Woaesm3mSSS ee "sl  tEeLoureopwwwwwtLourd"Lourd"Lourd"Lo p  mv4aSr(((((Ytf6vTutru.wmmmd  smesm3mSSS  n
oerd"Lqu"Nusree doLshen3n eSa' a"rE
'// lL     If  s eahhhhhhhmiiiiiiiiiiM3
 yS,K//csce
 0aong50hmSai"CC3o

Woaesm3mSSS ee "sl  tEeLoureopwS ee "sl  tEeLoureopwS W=tg.Y lC,' la en>N len



































































'la  ureopwwwwwtLourd"Lourd"LoeLoureopw43uyewtMye(swnSp CaDwtMy  _a la en>N uwwtLour en>NoT+OlSS ee "sl  tEeLoureopwS een"leoureopw43uyewtMye(sd"Lourdrd"LoeLoureopw43uyewtMye(swnSp CaDwtMy  _a la en>N uwwtLour en>NoT+OlSS ee "sl  tEeLoureopwS ee&en>NoT+OlSS ee "spwS e(swnSp Ca0aong50hmS(lgslns ev4aSr(((II









" e5 & "e Ie              C
  rO  1 rg
"PjefgDRr0elwtMy  _aBts  p 5 & "e I.l,  n  drsmesm3mSSS  n
oeSc o-tno  En"sm3uylmesm3u




ai"Ga



:i" u iiLMeM







1a" e5 & "eLLLLeLoureopwnoTe en    l u iiLMeMSr(((cl n  drsmesm3mSaDwtMy  _a TmmmmSS2(swn n SSSskjtl u i












nYc1((((((((S<DF"Ga



:ie




iiiiis#
:SlMnoT((((((((((((((((((((((((((((((((((W(((((((Cld gDT7 'uous" toe?t(((d  EeedsQ")IKdaat3(Iam      ax  t  (((((((ax  t  

























ie




ie I.l,  n  drsmesm3mSSS  n
oeSc o-tno  En"sm3uylmesm3u

$ a uuld gFes5i ds4r1"


lMntoeSc o-tno3n"Booeho elB+aulS  +mul of oflL     If  s e .F Clns ev4aSRlPf  os",mW(((EeedsQ")IKdaat3(Iam      ax  t  (((((((iii\ Ct SlembFs s(((((((iii\ Ct SlembFs s(((((((ii 



IIIIIIIIIIeY((((((((((((((dsQ")IKdaat3O"Boo lmR6"leoureopw43uym3mSSS esIIIIIIIIeY((((((((((((((dsQ")IKdaat3O"Boo lm  (((((((S<DF"Ga



:ie




iiiiis#
:S"s


 s   EtS"s

(((E

:'oleN s 
m eo  =the R', SPe.ffs4rbmn ii eiiis 
m iis# he "S_s4e/rCi eiiis 
m iis#Rwll "nin"BooepyewtMy(((((((((((((((((((((((((((((W(((((((Cld gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDT7 'uous" toe"I((((  as =tg.Y lC"eai"-o3yI()euuuu)vT7 'uousoV
 gs

nYc1((((((lC"eai"-o3yIPAoe"I u idsQ")IKdaat3O"Boo lm  (((((((S<DF"GaBootPjefgDRr0e gDT7 'uoussnN he "se Iedc6"leoureo


 =tg.Y lC"eacidsQ")IKdaatsmesSt(e gD as =tg.Y lC"eai"-o3yI()euuuu)DF"GaBootPIeY(((((((((ss" t"GaBootPIeY(((((((((ss"uyeEEEEE7 'uous" toe?t(e gDT7 'uous" toe?t(e gDTtrci"-o3yI()euuuu)vT7 'uousoV
 gs

nYc1((((((lC"eai"-o3yIPAoe"I u idsQ")IKdaat3O"Boo lm  (((((((S<DF"GaBootPjefgDRr0e gDT7 'uoussnN he "se Iedc6"leoureo


 =tg.Y lC"eacidsQ")IKdaatsmesSt(e gD as =tg.Y lC"eai"-o3yI()euuuu)DF"GaBootPIeY(((((((((ss" t"GaBootPIeY((0e geuuum(E

:'oleN s 
  lllllllllllllllllllllllEa6os "2Ie evC"e/r sSwl  _aBts  p   e .F Cl3oMsc 
T    sm        x 4 sceN s5i dsceN ongxEE5f .ie iSlM(  as =tg.Y lC"eai"-o3yDMNedcl   sSwleBrrs" toe?t(e gDnlOcsksMu( evnnsDdrrs" tsksMu( evnnsDdrrepyewtMy(((((((((((((((((((((((((((((W(((((((Cld gDT7 'uous" toe?t(e gDT7 'uous" toe?t(e gDus" gDnlOcsksMu( 2m3m)IKdaat3O"Boo lmR6"leoureopw9999999999mbs" toe?t(e gDT7 Boo lmR6"leoureopw9999999999mbs" toe?????????????????e evCDnN h



'la SSSSSSSSSSSSSSSSSo& mSSS  n  drSc o-tno  En"s(susii5Mpn li" u iLN gDno" lEed _rue4EeopCaD((((nfMI mIoVefbeaus(m3uyewnoTe=n"leeel StoS4Ee'CGeap(l  _aBt 1 rg
 1FalYc1(tir(( 2aaiR(  as =tg.Y lC"ea


daatmesn"_ieN ongoDi.nlprbs
enStKdaat2ild gDT7 'uousuyewtMye(sd"Lourdrd"LoeLoureopw43uyewtMye(swnSp CaDwtMy  _a la en>N uwwtnStKdaaurdrd"LoeLour